package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
 /*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 */
import java.util.Enumeration;
import com.ibm.dse.base.*;
import com.ibm.dse.automaton.*;
import com.ibm.dse.cs.html.HtmlConstants;
 /**
 * This is a subclass of DSEProcessor that implements behavior specific
 * to the HomeBanking HTML Client. This behaviour include supporting
 * "virtual" processors (that is, processors which can
 * be persisted with enough information so that they can be restored to the state they
 * were in when they were persisted) and
 * changing the default behavior of DSEProcessor so that it executes the
 * state machine to its end. The changed default behavior consists of checking state types
 * before they are processed, and suspending the processor for the <i>page</i>, and <i>subflow</i>
 * states. In addition, this class adds support for a validator class. The validator
 * validates the business data required for the
 * workflow that the processor is implementing.
 *
 * @copyright(c) Copyright IBM Corporation 2000, 2003
 */
public class DSEHtmlProcessor extends com.ibm.dse.automaton.DSEProcessor implements InterStateFormatEnabledProcessor
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	// trace ID for the HTML client module
	public static final String COMPID = "#HTML" ;
	public static final String HTML = "html" ;	
 	// these fields are used to define consequentiality for states and
	// transitions. They have been placed here for common access by both.
	public static final String CONSEQ_UNDEFINED = "conseq_undef" ;
	public static final String CONSEQ_TRUE = "conseq_true" ;
	public static final String CONSEQ_FALSE = "conseq_false" ;
	public static final String CONSEQ_ERROR_PAGE = "conseqErrorPage";
	public static final String XVALIDATION = "xValidation";
	public static final String PROCESSTIMEOUT = "processorTimeout";
 	// the next field is used to delegate validation assesment for a transition
	public static final String DELEGATEVALASSESSMENT = "delegateValAssessment";
	
	// processor statuses
	public static final String CONSEQUENTIAL = "_consequential" ;
	public static final String ABORTING = "_aborting" ;
	
 	private java.lang.String parentId = null;
	private ProcessorXValidate xValidator = null;
	private java.lang.String conseqErrorResponse = null;
	// Time-out related to this specific processor
	private java.lang.Long processorTimeout = null;
 	/** Support for application session **/
	protected String parentSessionId = null;
	/** Support for application session **/
	protected String parentAppSessionId = null;
	
	/** <HSBC> **/
	/** Attribute for InterStateFormat support **/
	protected Context prevActionContext = null;
	protected Tag isFormatTag = null;
	/** </HSBC> **/
	
 /**
 * DSEHtmlProcessor constructor.
 */
public DSEHtmlProcessor() {
	super();
}
/**
 * DSEHtmlProcessor constructor.
 * @param aName java.lang.String
 */
public DSEHtmlProcessor(String aName) {
	super(aName);
}
/**
 * DSEHtmlProcessor constructor.
 * @param aName java.lang.String
 * @param aTID java.lang.String
 */
public DSEHtmlProcessor(String aName, String aTID) {
	super(aName, aTID);
}
/**
 * DSEHtmlProcessor constructor.
 * @param mustRegister boolean
 */
public DSEHtmlProcessor(boolean mustRegister) {
	super(mustRegister);
}
/**
 * Aborts the state machine.
 * 
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
public void abort() throws DSEProcessorException
{
	setStatus(ABORTING) ;
	super.abort() ;
}
/** 
 * Adds the DataElements whose names are in elementsToAdd to the transition context keyed collection
 * @param KeyedCollection - The transition context kColl
 * @param String [] - The names of the elements to be added in the kColl
 * @param int - The number of elements to be added
 */
protected void addElementsToTransitionContext(Context  tcContext, String [] elementsToAdd, int size)
{
	KeyedCollection tcContextKColl=tcContext.getKeyedCollection();
	for (int i=0; i<size; i++)
	{
			String dataName=elementsToAdd[i];
			try {				
				DataElement dataElement=this.getContext().getElementAt(dataName);
				//Added for PMR82947 begin,
				DataElement newDataElement=(DataElement)dataElement.clone();
				//Added for PMR82947 end
				// the next line should dynamically add the data element in the transition context kColl
				
				//Comment for PMR82947 begin
				//tcContextKColl.setValueAt(dataName, dataElement.getValue());
				//Comment for PMR82947 end
				
				//Added for PMR82947 begin
				tcContextKColl.setValueAt(dataName, newDataElement.getValue());
				//Added for PMR82947 end
				setTypedDataInfo(tcContext, dataName, "");
			}
			catch (Exception e)
			{
				if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "Unable to add DataElement in the transition context because it has not been found in the processor context: "+dataName) ;
					//Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;									
				}
			}
	}
}
/**
 * This is equivalent to calling changeState(String, false, null).
 *
 * @param newState java.lang.String
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
public void changeState(String newState) throws DSEProcessorException
{
	changeState(newState, false, (TransitionDescriptor) null) ;
}
/**
 * This is equivalent to calling changeState(String, false, TransitionDescriptor),
 * with a transition descriptor object that has its <i>ignoreEntryActions</i> flag
 * set to the boolean argument.
 *
 * @param newState java.lang.String
 * @param ignoreEntryActions boolean
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
public void changeState(String newState, boolean ignoreEntryActions) throws DSEProcessorException
{
	TransitionDescriptor transition = new TransitionDescriptor() ;
	transition.setIgnoreEntryActions(ignoreEntryActions) ;
	changeState(newState, false, transition) ;
}
/**
 * This method is equivalent to calling changeState(String, boolean, TransitionDescriptor).
 * The transition descriptor argument is obtained from the current state of the processor,
 * using the transition name supplied as argument to this method.
 *
 * @param newStateName java.lang.String
 * @param ignoreEntryActions boolean
 * @param transitionName java.lang.String
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
public void changeState(String newStateName, boolean activate, String transitionName) throws DSEProcessorException
{
	TransitionDescriptor transition = null ;
 	try
	{
		transition = (TransitionDescriptor) getCurrentState().getTransitions().get(transitionName) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "DSEHtmlProcessor.changeState: Could not find a transition named (" + transitionName + ") in current state. Processing will continue, but errors may result") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}
	}
 	changeState(newStateName, activate, transition) ;
}
/**
 * Attempts to change the current state being managed by the state
 * machine to the one specified by the state name argument. The following apply:<ul>
 *
 * <li>The <i>ignoreEntryActions</> attribute of the new state is set to correspond
 * to the <i>ignoreEntryActions</i>  attribute of the supplied transition.</li>
 *
 * <li>For HTML states, the <i>inputMapFormat</i> attribute of the supplied transition
 * is copied (if present), into the state.</li>
 *
 * <p>The consequentiality of the state specified by the state name argument and the
 * transition passed in as an argument are used to determine the consequential status of
 * the processor. State and transition consequentiality are attributes that define
 * whether the processor will be placed in a consequential state upon traversal of the
 * state or transition. Consequentiality for a state or transition can take one of three
 * values: undefined, true, or false. See com.ibm.dse.automaton.html.DSEHtmlState
 * for a formal definitions.</p>
 *  
 * <p>If the state specified by the state name argument defines consequentiality (true or
 * false), then its value for consequentiality is used to determine the consequential
 * status of the processor. If it does not but the transition does, the
 * transition's consequentiality is used to determine the processors consequential status.
 * If the processor is consequential and the state or transition has a status of false,
 * the processor is removed from the consequential state. If the processor is not
 * consequential, and the state or transition has a status of true, the processor
 * is placed into a consequential state. As mentioned above, the transitions values are
 * only used if the state has a value of undefined for its consequentiality.</p>
 *
 * <p>If the <i>activate</i> argument is true, the new state is activated upon
 * completion of this method.</p>
 *
 * @param newStateName java.lang.String
 * @param activate boolean
 * @param transition com.ibm.dse.automaton.TransitionDescriptor
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
public void changeState(String newStateName, boolean activate, TransitionDescriptor transition) throws DSEProcessorException
{
	State state = null ;
 	try
	{
		// get tag corresponding to the new state
		Tag aTag = getStateTag(newStateName) ;
 		// get an appropriate State instance
		state = getStateInstance(getCurrentState(), aTag) ;
 		// initialize the instance using the tag
//		state.initializeFrom(aTag) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(),"Exception caught while obtaining a state instance for: "+newStateName) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}		
		throw new DSEProcessorException(DSEException.critical, null, "Exception caught while obtaining a state instance from the transition: "+transition.getName()+" in the processor: "+getName()+", the exception is: " + e.toString()) ;
	}
 	// get transition attributes
	boolean ignoreEntryActions = false;
	String consequentiality = CONSEQ_UNDEFINED;
	Tag inputMapFormatTag = null;
	
	/** <HSBC> **/
	Tag interStateFmtTag = null;
	Context newContext = null;
	Context oldContext = null;
	/** </HSBC> **/
	
	try
	{
		ignoreEntryActions = transition.isIgnoreEntryActions() ;
		consequentiality = ((HtmlTransitionDescriptor) transition).getConsequentiality() ;
		inputMapFormatTag = ((HtmlTransitionDescriptor) transition).getInputFormatTag() ;
		
		/** <HSBC> **/
		/**  
		 * Originally in WSBCC processor, context is the main data model.  All data store in 
		 * and retrieve from it.  However, when a state context is being specified in a state, 
		 * it will act as a temporary buffer between the processor context and the execution 
		 * components (for example: JSP, operation and action).  
		 * Buffer goes away when the execution component done its tasks.
		 * State context is a child context of processor context.  For the duration of a state, 
		 * it will take the role of a processor context.  When a processor changes state, it will 
		 * check if a context is available in the state and map data into it with the specified formatter.  
		 * 
		 * State context from the previous state will then be discarded.  If an operation or an action 
		 * contains a context, this context will be chained to the state context as a child context.  
		 * However, if such context is not available in the operation or action, the state context will 
		 * be assigned to them instead of the processor context.
		 */
		
		// get old context	
		if (getCurrentState() instanceof DSEHtmlState && ((DSEHtmlState) getCurrentState()).getContext() != null) {
			oldContext = ((DSEHtmlState) getCurrentState()).getContext();
		}
			
		// get new context and inter-state format
		if (state instanceof DSEHtmlState) {
			
			// Get the inter state format from transistion
			interStateFmtTag = ((HtmlTransitionDescriptor) transition).getInterStateFormatTag();
			if (interStateFmtTag == null) {
				interStateFmtTag = ((DSEHtmlState) state).getInterStateFormatTag();
			}
						
			newContext = ((DSEHtmlState) state).getContext();
			
			if (interStateFmtTag != null) {
				try {
					
					if (oldContext != null && newContext != null) {
						// If state context is available in the new state, map data to state context
						// Map data from the previous state context to the current state context
						doDataMapping(interStateFmtTag, getContext(), newContext);
						
						// Data is mapped to the state context, the previouc action
						// context is no longer required
						InterStateFormatSupport.clearInfoForFormat(this);
						
					} else if (getPrevActionContext() != null && newContext != null) {
						// Previous does not defined a state context, but the new state does
						// Do mapping from the previous action context to the new state context here
						doDataMapping(interStateFmtTag, getPrevActionContext(), newContext);
						
						// Data is mapped to the state context, the previouc action
						// context is no longer required
						InterStateFormatSupport.clearInfoForFormat(this);
						
					} else {
						// no state context is available at distination.
						// Store format and map data directly to action context later
						InterStateFormatSupport.storeFormatTag(this, interStateFmtTag);
						
						// At this point, there shoud be a previous action context available
						// for format.  If not, use the old context
						InterStateFormatSupport.checkAndAssignActionCotext(this, oldContext);
					}

				} catch (DSEException e) {
					if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error)) {
						Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "DSEHtmlProcessor: Unable to map data into the new state " + getCurrentState().getName()) ;
						Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;									
					}
				}
			} else {
				// No inter state format is available.  
				// There is no reason to keep the prev action context around
				InterStateFormatSupport.clearInfoForFormat(this);
			}
		}
		
		/** </HSBC> **/
		
	}
	catch(ClassCastException e)
	{
		// Do nothing. The transition received is not a HtmlTransition. Deafult values
		// will be used to consequentiality and inputMapFormat.
	}
 	// set this state's owning processor if necessary
	if(state.getProcessor() == null) state.setProcessor(this) ;
 	// indicate whether this state will be ignoring entry actions
	state.setIgnoreEntryActions(ignoreEntryActions) ;
 	// manipulate the consequential status of this processor depending
	// on the consequential status of the new state and the transition.
	setProcessorConsequentialState(state, consequentiality) ;
 	// set the states input format attribute if relevant
	try
	{
		if(inputMapFormatTag != null) ((DSEHtmlState) state).setInputFormatTag(inputMapFormatTag) ;
	}
	catch(ClassCastException ignored) {
		((com.ibm.dse.automaton.ext.DSEOperationState) state).setInputFormatTag(inputMapFormatTag) ;
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "State "+newStateName+" is not an HtmlState") ;
		}		
	}
 	// terminate processors current state, and make it to be the new
	// state just created
	try
	{
		/** <HSBC> **/
		if (oldContext != null) {
			oldContext.unchain();
		}
		/** </HSBC> **/
		
		if(getCurrentState()!=null)
		{
			
		   getCurrentState().terminate() ;
		   //trace processor terminate the state
	        if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		       Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),"Processor:" + this.getName() + " terminate the state:" + getCurrentState().getName());
	        
	    
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "DSEHtmlProcessor: Unable to terminate old state "+getCurrentState().getName()+" while switching to the new state "+newStateName) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;									
		}
	}
 	setCurrentState(state) ;
 	
 	/** <HSBC> **/
	// Chain new state context to Processor context
 	if (newContext != null) {
		try {
			
			newContext.chainTo(getProcessorContext());
			
		} catch (DSEInvalidRequestException e) {
			throw new DSEProcessorException(e.getSeverity(), e.getCode(), e.getMessage());
		}
	}
	/** </HSBC> **/
 	
 	// If the activating state is a FINAL state, the predefined behaviour is to return control from 
	// automaton and dont activate the state. Set the ignoreReturnType to true in order to execute 
	// the Entry Actions.
	
	if (getCurrentState().getType() != null) 
	{
		setCurrentStateType(getCurrentState().getType());
 		if (getCurrentState().getType().equals("final")) 
		{
			( (DSEHtmlState) getCurrentState() ).setIgnoreReturnType(true);
		}
	}
 
	// clean the processor queue if necessary
	if(cleanQueueOnSwitch)
	{
		cleanEventQueue() ;
	}
 	// optionally activate the state
	if(activate)
	{
		try
		{
			//trace processor enter the state
	        if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		        Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),"Processor:" + this.getName() + " enter the state:" + getCurrentState().getName());
	        
			getCurrentState().activate();
		}
		catch (Exception e)
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "DSEHtmlProcessor: Exception caught while activating new state: " + newStateName);
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
			}
		}
	}
}
/**
 * Checks if the 'delegateValidationAssessment' parameter has been dynamically set by the JSP and
 * if so updates the html transition attribute with the proper value
 * @param HtmlTransitionDescriptor - The transition
 * @param String - the transition name
 * @param String - the session ID
 */
protected void checkDelValidationAssessmentFlag(HtmlTransitionDescriptor transition, String transitionName, String sessionID)
{
	HtmlProcessorInfo info = null;
	HtmlStateInfo stateInfo = null;
	info = HtmlProcessorManager.getUniqueHTMLInstance().getInfo(sessionID, processorId);
	if (info!=null) stateInfo = info.getCurrentStateInfo();
	if (stateInfo!=null) 
	{
		String flag=stateInfo.getTransitionParameter(transitionName, HtmlConstants.DELVALASSESSMENT);
		if (flag!=null)
		{
			if (flag.equals("true"))
				transition.setDelegateValAssessment(true);
			else
				transition.setDelegateValAssessment(false);
		}
	}
}

/**
 * <HSBC>
 * This utility method maps data from the source context to the target context
 * using the supplied DataMapperFormat.
 * 
 * @param format com.ibm.dse.base.DataMapperFormat
 * @param sourceCtx com.ibm.dse.base.Context
 * @param targetCtx com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.DSEException
 */
protected void doDataMapping(Tag formatTag, Context sourceCtx, Context targetCtx) throws com.ibm.dse.base.DSEException
{
	DataMapperFormat format = null;
	try
	{	format = (DataMapperFormat) FormatElement.getExternalizer().convertTagToObject(formatTag);
		format.mapContents(sourceCtx, targetCtx) ;
		FormatExternalizer.putIntoCache(format);
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while mapping data between source and target contexes."+
				"Format = "+format+"; Source = "+sourceCtx+"; Target = "+targetCtx) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}				
		throw new DSEException(DSEException.critical, null, "Exception caught while mapping data between source and target contexes: " + e.getMessage()) ;
	}
}

/**
 * This utility method returns a boolean indicating whether it is legal
 * for the state machine to switch to state supplied as argument. Legality
 * is determined thus:
 *
 * If the processor is in a consequential state, and an attempt is made to
 * switch to a state that does not define consequentiality, then legality
 * is set to false.
 * All other cases set legality to true ;
 * 
 * @return boolean
 * @param stateName java.lang.String
 */
public boolean checkLegalSwitch(String stateName)
{
	boolean legality = true ;
	State state = null ;
	TransitionDescriptor transition = null ;
 	try
	{
		// get state
		Tag aTag = getStateTag(stateName) ;
		state = getStateInstance(getCurrentState(), aTag) ;
//		state.initializeFrom(aTag) ;
	}
	catch(Exception e) {
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when initializing state: "+stateName) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}		
	}
	return checkLegalSwitch(state, transition) ;
}

/**
 * This utility method returns a boolean indicating whether it is legal
 * for the state machine to switch to state supplied as argument. Legality
 * is determined thus:
 *
 * If the processor is in a consequential state, and an attempt is made to
 * switch to a state that does not define consequentiality, then legality
 * is set to false.
 * All other cases set legality to true ;
 * 
 * @return boolean
 * @param stateName java.lang.String
 * @param eventName java.lang.String
 */
public boolean checkLegalSwitch(String stateName, String eventName)
{
	boolean legality = true ;
	State state = null ;
	TransitionDescriptor transition = null ;
 	// get state and transition
	try
	{
		// get state
		Tag aTag = getStateTag(stateName) ;
		state = getStateInstance(getCurrentState(), aTag) ;
//		state.initializeFrom(aTag) ;
 		// get transition
		transition = (TransitionDescriptor) state.getTransitions().get(eventName) ;
	}
	catch(Exception e) {
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception when initializing state: "+stateName+" for event name "+eventName) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}				
	}
	return checkLegalSwitch(state, transition) ;
}
/**
 * This utility method returns a boolean indicating whether it is legal
 * for the state machine to switch to state supplied as argument. Legality
 * is determined thus:
 *
 * If the processor is in a consequential state, and an attempt is made to
 * switch to a state that does not define consequentiality, then legality
 * is set to false.
 * All other cases set legality to true ;
 * 
 * @return boolean
 * @param state com.ibm.dse.automaton.State
 * @param transition com.ibm.dse.automaton.TransitionDescriptor
 */
protected boolean checkLegalSwitch(State state, TransitionDescriptor transition)
{
	boolean legality = true ;
 	// get the consequentiality of the state
	String conseq = CONSEQ_UNDEFINED ;
	try
	{
		conseq = ((DSEHtmlState)state).getConsequentiality() ;
		if(conseq.equals(CONSEQ_UNDEFINED)) conseq = ((HtmlTransitionDescriptor)transition).getConsequentiality();
	}
	catch(Exception e)
	{
	}
 	if(this.getStatus().equals(CONSEQUENTIAL) && conseq.equals(CONSEQ_UNDEFINED)) legality = false ;
 	return legality ;
}
/**
 * Unchains the context and releases the eventQueue object.
 *
 * @exception DSEInvalidRequestException
 * @exception DSEObjectNotFoundException
 */
public void close() throws DSEInvalidRequestException, DSEObjectNotFoundException {
	
	/** <HSBC> **/
	//** getContext().unchain();
	getProcessorContext().unchain();
	/** </HSBC> **/
	
	setEventQueue(null);
 }
/**
 * Serves as an entry point for execution of the state machine.
 * This method also supports the 
 * suspension of the state machine when states of a certain type
 * are encountered during processing.
 *
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
public void execute() throws DSEInvalidArgumentException, DSEProcessorException
{
	setStatus(RUNNING);
 	while(true)
	{
		State state = getCurrentState();
 		// check if the state has a type for which we will return control
		// to the caller
		if(stateReturnsControl(state))
		{
			// check if we have the final state, in this case, set the
			// processor status to DONE
			if(state.getType().equals(DSEState.FINAL_TYPE))
			{
				setStatus(COMPLETED);
			} else {
				if(state instanceof DSEHtmlState)
				{
					//trace processor enter the state
	                if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		                Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),"Processor:" + this.getName() + " enter the state:" + state.getName());
	                
	                
					state.activate();
					((DSEHtmlState) state).setIgnoreReturnType(true);
				}
			}
 			break;
		}
 		// if the state type is not one for which we return control, then
		// activate it
		//trace processor enter the state
	    if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		    Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),"Processor:" + this.getName() + " enter the state:" + state.getName());
	    
		state.activate();
	}
}
/**
 * Starts executing the HTML processor. An event
 * is dispatched to the state machine corresponding to the supplied event name.
 * This event is essentially the entry event used by the state machine to begin
 * processing.
 *
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.automaton.DSEProcessorException
 */
public void execute(String eventName) throws DSEInvalidArgumentException, DSEProcessorException
{
	DSEEventObject event = null ;
 	// check for a fully qualified event name
	int dotIndex = eventName.indexOf(".") ;
 	// if we have one, we need to handle it
	if(dotIndex != -1)
	{
		// we have to create a dummy notifier for the event
		String notifierName = eventName.substring(0, dotIndex) ;
		String actualEventName = eventName.substring(dotIndex + 1) ;
		//DSENotifier notifier = new DSENotifier(notifierName) ;
		DSENotifier notifier = new DSENotifier(false);
        notifier.setName(notifierName);
		event = new DSEEventObject(actualEventName, notifier) ;
	}
	else
	{
		event = new DSEEventObject(eventName, this) ;
	}
 	getCurrentState().dispatchEvent(event) ;
	execute() ;
}
/**
 * Returns the consequential error response defined for this processor.
 * 
 * @return java.lang.String
 */
public java.lang.String getConseqErrorResponse() {
	return conseqErrorResponse;
}
/**
 * Gets the parentAppSessionId
 * @return Returns a String
 */
public String getParentAppSessionId() {
		return parentAppSessionId;
}
/**
 * Returns the parent id attribute of this processor.
 * 
 * @return java.lang.String
 */
public java.lang.String getParentId() {
	return parentId;
}
/**
 * Gets the parentSessionId
 * @return Returns a String
 */
public String getParentSessionId() {
		return parentSessionId;
}
/**
 * Returns the specified timeout in miliseconds for the processor
 * 
 * @return java.lang.Lang
 */
public java.lang.Long getProcessorTimeout() {
	return processorTimeout;
}
/**
 * This is a utility method used to obtain an appropriate State
 * instance for the supplied tag. The class of the supplied State
 * argument is used if a class instance cannot be obtained in any
 * other way.
 * 
 * @return com.ibm.dse.automaton.State
 * @param aTag com.ibm.dse.base.Tag
 * @exception java.lang.Exception
 */
protected State getStateInstance(State state, Tag aTag) throws java.lang.Exception
{
	return (State) DSEProcessor.getExternalizer().convertTagToObject(aTag);
 }
/**
 * Gets the TC if any and updates it with the data elements whose
 * names are stored in the TCInfo of the given transactionName or creates
 * a new one.
 * @param HtmlTransitionDescriptor - The transition
 * @param String - the transition name
 * @param String - the session ID
 * @return the transition context
 */
public Context getTransitionContext(HtmlTransitionDescriptor transition, String transitionName, String sessionID)
{
	HtmlProcessorInfo info = null;
	HtmlStateInfo stateInfo = null;
	TCInfo tcInfo = null;
	info = HtmlProcessorManager.getUniqueHTMLInstance().getInfo(sessionID, processorId);
	if (info!=null) stateInfo = info.getCurrentStateInfo();
	else return ((HtmlTransitionDescriptor) transition).getContext();
	if (stateInfo!=null) tcInfo=stateInfo.getTCInfo(transitionName);
	else return ((HtmlTransitionDescriptor) transition).getContext();
	if (tcInfo!=null) 
	{
		// Get current transition context if any and set its keyed collection as a dynamic kColl
		Context tcContext = ((HtmlTransitionDescriptor) transition).getContext();
		KeyedCollection tcContextKColl=null;
		if (tcContext!=null)
		{
			tcContextKColl=tcContext.getKeyedCollection(); 
			tcContextKColl.setDynamic(true);	
		}
		
		int size=tcInfo.getCreateElementsLength();
		if (size > 0)
		{
				String [] elementsToCreate=tcInfo.getCreateElements();
				tcContext= new Context();
				tcContextKColl= new KeyedCollection();
				tcContextKColl.setDynamic(true);
				tcContext.setKeyedCollection(tcContextKColl);
				addElementsToTransitionContext(tcContext, elementsToCreate, size);
		}
		// add now the addElements and remove the removeElements
		int addSize=tcInfo.getAddElementsLength();
		if (addSize>0)
		{
			String [] elementsToAdd=tcInfo.getAddElements();
			if (tcContext==null)
			{
				tcContext= new Context();
				tcContextKColl= new KeyedCollection();
				tcContextKColl.setDynamic(true);
				tcContext.setKeyedCollection(tcContextKColl);
			}
			addElementsToTransitionContext(tcContext, elementsToAdd, addSize);
		}
		int removeSize=tcInfo.getRemoveElementsLength();
		if (removeSize>0) 
		{
			String [] elementsToRemove=tcInfo.getRemoveElements();
			if (tcContext==null)
			{
				tcContext= new Context();
				tcContextKColl= new KeyedCollection();
				tcContextKColl.setDynamic(true);
				tcContext.setKeyedCollection(tcContextKColl);
			}
			removeElementsFromTransitionContext(tcContextKColl, elementsToRemove, removeSize);
		}
		return tcContext;
	}
	else return ((HtmlTransitionDescriptor) transition).getContext();
}
/**
 * Returns the validator attribute for this class.
 * 
 * @return com.ibm.dse.automaton.html.ProcessorXValidate
 */
public ProcessorXValidate getXValidator() {
	return xValidator;
}
/**
 * This is a utility method used to obtain a number inside the given string. If this method finds one,  
 * the method returns its index in the String. If the method does not find one, the method returns -1. The 
 * number is supposed to be delimitated by dots ("asas.123.dfg.dfg"). If there 
 * is more than one number, the method returns the index of the first one.
 *
 * @return int
 * @param address java.lang.String
 */
public int indexOfInteger(String address)
{
	int number=-1;
	String pattern = address;
	
	for (int i = pattern.indexOf(Constants.dot); i != -1; i = pattern.indexOf(Constants.dot) )
	{
		try
		{
			number = Integer.parseInt( pattern.substring(0, i) );
		}
		catch (NumberFormatException e)
		{
			// Not a number, go on looking for numbers
			pattern = pattern.substring(i+1, pattern.length());
			continue;
		}
		// It is a number, just break and return it
		break;
	}
 	if ( number == -1 ) return -1;
	else return address.indexOf(new Integer(number).toString());
}
/**
 * Initializes this processor object's state from externalized
 * data definitions.
 * 
 * @return java.lang.Object
 * @param aTag com.ibm.dse.base.Tag
 * @exception java.io.IOException
 * @exception com.ibm.dse.base.DSEException
 */
public Object initializeFrom(Tag aTag) throws java.io.IOException, DSEException
{
	super.initializeFrom(aTag) ;
 	String name = null, value = null ;
	TagAttribute attribute = null ;
 	for(java.util.Enumeration e = aTag.getAttrList().elements() ; e.hasMoreElements() ; )
	{
		attribute = (TagAttribute) e.nextElement() ;
		name = (String) attribute.getName() ;
		value = (String) attribute.getValue() ;
 		if(name.equals(DSEHtmlProcessor.CONSEQ_ERROR_PAGE)) setConseqErrorResponse(value) ;
		if(name.equals(DSEHtmlProcessor.PROCESSTIMEOUT)) setProcessorTimeout(new Long(value));
		if(name.equals(DSEHtmlProcessor.XVALIDATION))
		{
			try
			{
				ProcessorXValidate validator = (ProcessorXValidate) Class.forName(value).newInstance() ;
				setXValidator(validator) ;
			}
			catch(Exception exception)
			{
				if(Trace.doTrace(DSEHtmlProcessor.COMPID,Trace.High, Trace.Debug))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "DSEHtmlProcessor.initializeFrom(Tag): Exception was caught while creating validator class") ;
				}
				throw new DSEException(DSEException.critical, null, "Exception caught while obtaining validator class: " + exception.getMessage()) ;
			}
		}
	}
 	//setProcessorId(JavaExtensions.getAlphaUniqueCode()) ;
 	return this ;
}
/**
 * This utility method returns true if the string value passed represents
 * the name of a HTML client system variable, which is defined as one
 * prefixed with <i>dse_</i>.
 * 
 * @return boolean
 * @param valueName java.lang.String
 */
protected boolean isSystemValue(String valueName)
{
	if(valueName == null) return false ;
 	return valueName.startsWith("dse_") ;
}
/**
 * Reinitializes THIS object.
 * It should be called immediately after being
 * extracted from the cache.
 */
public void reinitialize() 
{
	try
	{
		setEventQueue(new Queue());
		//		initialize();
	}
	catch(Exception e)
	{
		if(Trace.doTrace(Processor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(Processor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while reinitializing the processor: " + processorName + ". The processor is not reinitialized. Exception: " + e);
		}
	}
}
/** 
 * Removes the DataElements whose names are in elementsToAdd from the transition context keyed collection
 * @param KeyedCollection - The transition context kColl
 * @param String [] - The names of the elements to be removed from the kColl
 * @param int - The number of elements to be removed
 */
protected void removeElementsFromTransitionContext(KeyedCollection tcContextKColl, String [] elementsToRemove, int size)
{
	for (int i=0; i<size; i++)
	{
			String dataName=elementsToRemove[i];
			try {
				// the next line should dynamically remove the data element in the transition context kColl
				tcContextKColl.removeElementAt(dataName);
			}
			catch (Exception e)
			{
				if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "Unable to remove DataElement in the transition context: "+dataName) ;
					//Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;									
				}
			}
	}
}

/**
 * <HSBC>
 * Returns previous action context
 */
public Context getPrevActionContext() {
	return prevActionContext;
}

/**
 * <HSBC>
 * Returns inter-state format tag
 */
public Tag getInterStateFormatTag() {
	return isFormatTag;
}

/**
 * <HSBC>
 * Return the current contect using by this processor
 */
public Context getContext() {
	
	Context curContext = null;
	
	if (getCurrentState() != null && getCurrentState() instanceof DSEHtmlState && ((DSEHtmlState) getCurrentState()).getContext() != null) {
		curContext = super.getContext().getCurrentContext();
	}
	
	if (curContext == null) {
		curContext = super.getContext();
	}
	
	return curContext;
}

/**
 * <HSBC>
 * Returns the processor context
 */
public Context getProcessorContext() {
	return super.getContext();
}


/**
 * <HSBC>
 * Set the previous context from the previous action
 */
public void setPrevActionContext(Context paContext) {
	prevActionContext = paContext;
}

/**
 * <HSBC>
 * Set the inter state format tag
 */
public void setInterStateFormatTag(Tag formatTag) {
	isFormatTag = formatTag;
}

/**
 * Sets the consequential error response for this processor object.
 * 
 * @param newConseqErrorResponse java.lang.String
 */
public void setConseqErrorResponse(String newConseqErrorResponse) {
	conseqErrorResponse = newConseqErrorResponse;
}
/**
 * Sets the parentAppSessionId
 * @param parentAppSessionId The parentAppSessionId to set
 */
public void setParentAppSessionId(String parentAppSessionId) {
	this.parentAppSessionId = parentAppSessionId;
}
/**
 * Sets the parentId property of this processor object.
 * 
 * @param newParentId java.lang.String
 */
public void setParentId(String newParentId) {
	parentId = newParentId;
}
/**
 * Sets the parentSessionId
 * @param parentSessionId The parentSessionId to set
 */
public void setParentSessionId(String parentSessionId) {
	this.parentSessionId = parentSessionId;
}
/**
 * This utility method changes the status of the processor depending on the
 * consequentiality of the state and the supplied <i>consequentiality</i> argument.
 * 
 * @param state com.ibm.dse.automaton.State
 * @param consequentiality java.lang.String
 */
protected void setProcessorConsequentialState(State state, String consequentiality)
{
	// get the consequentiality of the new state
	String conseq = CONSEQ_UNDEFINED ;
	try
	{
		conseq = ((DSEHtmlState)state).getConsequentiality() ;
		if(conseq.equals(CONSEQ_UNDEFINED)) conseq = consequentiality ;
	}
	catch(Exception e) {}
 	// get the processor status
	String status = this.getStatus() ;
 	// if the processor is not consequential, and the new state is, set the processor
	// to be consequential
	if((!status.equals(CONSEQUENTIAL)) && conseq.equals(CONSEQ_TRUE)) this.setStatus(CONSEQUENTIAL) ;
 	// if the processor is consequential, and the new state resets, set the processor
	// to a status of RUNNING
	if(status.equals(CONSEQUENTIAL) && conseq.equals(CONSEQ_FALSE)) this.setStatus(RUNNING) ;
}
/**
 * Sets the specified time to the processor's timeout in miliseconds.
 * 
 * @param newParentId java.lang.Long
 */
public void setProcessorTimeout(java.lang.Long newProcessorTimeout) {
	processorTimeout = newProcessorTimeout;
}
private void setTypedDataInfo(Context tcContext, String dataName, String pathName) throws DSEObjectNotFoundException
{
	int index=dataName.indexOf(".");
	if (index==-1)
	{
		String fieldName=dataName;
		if (pathName.equals("")==false) fieldName=pathName+"."+dataName;
		DataField dataFieldInContext=(DataField)this.getContext().getElementAt(fieldName);
		DataField dataFieldInTC=(DataField)tcContext.getElementAt(fieldName);
		if (dataFieldInContext.getErrorInfo()!=null)
			dataFieldInTC.setErrorInfo(dataFieldInContext.getErrorInfo());
		if (dataFieldInContext.getDescriptor()!=null)
			dataFieldInTC.setDescriptor(dataFieldInContext.getDescriptor());
		if (((DataElement)dataFieldInContext).getState()!=null)
			((DataElement)dataFieldInTC).setState(((DataElement)dataFieldInContext).getState());
		if (((DataElement)dataFieldInContext).getParameters()!=null)
		((DataElement)dataFieldInTC).setParameters(((DataElement)dataFieldInContext).getParameters());
		return;	
	}
	else
	{
            	String firstElementName = pathName.length() > 0 ? pathName +"." : "";
            	firstElementName += dataName.substring(0, index);
		DataElement firstElementInTC=(DataElement)tcContext.getElementAt(firstElementName);		
		DataElement firstElementInContext=(DataElement)this.getContext().getElementAt(firstElementName);
		if ((firstElementInContext.getErrorInfo()!=null) && (firstElementInTC.getErrorInfo()==null))
			firstElementInTC.setErrorInfo(firstElementInContext.getErrorInfo());
		if ((firstElementInContext.getDescriptor()!=null) && (firstElementInTC.getDescriptor()==null))
			firstElementInTC.setDescriptor(firstElementInContext.getDescriptor());
		if ((firstElementInContext.getState()!=null) && (firstElementInTC.getState()==null))
			firstElementInTC.setState(firstElementInContext.getState());
		if ((firstElementInContext.getParameters()!=null) && (firstElementInTC.getParameters()==null))
		    firstElementInTC.setParameters(firstElementInContext.getParameters());
            	setTypedDataInfo(tcContext, dataName.substring(index + 1),(pathName.length() > 0 ? (pathName + ".") : "") + dataName.substring(0,index));
	}
}
/**
 * Sets the validator attribute of this processor object.
 * 
 * @param newXValidator com.ibm.dse.automaton.html.ProcessorXValidate
 */
public void setXValidator(ProcessorXValidate newXValidator) {
	xValidator = newXValidator;
}
/**
 * This method is a utility method for HTML processors and determines
 * whether the state supplied to it as argument is of a type for which the
 * processor will suspend execution of the state machine. It returns a boolean
 * indicating whether this should occur or not.
 * 
 * @return boolean
 * @param state com.ibm.dse.automaton.State
 */
public static boolean stateReturnsControl(State state)
{
	// if it is not an HTML state, then return immediately
	if(!(state instanceof com.ibm.dse.automaton.html.DSEHtmlState)) return false ;
 	com.ibm.dse.automaton.html.DSEHtmlState htmlState = (com.ibm.dse.automaton.html.DSEHtmlState) state ;
	try
	{
		// if the state specifies to ignore suspension, then return false
		if ( htmlState.isIgnoreReturnType() ) return false;
 		String stateType = htmlState.getType();
 		if ( stateType == null ) return false;
		
		return (stateType.equals(DSEState.PAGE_TYPE) || stateType.equals(DSEState.SUBFLOW_TYPE) || stateType.equals(DSEState.FINAL_TYPE)) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while parsing the state: "+state.getName());
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
		}		
		return false;
	}
}
/**
 * Updates the processor context by copying values by name
 * from the supplied context into the processor context.
 *
 * @param context com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 * @exception com.ibm.dse.base.DSEException
 */
public void updateContext(Context context) throws DSEObjectNotFoundException, DSEException
{
	if(context == null) return;
 	String qualifiedName = null;
	DataElement data = null;
	Hashtable nestedQualifiedElements = context.getKeyedCollection().calculeNestedQualifiedElements();
	
	// Get the processor's context.
	Context targetContext = getContext();
 	// Get an enumeration with the qualified names of all the fields of the transition's context keyed collection.
	for(java.util.Enumeration e = nestedQualifiedElements.keys(); e.hasMoreElements(); )
	{
		qualifiedName = (String) e.nextElement();
 		//------
		// Get the field which matches with the name.
		// We need to get the data element in this way instead of using context.getElementAt()
		// because the name is full qualified (kCollName.dataFieldName)
		//------	
		data = (DataElement) nestedQualifiedElements.get(qualifiedName);
 		if ( data instanceof HtmlIndexedCollection )
		{
			HtmlIndexedCollection htmlIColl = (HtmlIndexedCollection) data;
			// Look for an indexed collection with this name in the processor's context
			IndexedCollection iColl = (IndexedCollection) targetContext.getElementAt(data.getName());
			// Loop the transition's context iColl and add an element per each one in the processor's iColl
			for ( java.util.Enumeration enum = htmlIColl.getElements().elements(); enum.hasMoreElements(); )
			{
				iColl.addElement((DataElement) enum.nextElement());
			}
		}
		
		else
		{
			if (data instanceof DataCollection)
				continue;
			else
			{
				Hashtable parameters=data.getParameters();
				Object ignoreValidationFlag=null;
				if (parameters!=null) ignoreValidationFlag=parameters.get(HtmlConstants.IGNOREVALIDATIONRESULTS);
				if ((ignoreValidationFlag!=null) && (ignoreValidationFlag.equals("true")))
				{
					//promote the inputObject
					Object inputObject=parameters.get(HtmlConstants.INPUTOBJECT);
					DataElement targetElement=targetContext.getElementAt(qualifiedName);
					if (targetElement.getParameters()==null)
					{
						Hashtable params=new Hashtable(1);
						params.put(HtmlConstants.INPUTOBJECT, inputObject);
						targetElement.setParameters(params);
					}
					else targetElement.getParameters().put(HtmlConstants.INPUTOBJECT, inputObject);
				}
				else targetContext.setValueAt(qualifiedName, data.getValue());
			}
		}
	}
}
/**
 * Validates the data received from client and copies it into the processor context.
 * The validation includes the field validation and the cross validation. If a 
 * transition context has been defined, only the sent elements that appear in this 
 * context are copied.
 *
 * @return boolean
 * @param coll com.ibm.dse.base.KeyedCollection
 * @exception com.ibm.dse.base.types.DSETypeException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 */
public boolean updateContext(KeyedCollection coll) throws com.ibm.dse.base.types.DSETypeException, DSEObjectNotFoundException, DSEInvalidArgumentException
{
	boolean usingProcCtx = false, skipValidation = false;
	Context context = null ;
	TransitionDescriptor transition = null ;
 	// Get the transition corresponding to the event name.
	String eventName = (String) coll.getValueAt(HtmlConstants.NEXTEVENTNAME);
	if(eventName.indexOf(".") == -1)
			eventName = getCurrentState().getName() + "." + eventName;
 	try 
	{
		transition = (TransitionDescriptor) getCurrentState().getTransitions().get(eventName) ;
	}
	catch (NullPointerException e)
	{ /* Do nothing, transition will be null, next line will throw the exception */ }
	
	if ( transition == null ) throw new DSEInvalidArgumentException(DSEException.critical, "Html processor", "Unable to find a transition in the state: "+getCurrentState().getName()+" corresponding with the event name: "+eventName);
 	// Get the context for the transition.
	if(transition instanceof HtmlTransitionDescriptor)	
	{
		//context = ((HtmlTransitionDescriptor) transition).getContext();
		String sessionID=(String) coll.getValueAt(HtmlConstants.SESSIONID);
		context=getTransitionContext((HtmlTransitionDescriptor)transition, ((HtmlTransitionDescriptor)transition).getName(), sessionID); 
	}
	if(context == null) 
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Could not obtain a context for the transition to use" +
				" for validation. Using processor context instead. Transition = "+transition.getName());
		}
		context = getContext();
 		// If the transition does not have a context, validate using the processor context.
		usingProcCtx = true;
	}
 	// Chain the transition's context to the processor's context.
	try
	{
		if(!usingProcCtx)
		{
			if(context.isChained()) context.unchain();
			context.chainTo(getContext());
		}
	}
	catch(DSEInvalidRequestException e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while chaining transition context "+context+" to processor context "+getContext());
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}
		throw new DSEInvalidArgumentException(DSEException.critical, null, "Exception caught while chaining transition context to processor context: " + e.toString());
	}
 	// Perform field validation of the request.
	try
	{
		if (usingProcCtx) validate(coll, context);
		else validateFromTransition(coll, context);
	}
	catch(Exception e)
	{
		// If a field validation error occur, trace and return control.
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while validating data") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
	
		// If the Error Messages field doesn't exist it's ok. If it exists it has to be cleaned.
		try 
		{
			context.setValueAt(HtmlConstants.ERRORMESSAGES, new String[0]);
		}
		catch (DSEObjectNotFoundException ex) {}
		
		return false;
	}
 	// Cross validate the data
	try
	{
		xValidate(context, transition.getName()) ;
	}
	catch(Exception e)
	{
		// If the cross validation is unsuccessful, trace and return control.
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while cross validating data.") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}
 		return false ;
	}
 	try
	{
		if(!usingProcCtx) 
		{
			if ( ((HtmlTransitionDescriptor) transition).isSkipValidation() )
			{
				// Store the trCtx in the prCtx to get access afterwards to construct the response
				DataField df = new DataField();
				df.setName("dse_trCtx");
				df.setValue(context);
				getContext().addElement(df);
			}
			else
			{
				// Update the processor context from the transition context.
				updateContext(context);
			}
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while updating processor context from transition context.") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
 		return false ;
	}
 	// Unchain the transition's context.
	try
	{
		if(! usingProcCtx) context.unchain() ;
	}
	catch(DSEInvalidRequestException dire)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while "+
								"unchaining transition's context from the processor's context. Exception: " + dire);			
		}
		throw new DSEInvalidArgumentException(DSEException.harmless, null, "Exception caught while unchaining context; " + dire.toString()) ;
	}
 	// Returns true because the request's data was valid.
	return true ;
}
/**
 * Validates the data received from client and copies it into the processor context.
 * The validation includes the field validation and the cross validation. If a 
 * transition context has been defined, only the sent elements that appear in this 
 * context are copied.
 *
 * @return boolean
 * @param coll com.ibm.dse.base.KeyedCollection
 * @exception com.ibm.dse.base.types.DSETypeException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 */
public boolean updateContext(KeyedCollection coll, String sessionID) throws com.ibm.dse.base.types.DSETypeException, DSEObjectNotFoundException, DSEInvalidArgumentException
{
	boolean usingProcCtx = false, skipValidation = false;
	Context context = null ;
	TransitionDescriptor transition = null ;
 	// Get the transition corresponding to the event name.
	String eventName = (String) coll.getValueAt(HtmlConstants.NEXTEVENTNAME);
	if(eventName.indexOf(".") == -1)
			eventName = getCurrentState().getName() + "." + eventName;
 	try 
	{
		transition = (TransitionDescriptor) getCurrentState().getTransitions().get(eventName) ;
	}
	catch (NullPointerException e)
	{ /* Do nothing, transition will be null, next line will throw the exception */ }
	
	if ( transition == null ) throw new DSEInvalidArgumentException(DSEException.critical, "Html processor", "Unable to find a transition in the state: "+getCurrentState().getName()+" corresponding with the event name: "+eventName);
 	
	
	
	if(transition instanceof HtmlTransitionDescriptor)	
	{
		// Check if the flag 'checkDelValidationAssessment' has been activated by the JSP
		checkDelValidationAssessmentFlag((HtmlTransitionDescriptor) transition, eventName, sessionID);
		// Get the context for the transition.	
		context=getTransitionContext((HtmlTransitionDescriptor) transition, eventName, sessionID);
	}
	if(context == null) 
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Could not obtain a context for the transition to use" +
				" for validation. Using processor context instead. Transition = "+transition.getName());
		}
		context = getContext();
 		// If the transition does not have a context, validate using the processor context.
		usingProcCtx = true;
	}
 	// Chain the transition's context to the processor's context.
	try
	{
		if(!usingProcCtx)
		{
			if(context.isChained()) context.unchain();
			context.chainTo(getContext());
		}
	}
	catch(DSEInvalidRequestException e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while chaining transition context "+context+" to processor context "+getContext());
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}
		throw new DSEInvalidArgumentException(DSEException.critical, null, "Exception caught while chaining transition context to processor context: " + e.toString());
	}
 	// Perform field validation of the request.
	try
	{
		if (usingProcCtx) validate(coll, context);
		else validateFromTransition(coll, context);
	}
	catch(Exception e)
	{
		if ((!(transition instanceof HtmlTransitionDescriptor)) || (((HtmlTransitionDescriptor)transition).getDelegateValAssessment()==false))
		{
			// If a field validation error occur, trace and return control.
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while validating data") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
			}
		
			// If the Error Messages field doesn't exist it's ok. If it exists it has to be cleaned.
			try 
			{
				context.setValueAt(HtmlConstants.ERRORMESSAGES, new String[0]);
			}
			catch (DSEObjectNotFoundException ex) {}
		
			return false;
		}
	}
 	// Cross validate the data
	try
	{
		xValidate(context, transition.getName()) ;
	}
	catch(Exception e)
	{
		if ((!(transition instanceof HtmlTransitionDescriptor)) || (((HtmlTransitionDescriptor)transition).getDelegateValAssessment()==false))
		{
			
			// If the cross validation is unsuccessful, trace and return control.
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while cross validating data.") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}
			return false ;
		}
	}		
 	try
	{
		if(!usingProcCtx) 
		{
			if ( ((HtmlTransitionDescriptor) transition).isSkipValidation() )
			{
				// Store the trCtx in the prCtx to get access afterwards to construct the response
				DataField df = new DataField();
				df.setName("dse_trCtx");
				df.setValue(context);
				getContext().addElement(df);
			}
			else if (((HtmlTransitionDescriptor)transition).getDelegateValAssessment()==true) 
			{
				// Set the transition context as the process context
				setContext(context);
				DataField delValAssessmentField = new DataField();
				delValAssessmentField.setName(HtmlConstants.DELVALASSESSMENT);
				delValAssessmentField.setValue("true");
				context.addElement(delValAssessmentField);				
			}
			else 
			{
				// Update the processor context from the transition context.
				updateContext(context);//context.getParent().getElementAt("incone"
			}
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while updating processor context from transition context.") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
 		return false ;
	}
 	// Unchain the transition's context .
	try
	{
		if ((! usingProcCtx) && (((HtmlTransitionDescriptor)transition).getDelegateValAssessment()==false))
			context.unchain() ;
	}
	catch(DSEInvalidRequestException dire)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while "+
								"unchaining transition's context from the processor's context. Exception: " + dire);			
		}
		throw new DSEInvalidArgumentException(DSEException.harmless, null, "Exception caught while unchaining context; " + dire.toString()) ;
	}
 	// Returns true because the request's data was valid.
	return true ;
}
/**
 * Performs basic validation of the business data passed in the
 * keyed collection. Basic validation is a copy-by-name into the supplied
 * context. Extensive validation is only possible when using typed
 * data. With typed data, the data element validator can
 * verify the data value before it is copied into the context.
 *
 * @return boolean
 * @param coll com.ibm.dse.base.KeyedCollection
 * @param context com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.DSEException
 */
public void validate(KeyedCollection coll, Context context) throws DSEObjectNotFoundException, com.ibm.dse.base.types.DSETypeException, DSEInvalidArgumentException
{
	DataElement data = null, contextData = null;
	com.ibm.dse.base.types.DSETypeException allExceptions = null;
	ProcessorXValidate validator = getXValidator() ;
	Object initialValue=null;
	
 	// Get an enumeration with all the fields of the keyed collection created from the client's request.
	for (java.util.Enumeration e = coll.getEnumeration(); e.hasMoreElements(); )
	{
		data = (DataElement) e.nextElement();
		if(isSystemValue(data.getName())) continue;
		
		try
		{		
			// Look for a context field which matches with the received element.
			contextData = context.getElementAt( data.getName() );
 			// Keep the initial value
			initialValue=data.getValue();
			// Perform the sintactic validation
			Object value=contextData.validate(data.getValue(),DSEHtmlProcessor.HTML);
			// Perform the semantic validation only for DataFields
			if (data instanceof DataField)
			{
				data.setValue(value);
				if (validator!=null) validator.validate(data.getName(), (DataField)data, context);
			}
			// As both validations has been passed set the input data value to the context
			contextData.setValue(value);
		}
		catch (DSEObjectNotFoundException nfe)
		{
			//------
			// Two possibilities: the element does not really exist, so throw the exception,
			// or the element is wished to integrate in an indexed collection.
			// If the element contains the String ".int." where int is any integer means that it is 
			// intended to integrate in an iColl
			//------
			String name = (String) data.getName();
			int i = indexOfInteger(name);
			if (i == -1)
				throw nfe;
			
			// Get the name of the iColl
			int beginIndex;
			String iCollName = name.substring(0, i-1);
			//beginIndex = iCollName.lastIndexOf(".");
			//if ( beginIndex != -1 ) iCollName = iCollName.substring(beginIndex+1, iCollName.length());
 			// Create a DataField, the name must begin with the indes in the iColl, followed by the path inside this kColl
			DataField df = new DataField();
			df.setName( name.substring(i) );
			df.setValue( data.getValue() );
 			// Look for the iColl and send to it the data field
			HtmlIndexedCollection htmlIColl = (HtmlIndexedCollection) context.getElementAt( iCollName );
			htmlIColl.setValueAt(name.substring(i), htmlIColl.validate(df));
		}
		catch(com.ibm.dse.base.types.DSETypeException dte)
		{
			// if the semantic validation has failed store the initialValue in the inputObject of the errorInfo
			if (data.getValue()!=initialValue) 
			{
				ErrorInfo errorInfo=contextData.getErrorInfo();
				if (errorInfo==null) 
				{
					errorInfo=new ErrorInfo(initialValue);
					contextData.setErrorInfo(errorInfo);
				}
				else errorInfo.setInputObject(initialValue);
			}
			// First of throwing the exception, validate each field.
			if (allExceptions == null) allExceptions=dte;
			else allExceptions=dte.chainTo(allExceptions);
		}
	}
 	// Once all the fields are validated, if an exception has been thrown, throw it.
	if (allExceptions!=null) throw allExceptions;
}
/**
 * Performs basic validation of the business data passed in the
 * keyed collection. Basic validation is a copy-by-name into the supplied
 * context. Extensive validation is only possible when using typed
 * data. With typed data, the data element validator can
 * verify the data value before it is copied into the context.
 *
 * @return boolean
 * @param coll com.ibm.dse.base.KeyedCollection
 * @param context com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.DSEException
 */
public void validateFromTransition(KeyedCollection coll, Context context) throws DSEException
{
	DataElement data = null, contextData = null;
	com.ibm.dse.base.types.DSETypeException allExceptions = null;
	String qualifiedName = null;
	com.ibm.dse.base.types.ElementState elementState;
 	Hashtable ht = context.getKeyedCollection().calculeNestedQualifiedElements();
 	ProcessorXValidate validator = getXValidator() ;
 	Object initialValue = null;
	
	// Get an enumeration with all the fields of the transition's context keyed collection.
	for(java.util.Enumeration e = ht.keys(); e.hasMoreElements(); )
	{
		qualifiedName = (String) e.nextElement();
		contextData = (DataElement) ht.get(qualifiedName);
		DataElement dataInTC=context.getElementAt(qualifiedName);

		if (contextData instanceof KeyedCollection)
		{
			// From an HTML page we can not receive in a parameter a whole KeydCollection,
			// we can receive the elements, but if it is a KeyedCollectio, we will not find it in coll, so continue to avoid useless tracing.
			continue;
		}

		Hashtable parameters=dataInTC.getParameters();
		Object ignoreValidationFlag=null;
		if (parameters!=null) ignoreValidationFlag=parameters.get(HtmlConstants.IGNOREVALIDATIONRESULTS);
 		try
		{	
			if (contextData instanceof HtmlIndexedCollection)
			{
				DataElement de;
				String name, iCollName;
				// Get the fields of the request which are part of these iColl and pass them to the transition context
				for (java.util.Enumeration enum = coll.getEnumeration(); enum.hasMoreElements(); )
				{
					de = (DataElement) enum.nextElement();
					name = de.getName();
					if ( name.indexOf(Constants.dot + qualifiedName + Constants.dot) != -1 )
					{
						int i = indexOfInteger(name);
						if (i != -1)
						{
							iCollName = contextData.getName();
							
							// Create a DataField, the name must begin with the index in the iColl, followed by the path inside this kColl
							DataField df = new DataField();
							df.setName( name.substring(i) );
							df.setValue( de.getValue() );
 							// Look for the iColl and send to it the data field
							HtmlIndexedCollection htmlIColl = (HtmlIndexedCollection) context.getElementAt( iCollName );
							htmlIColl.setValueAt(name.substring(i), htmlIColl.validate(df));
						}
					}
				}
				continue;
			}
 			//------
			// Look for a received field which matches with the transition context element.
			// We need to get the data element in this way instead of using coll.getElementAt()
			// because the name is full qualified (kCollName.dataFieldName)
			//------
			data = (DataElement) coll.getElements().get(qualifiedName);
 			if (data==null)
			{
				// There is a field defined in the context which does not exist in the request, do trace.
				if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Warning))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Warning, Settings.getTID(), "Field validation for transition context "+ context.getName()+
						". Expected data wasn't found in the request's data: "+qualifiedName) ;
				}
			}				 
			else
			{
				// Keep the initialValue in case semantic validation fails
				initialValue=data.getValue();			
				// Perform the sintactic validation
				Object value=contextData.validate(data.getValue(),DSEHtmlProcessor.HTML);
				// Perform the semantic validation only for DataFields
				if (data instanceof DataField)
				{
					data.setValue(value);

					if (validator!=null) validator.validate(qualifiedName, (DataField)data, context);
				}
				// As both validation has been passed set the input data value to the context
				context.setValueAt( qualifiedName, value);
				elementState = context.getParent().getElementAt(qualifiedName).getState();
				if (!(elementState == null)) contextData.setState(elementState);
				if ((ignoreValidationFlag!=null) && (((String)ignoreValidationFlag).equals("true")))
				{
					// store the correct value in the dse_inputObject
					parameters.put(HtmlConstants.INPUTOBJECT, data.getValue());
				}
			}
		}
 		catch (ClassCastException cce) 
		{
			//------
			// Do nothing, means that the data readed from the transition ctx is a DataCollection
			// instread of a DataField. It appears here because calculeNestedQualifiedElements()
			// returns both DataFields and DataCollections.
			//------
		}
 		catch(com.ibm.dse.base.types.DSETypeException dte)
		{
			// if the semantic validation has failed store the initialValue in the inputObject of the errorInfo
			if (data.getValue()!=initialValue) 
			{
				ErrorInfo errorInfo=contextData.getErrorInfo();
				if (errorInfo==null) 
				{
					errorInfo=new ErrorInfo(initialValue);
					contextData.setErrorInfo(errorInfo);
				}
				else errorInfo.setInputObject(initialValue);
			}
			
			if ((ignoreValidationFlag!=null) && (((String)ignoreValidationFlag).equals("true")))
			{
				// store the input object in an additional parameter of the data element and reset the errorInfo	
				ErrorInfo errorInfo=dataInTC.getErrorInfo();
				parameters.put(HtmlConstants.INPUTOBJECT, errorInfo.getInputObject());
				dataInTC.setErrorInfo(null);
			}
			else
			{	
				// First of throwing the exception, validate each field.
				if (allExceptions == null) allExceptions=dte;
				else allExceptions=dte.chainTo(allExceptions);
			}
		}
	}
	// Once all the fields are validated, if an exception has been thrown, throw it.
	if (allExceptions!=null) throw allExceptions;
}
/**
 * Uses the processor's defined validator class to perform validation
 * of data and stores any exceptions thrown in the course of validation in the
 * context heirarchy to be utilized by the HomeBanking toolkit.
 *
 * @param context com.ibm.dse.base.Context
 * @param transitionName java.lang.String
 * @exception java.lang.Exception
 */
public void xValidate(Context context, String transitionName) throws Exception
{
	ProcessorXValidate validator = getXValidator() ;
	if(validator == null) return ;
 	String[] messages = validator.xValidate(context, transitionName) ;
 	if(messages != null)
	{
		context.setValueAt(HtmlConstants.ERRORMESSAGES, messages);
		throw new Exception("Cross validation of the business data produced errors") ;
	}
 	return ;
}
/**
 * release the the references of other objects, Some of these objects are candidates for garbage collection
 * 
 */
public void clear()
{
	if( this.currentState != null )
	{
	  this.currentState.terminate();
	  //trace processor terminate the state
	  if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		  Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(),"Processor:" + this.getName() + " terminate the state:" + this.currentState.getName());
	  
	    
	}
	this.currentState=null;
	this.context=null;
	this.eventQueue=null;
}
}
