package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import java.util.Enumeration;
 
import com.ibm.dse.base.* ;
import com.ibm.dse.automaton.* ;
import com.ibm.dse.automaton.html.* ;
import com.ibm.dse.cs.html.HtmlConstants ;

/**
 * This class represents a virtual processor manager for the Homebanking
 * HTML Client. It enables processes to be run over multiple invocations across
 * the HTML C/S Mechanism without having to maintain the processor object on the
 * server machine. This class provides for persistence of a processor object so
 * that it can be reinstantiated in the state it was in on the last CS invocation.
 * The class also provides for updating the processor context with data obtained
 * on each invocation across the CS mechanism.
 *
 * @copyright(c) Copyright IBM Corporation 2000, 2003
 */
public class DSEHtmlProcessorOperation extends com.ibm.dse.base.DSEServerOperation implements ProcessorOperation
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2003 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	public static final String NORMAL = "NORMAL" ;
	public static final String ILLEGALSWITCH = "ILLEGALSWITCH" ;
	private com.ibm.dse.automaton.Processor processor = null ;
	private java.lang.String status = NORMAL ;
	//Add for PMR27673 begin,  Declare a new varaible that indicate if the processor is new
	private boolean ProcessorSaved = true ;
	// PMR27673 end
/**
 * DSEHtmlProcessorOperation constructor.
 */
public DSEHtmlProcessorOperation() {
	super();
}
/**
 * DSEHtmlProcessorOperation constructor.
 * @param anOperationName java.lang.String
 * @exception java.io.IOException
 */
public DSEHtmlProcessorOperation(String anOperationName) throws java.io.IOException {
	super(anOperationName);
}
/**
 * DSEHtmlProcessorOperation constructor.
 * @param anOperationName java.lang.String
 * @param aParentContext java.lang.String
 * @exception java.io.IOException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 * @exception com.ibm.dse.base.DSEInvalidRequestException
 */
public DSEHtmlProcessorOperation(String anOperationName, String aParentContext) throws java.io.IOException, com.ibm.dse.base.DSEObjectNotFoundException, com.ibm.dse.base.DSEInvalidRequestException {
	super(anOperationName, aParentContext);
}
/**
 * DSEHtmlProcessorOperation constructor.
 * @param anOperationName java.lang.String
 * @param aParentContext com.ibm.dse.base.Context
 * @exception java.io.IOException
 * @exception com.ibm.dse.base.DSEInvalidRequestException
 */
public DSEHtmlProcessorOperation(String anOperationName, com.ibm.dse.base.Context aParentContext) throws java.io.IOException, com.ibm.dse.base.DSEInvalidRequestException {
	super(anOperationName, aParentContext);
}
/**
 * This utility method chains the supplied context to the session context
 * identified by the <i>sessionId</i> argument. This method first 
 * ensures that the supplied context is not already chained to the session
 * context.
 * 
 * @param processor com.ibm.dse.base.Context
 * @param sessionId java.lang.String
 * @exception com.ibm.dse.base.DSEException
 */
public static void chainToApplicationSessionContext(Context context, String sessionId, String applicationId) throws DSEException
{
	Context applicationContext = Context.getCurrentContextForSession(sessionId , applicationId) ;

	// check to ensure the context is not already chained to the session
	// context specified by the supplied session id
	try
	{
		Context parent = context.getParent() ;
		while(parent != null)
		{
			if(parent==applicationContext)
			{
				if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Requested context is already chained to the application session context.") ;
				}
				return ;
			}
			parent = parent.getParent() ;
		}
	}
	catch(Exception ignored) 
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Exception when checking whether the context: "+context+" was already chained to the application session context:"+applicationContext) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), ignored) ;								
		}
	}

	context.chainTo(applicationContext) ;
}
/**
 * This utility method chains the supplied context to the session context
 * identified by the <i>sessionId</i> argument. This method first
 * ensures that the supplied context is not already chained to the session
 * context.
 * 
 * @param processor com.ibm.dse.base.Context
 * @param sessionId java.lang.String
 * @exception com.ibm.dse.base.DSEException
 */
public static void chainToSessionContext(Context context, String sessionId) throws DSEException
{
	Context sessionContext = Context.getCurrentContextForSessionOrApplication(sessionId) ;

	// check to ensure the context is not already chained to the session
	// context specified by the supplied session id
	try
	{
		Context parent = context.getParent() ;
		while(parent != null)
		{
			if(parent==sessionContext)
			{
				if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Requested context is already chained to the session context.") ;
				}
				return ;
			}
			parent = parent.getParent() ;
		}
	}
	catch(Exception ignored) 
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Exception when checking whether the context: "+context+" was already chained to the session context:"+sessionContext) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), ignored) ;								
		}
	}

	context.chainTo(sessionContext) ;
}
/**
 * This utility method checks to see if this invocation of the processor
 * operation has occured as a result of a back click by the user (either by
 * using the browser navigation bar, or browser history or by typing a
 * previously viewed URL in the location bar). This method relies on system
 * values defined for the HTML client to determine whether such a back click
 * has occured.
 * 
 * @return boolean - True if the back click occured
 * @param pageId java.lang.String
 * @param lastPageId java.lang.String
 */
protected boolean checkBackClick(String currentPageId, String previousPageId)
{
	int currPageNum = 0, lastPageNum = 0 ;
	try 
	{
		currPageNum = Integer.parseInt(currentPageId);
		lastPageNum = Integer.parseInt(previousPageId);
	}
	catch (Exception e) 
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Could not obtain current and/or last page id processed. Using defaults of 0: " + e.toString()) ;
		}
	}
	
	if(currPageNum < lastPageNum) return true; 
	return false;
}
/**
 * This is a utility method that checks to see whether it is legal (from
 * the point of view of consequentiality) for this
 * processor to be put in the state specified by the supplied state name.
 * 
 * @return boolean
 * @param processor com.ibm.dse.automaton.Processor
 * @param processorState java.lang.String
 */
protected boolean checkLegalSwitch(Processor processor, String processorState)
{
	if(processor instanceof DSEHtmlProcessor)
	{
		return ((DSEHtmlProcessor)processor).checkLegalSwitch(processorState) ;
	}
	else return true ;
}
/**
 * Closes the processor operation. This method closes the processor currently being
 * managed and releases its hold on the instance.
 *
 * @exception com.ibm.dse.base.DSEInvalidRequestException
 * @exception com.ibm.dse.base.DSEDSEObjectNotFoundException
 */
public void close() throws com.ibm.dse.base.DSEInvalidRequestException, com.ibm.dse.base.DSEObjectNotFoundException
{
	getProcessor().close() ;
	setProcessor(null) ;
}
/**
 * This is equivalent of calling closeProcessor(Processor, false)
 * 
 * @param processor com.ibm.dse.automaton.Processor
 */
public void closeProcessor(Processor processor)
{
	this.closeProcessor(processor, false) ;
}
/**
 * Closes the processor supplied to it as argument. If
 * the boolean argument is true, any parent processor is also
 * closed.
 * 
 * @param processor com.ibm.dse.automaton.Processor
 * @param removeParents boolean
 */
public void closeProcessor(Processor processor, boolean removeParents)
{
	HtmlProcessorManager.getUniqueHTMLInstance().removeProcessor(getSessionId(), processor.getProcessorId(), removeParents);

	// now, terminate and close the processor
	try
	{
		processor.terminate();
		processor.close() ;
	}
	catch (Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "DSEHtmlProcessorOperation.closeProcessor: Could not terminate the processor instance: " + e.toString()) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;						
		}
	}
}
/**
 * Closes the processor operation. This method only will unchain the context and clean 
 * the event queue.
 * @exception com.ibm.dse.base.DSEInvalidRequestException
 * @exception com.ibm.dse.base.DSEDSEObjectNotFoundException
 */
public void closeWithoutCleanProcessor() throws com.ibm.dse.base.DSEInvalidRequestException, com.ibm.dse.base.DSEObjectNotFoundException
{
	getProcessor().close() ;
	
}
/**
 * Creates a processor object of the type specified by the
 * <i>name</i> argument supplied to it.
 * 
 * @return com.ibm.dse.automaton.Processor
 * @param processorName java.lang.String
 * @exception com.ibm.dse.automaton.html.DSEHtmlCreateProcessorException
 */
protected Processor createProcessor(String processorName) throws com.ibm.dse.automaton.html.DSEHtmlCreateProcessorException
{
	Processor processor = null ;

	try
	{
		// create a new processor instance
		processor = HtmlProcessorManager.getUniqueHTMLInstance().createProcessor(processorName) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while creating processor object") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}		
		throw new DSEHtmlCreateProcessorException(DSEException.critical, null, "Exception caught while creating processor object: " + e.toString()) ;
	}

	// chain processor context to the session context or application session context
	try
	{
		if((!(getApplicationId().equals(HtmlConstants.IGNORE)))){
			chainToApplicationSessionContext(processor.getContext(), getSessionId(), getApplicationId());
		}else{
			chainToSessionContext(processor.getContext(), getSessionId()) ;
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "Could not chain processor to session context. Ensure that the processor context definition in the external files is not chained.") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), e) ;						
		}
		throw new DSEHtmlCreateProcessorException(DSEException.critical, null, "Exception caught while creating processor. Could not chain processor to session context: " + e.getMessage()) ;
	}

	return processor ;
}
/**
 * Creates a sub processor for the supplied processor
 * object using the supplied processor name. The new processor object
 * is chained to its parent.
 * 
 * @return com.ibm.dse.automaton.Processor
 * @param procName java.lang.String
 * @param parent com.ibm.dse.automaton.Processor
 * @exception com.ibm.dse.automaton.html.DSEHtmlCreateProcessorException
 */
public Processor createSubProcessor(String procName, Processor parent) throws com.ibm.dse.automaton.html.DSEHtmlCreateProcessorException
{
	if(parent == null) throw new DSEHtmlCreateProcessorException(DSEException.critical, null, "Cannot create a subprocessor for a null parent") ;


	Processor processor = createProcessor(procName) ;


	try
	{
		((DSEHtmlProcessor)processor).setParentId(parent.getProcessorId()) ;
		
		((DSEHtmlProcessor)processor).setParentSessionId(getSessionId());
		((DSEHtmlProcessor)processor).setParentAppSessionId(getApplicationId());
	}
	catch(ClassCastException e) {
		if(processor instanceof DSEHtmlProcessor)
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Processor "+procName+" is not an Html processor. Check the definitions in the XML files") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}
		} else {
			Trace.trace(DSEProcessor.COMPID, Trace.Low, Trace.Information, Settings.getTID(), "Instantiated DSEProcessor") ;
		}
	}


	return processor ;
}
/**
 * This utility method maps data from the source context to the target context
 * using the supplied DataMapperFormat.
 * 
 * @param format com.ibm.dse.base.DataMapperFormat
 * @param sourceCtx com.ibm.dse.base.Context
 * @param targetCtx com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.DSEException
 */
protected void doDataMapping(DataMapperFormat format, Context sourceCtx, Context targetCtx) throws com.ibm.dse.base.DSEException
{
	try
	{
		format.mapContents(sourceCtx, targetCtx) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while mapping data between source and target contexes."+
				"Format = "+format+"; Source = "+sourceCtx+"; Target = "+targetCtx) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}				
		throw new DSEException(DSEException.critical, null, "Exception caught while mapping data between source and target contexes: " + e.getMessage()) ;
	}
}
/**
 * This utility method maps data from the source context to the target context
 * using the supplied DataMapperFormat.
 * 
 * @param format com.ibm.dse.base.DataMapperFormat
 * @param sourceCtx com.ibm.dse.base.Context
 * @param targetCtx com.ibm.dse.base.Context
 * @exception com.ibm.dse.base.DSEException
 */
protected void doDataMapping(Tag formatTag, Context sourceCtx, Context targetCtx) throws com.ibm.dse.base.DSEException
{
	DataMapperFormat format = null;
	try
	{	format = (DataMapperFormat) FormatElement.getExternalizer().convertTagToObject(formatTag);
		format.mapContents(sourceCtx, targetCtx) ;
		FormatExternalizer.putIntoCache(format);
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while mapping data between source and target contexes."+
				"Format = "+format+"; Source = "+sourceCtx+"; Target = "+targetCtx) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}				
		throw new DSEException(DSEException.critical, null, "Exception caught while mapping data between source and target contexes: " + e.getMessage()) ;
	}
}
/**
 * Serves as the entry point for executing the processor object
 * currently being managed by the processor operation. This method first 
 * ensures that consequentiality has not been violated for the processor object
 * being managed. If it has, execution is terminated. If not, this method passes control
 * over to the processor to carry on execution using the event
 * name supplied as argument to this method. Upon return of control from the
 * processor, the processor operation examines its status to determine the next
 * course of action. These are the possible course of action:
 * <ul>
 * <li> If the processor returns with an abort status, execution is terminated
 *		and an exception is thrown to indicate an abnormal exit.</li>
 * <li> If the processor returns with a running status, information is extracted
 *		from the current state of the processor to determine the appropriate response
 *		to be set by this method.</li>
 * <li> If the processor returns with a completed status, a check is made to see
 *		if it has a parent. If it does, execution is turned over to the parent using
 *		an event extracted from the sub-processor. If the processor did not have a
 *		parent, an appropriate response is set and this method exits.</li>
 * </ul>
 *
 * @exception com.ibm.dse.base.DSEException
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 */
public void execute(String eventName) throws com.ibm.dse.base.DSEException, com.ibm.dse.base.DSEInvalidArgumentException
{
	if(getProcessor() instanceof DSEHtmlProcessor)
	{
		DSEHtmlProcessor processor = (DSEHtmlProcessor) getProcessor() ;
	} else {
		DSEProcessor processor = (DSEProcessor) getProcessor();
	}


	// check status if an illegal switch has been made in the processor
	if(getStatus().equals(ILLEGALSWITCH))
	{
		if ( ((DSEHtmlProcessor) processor).getConseqErrorResponse() != null )
		{
			setResponseId( ((DSEHtmlProcessor) processor).getConseqErrorResponse() );
			closeProcessor(processor, true);
			setStatus(NORMAL);
			return;
		}
		else
		{
			// No conseqErrorPage defined, throw an exception			
			throw new com.ibm.dse.clientserver.DSECSInvalidRequestException( "A consequentiality error has occurred, but no conseqErrorPage has been found." );
		}
	}
    // Add for PMR27673 begin
	try {
    // PMR27673 end
		if(processor instanceof DSEHtmlProcessor)
		{
			eventName = getValidEventName(processor, null, eventName);
			//Treating the NLS 
			String shortName = eventName.substring(eventName.indexOf(".")+1);
			if(shortName.startsWith("dse_lang_")){
			
				String newLang = shortName.substring(9);
				updateLocale(newLang);
				String extra = (String)getProcessor().getContext().getValueAt(com.ibm.dse.cs.html.HtmlConstants.EXTRAPATHINFO);
				String jspPath = (String)getProcessor().getContext().getValueAt(com.ibm.dse.cs.html.HtmlConstants.JSPBASEPATH);
				if(extra != null){
					getProcessor().getContext().setValueAt(com.ibm.dse.cs.html.HtmlConstants.EXTRAPATHINFO, extra.substring(0,extra.length()-5) + newLang);
				} else {
					DataField df = new DataField();
					df.setName(com.ibm.dse.cs.html.HtmlConstants.EXTRAPATHINFO);
					df.setValue(newLang);
					getProcessor().getContext().addElement(df);
				}
				getProcessor().getContext().setValueAt(com.ibm.dse.cs.html.HtmlConstants.JSPBASEPATH, jspPath.substring(0,jspPath.length()-6) + newLang+"/");


			} else {
						
				((DSEHtmlProcessor)processor).execute(eventName) ;
				setApplicationId((String)getContext().getValueAt(HtmlConstants.APPLICATIONID));
			}
	
		} else {
			((DSEProcessor)processor).execute();
		}
    //Add for PMR 27673 begin. If exception occrues before it saved in processor info table, framework should take action to clean the proc entry in processor registry.
	} catch (DSEException e){  
		if (!isProcessorSaved())
			HtmlProcessorManager.getUniqueHTMLInstance().removeProcessor(processor.getProcessorId());
		throw e;	
	}
    //PMR27673 end

	// check status of processor after execution is complete
	String status = processor.getStatus() ;


	// if processor was aborted, then throw an exception
	if(status.equals(DSEHtmlProcessor.ABORTED))
	{
		closeProcessor(processor) ;
		throw new DSEHtmlAbortProcessorException(DSEException.critical, null, "Execution possibly failed: The process returned with status of abort.") ;
	}


	// if processor is done executing, then we have to check whether
	// it has a parent. If it does, execution will be continued in it.
	// If the processor has no parent, then we indicate that processing
	// is over for this business request


	else if(status.equals(DSEHtmlProcessor.COMPLETED))
	{
	  // if processor has parent, continue processing in it
		if(processorHasParent(processor))
		{
			Processor parent = getParent(processor);
			String exitEventName = getValidEventName( processor, parent, null);



			// map data from the child processor to the parent
			mapDataFromChild(processor, parent, exitEventName) ;

			/** <HSBC> **/
			// Store the action context for future mapping
			// if the subflow is completed
			if (parent instanceof InterStateFormatEnabledProcessor) {
				InterStateFormatSupport.storeActionCotext((InterStateFormatEnabledProcessor) parent, processor.getContext());
			}
			/** </HSBC> **/

			// close child processor
			closeProcessor(processor) ;


			// now we can resume execution in the parent
			setProcessor(parent) ;
			execute(exitEventName) ;
			return ;
		}


		// if processor has no parent, indicate end of processing condition
		else
		{
			// Set the page that the processor will display if it exists
			if (processor.getCurrentState() instanceof DSEHtmlState) 
			{
				setFinalResponseId( ((DSEHtmlState) processor.getCurrentState() ).getFinalPage() );
			}


			// clean up processor
			closeProcessor(processor);
		}
	  	}


	// if the processor has not finished running yet, then we obtain
	// the state type for the current state and use it to determine the
	// next step of action
	else
	{
		String stateType = processor.getCurrentState().getType() ;
		String stateInfo = processor.getCurrentState().getTypeIdInfo() ;


		if(stateType.equals(DSEState.PAGE_TYPE))
		{
			setResponseId(stateInfo);
		}
		else if(stateType.equals(DSEState.SUBFLOW_TYPE))
		{
			startSubProcessor(stateInfo) ;
			return ;
		}
		else throw new DSEException(DSEException.critical, null, "Execution ended with processor in an unsupported state.") ;


		// by this point, we should have a valid response id. We save the
		// current processor state and return
		/*if(!(getApplicationId().equals(HtmlConstants.IGNORE))){
			HtmlProcessorManager.getUniqueHTMLInstance().saveProcessor(getSessionId(), (DSEHtmlProcessor) getProcessor()) ;
		}else{
			HtmlProcessorManager.getUniqueHTMLInstance().saveProcessor(getSessionId(), (DSEHtmlProcessor) getProcessor()) ;
		}*/
		return ;
	}
}

/**
 * <HSBC>
 * Overrided the getContext() method to return the processor context
 */
public Context getContext(){
	if (processor != null) {
		return processor.getContext();
	} else {
		return super.getContext();
	}
}

/**
 * Obtains the parent processor, if it exists, for the processor
 * supplied as argument.
 * 
 * @return com.ibm.dse.automaton.Processor
 * @param processor com.ibm.dse.automaton.Processor
 */
protected Processor getParent(Processor processor)
{
	if(processor == null) return null ;


	Processor parent = null ;
	String parentId = null, parentSessionId = null, parentAppSessionId = null;
	try
	{
		if(processor instanceof DSEHtmlProcessor)
		{
			parentId = ((DSEHtmlProcessor)processor).getParentId() ;
			parentSessionId = ((DSEHtmlProcessor)processor).getParentSessionId();
			parentAppSessionId = ((DSEHtmlProcessor)processor).getParentAppSessionId();
		} else {
			parentId = (String)((DSEProcessor)processor).getContext().getValueAt("dse_parentId");
			parentSessionId = getSessionId();
			parentAppSessionId=  getApplicationId();
		}
		if (parentAppSessionId.equals(HtmlConstants.IGNORE))
			parent = HtmlProcessorManager.getUniqueHTMLInstance().loadProcessor(parentSessionId, parentId);
		else
			parent = HtmlProcessorManager.getUniqueHTMLInstance().loadProcessor(parentAppSessionId, parentId);
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "DSEHtmlProcessorOperation.getParent: An exception was caught while loading the parent processor for "
					+ processor.getProcessorId() + ". It is possible that this processor is not an HTML processor, or that it does not have a parent. An error might also have been encountered earlier"
					+ "in saving its parent processor.");
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;			
		}
	}


	return parent ;
}
/**
 * Returns the processor object currently being managed.
 *
 * @return com.ibm.dse.automaton.Processor
 */
public com.ibm.dse.automaton.Processor getProcessor()
{
	return processor ;
}
/**
 * Returns the current response ID existing in the toolkit.
 * 
 * @return java.lang.String
 * @exception com.ibm.dse.base.DSEException
 */
public String getResponseId() throws com.ibm.dse.base.DSEException
{
	try
	{
		return (String) getContext().getValueAt(HtmlConstants.REPLYPAGE) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while attempting to retrieve response id") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}				
		throw new DSEException(DSEException.critical, null, "Exception caught while attempting to retrieve response id: " + e.getMessage()) ;
	}
}
/**
 * Returns the status attributes of this object.
 * 
 * @return java.lang.String
 */
public java.lang.String getStatus() {
	return status;
}
/**
 * Returns a valid notifier name. The name of the notifier is very important because the transitions 
 * must be defined as <I>notifierName.eventName</I>. This method creates a dummy notifier name only 
 * when the event name does not include a notifier name. 
 * The convention is when the event comes from a page, the notifier 
 * is constructed using the name of the state. If the event
 * is the "start" event of the processor, the notifier is built using
 * the processor name. If the event comes from a subFlow's final state, the notifier is
 * built from the name of the subFlow.
 * 
 * @return java.lang.String
 * @param eventName java.lang.String
 * @param typePage boolean
 */
protected String getValidEventName(Processor currentProcessor, Processor parentProcessor, String currentEventName)
{
	if (currentEventName==null)
		currentEventName = (String) currentProcessor.getCurrentState().getTypeIdInfo();
	String eventName;
	int dotIndex = currentEventName.indexOf(".");
	if (dotIndex == -1)
	{
		String notifierName;
		if (parentProcessor==null)
			notifierName = currentProcessor.getCurrentState().getName();
		else
			notifierName = parentProcessor.getCurrentState().getName();
	eventName = notifierName + "." + currentEventName;
	}
	else eventName=currentEventName;


	return eventName;
}
/**
 * This utility method attempts to obtain the value corresponding to the supplied
 * name from the supplied context. If the value cannot be obtained and the boolean
 * <i>required</> attribute is true, an exception is generated. If the attribute is false,
 * this method returns a null.
 * 
 * @return java.lang.Object
 * @param context com.ibm.dse.base.Context
 * @param name java.lang.String
 * @param required boolean
 * @exception com.ibm.dse.base.DSEException
 */
protected Object getValueFromContext(Context context, String name, boolean required) throws com.ibm.dse.base.DSEException
{
	try
	{
		return context.getValueAt(name) ;
	}
	catch(Exception e)
	{
		if(required) 
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Value "+name+" not found in context: "+context) ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}					
			throw new DSEException(DSEException.critical, null, "Exception caught while retrieving value from context: " + e.getMessage()) ;}
	}


	return null ;
}
/**
 * This utility method attempts to obtain the value corresponding to the supplied
 * name from the supplied collection. If the value cannot be obtained and the boolean
 * <i>required</> attribute is true, an exception is generated. If the attribute is false,
 * this method returns a null.
 * 
 * @return java.lang.Object
 * @param coll com.ibm.dse.base.KeyedCollection
 * @param name java.lang.String
 * @param required boolean
 * @exception com.ibm.dse.base.DSEException
 */
protected Object getValueFromKColl(KeyedCollection coll, String name, boolean required) throws com.ibm.dse.base.DSEException
{
	try
	{
		return coll.getValueAt(name) ;
	}
	catch(Exception e)
	{
		if(required)
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Value "+name+" not found in data collection: "+coll) ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}					
			throw new DSEException(DSEException.critical, null, "Exception caught while retrieving required value ( " + name + " ) from context: " + e.getMessage()) ;}
	}


	return null ;
}
/**
 * Creates or restores a processor object. The data required to
 * carry this out is supplied within the collection supplied
 * as argument to the method:<ul>
 *
 * <li>processor ID		- keyed by HtmlConstants.PROCESSORID, <b>or</b> </li>
 * <li>processor name	- keyed by HtmlConstants.OPERATIONNAME</li>
 * <li>processor state	- keyed by HtmlConstants.PROCESSORSTATE</li>
 *
 * <p>If a processor id is supplied, this method makes an attempt to load the processor
 * from storage. If the processor ID is not supplied and the processor name is,
 * this method creates a new processor object corresponding to the name. An error
 * results if neither the processor ID nor processor name are supplied. Once the method
 * obtains a processor object, its state is changed to the one specified
 * in the keyed collection.
 *
 * @param coll com.ibm.dse.base.KeyedCollection
 * @exception com.ibm.dse.base.DSEException
 */
public void initializeProcessor(KeyedCollection coll) throws com.ibm.dse.base.DSEException
{
	// set the session id if this has not already been done
	try
	{
		if(isEmpty(getSessionId()))
		{
			setSessionId((String)coll.getValueAt(HtmlConstants.SESSIONID)) ;
		}


		setApplicationId(HtmlConstants.IGNORE);		
		setApplicationId((String)coll.getValueAt(HtmlConstants.APPLICATIONID));
		
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "SessionId or ApplicationId not present in data collection: "+coll) ;
		}
	}


	// get default values for processor
	String processorId = (String) getValueFromKColl(coll, HtmlConstants.PROCESSORID, false) ;
	String processorName = (String) getValueFromKColl(coll, HtmlConstants.OPERATIONNAME, false) ;


	// check the supplied processor id. If none has been supplied, then we need
	// to create a new processor object. Otherwise we restore a processor from
	// persistent data using the processor id as key.
	com.ibm.dse.automaton.Processor processor = null ;


	if(isEmpty(processorId))
	{
		processor = createProcessor(processorName) ;
		//Add for PMR27673 begin, set processorSaved false
		setProcessorSaved(false);	
		// PMR27673 end	
	}
	else
	{
		try
		{
			if((!(getApplicationId().equals(HtmlConstants.IGNORE)))){
				processor = HtmlProcessorManager.getUniqueHTMLInstance().loadProcessor(getApplicationId(), processorId) ;
			}else{
				processor = HtmlProcessorManager.getUniqueHTMLInstance().loadProcessor(getSessionId(), processorId) ;
			}
		}
		catch(DSEException dsee)
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Unable to load processor, given processor id (" + processorId + "): ") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), dsee) ;							
			}


			throw dsee ;
		}
	}


	String processorState = (String) getValueFromKColl(coll, HtmlConstants.PROCESSORSTATE, false) ;
	if (processorState == null) processorState = "initial";


	// change the state of the processor to one defined by the caller. A check is
	// made first for back-clicks.


	// get the present and last processed page id's
	String pageId = "0", lastPageId = "0" ;
	try
	{
		pageId = (String) coll.getValueAt(HtmlConstants.PAGEID) ;
		if(!(getApplicationId().equals(HtmlConstants.IGNORE))){
			lastPageId = (String) Context.getCurrentContextForSession(getSessionId(),getApplicationId()).getValueAt(HtmlConstants.PAGEID) ;
		}else{
			lastPageId = (String) Context.getCurrentContextForSession(getSessionId()).getValueAt(HtmlConstants.PAGEID) ;
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Information, Settings.getTID(), "Could not obtain current and/or last page id processed. Using default of 0.");
		}
	}


	if(checkBackClick(pageId, lastPageId) && (checkLegalSwitch(processor, processorState) == false))
	{
		setStatus(ILLEGALSWITCH) ;
	}
	else
	{
		if(! processor.getCurrentState().getName().equals(processorState))
		{
			processor.changeState(processorState, false) ;
		}
		/** <HSBC> **/
		else if (((DSEHtmlState) processor.getCurrentState()).getContext() != null) {
			// When reloading the process, replace context in state with
			// the current state context under the processor context
			String stateCtxtName = ((DSEHtmlState) processor.getCurrentState()).getContext().getName();
			Enumeration enum = ((DSEHtmlProcessor) processor).getProcessorContext().getChildren().elements();
			Context child = null;
			while (enum.hasMoreElements()) {
				child = (Context) enum.nextElement();
				if (child.getName().equals(stateCtxtName)) {
					((DSEHtmlState) processor.getCurrentState()).setContext(child);
					break;
				}
			}
		}
		/** </HSBC> **/
		
	}


	// set the processors current state to ignore return type
	try
	{
		((DSEHtmlState)processor.getCurrentState()).setIgnoreReturnType(true) ;
	}
	catch(Exception e) {
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when setting ignoreReturnType to processor current state") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}				
		}


	setProcessor(processor) ;
}
/**
 * Checks for an empty string by first checking if the string is null
 * and then checking if it contains any characters. If the boolean argument
 * is true, the method trims space characters from the beginning and
 * end of the string before it is checked for characters. Trimming the
 * string is the default behavior.
 * 
 * @return boolean
 * @param s java.lang.String
 */
protected boolean isEmpty(String s)
{
	return isEmpty(s, true) ;
}
/**
 * Checks for an empty string by first checking if the string is null
 * then checking if it contains any characters. If the boolean argument
 * is true, the method trims the space characters from the beginning and
 * end of the string before it is checked for characters. Trimming the
 * string is the default behavior.
 * 
 * @return boolean
 * @param string java.lang.String
 * @param trim boolean
 */
protected boolean isEmpty(String string, boolean trim)
{
	if(string == null) return true ;


	if(trim == true) return string.trim().equals("") ;
	else return string.equals("") ;
}
/**
 * This utility method maps data from a child processor to its parent
 * upon the child processor's completion.
 * 
 * @param child com.ibm.dse.automaton.Processor
 * @param parent com.ibm.dse.automaton.Processor
 * @param exitEventName java.lang.String
 * @exception com.ibm.dse.base.DSEException
 */
protected void mapDataFromChild(Processor child, Processor parent, String exitEventName) throws com.ibm.dse.base.DSEException
{
	Tag formatTag = null ;


	// first try and get a format definition from the transition corresponding
	// to the supplied exit event name
	try
	{
		TransitionDescriptor transition = (TransitionDescriptor) parent.getCurrentState().getTransitions().get(exitEventName) ;
		formatTag = ((HtmlTransitionDescriptor) transition).getOutputFormatTag() ;
	}
	catch(ClassCastException ignored){}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "An exception was caught while mapping data from the child processor to the parent processor: ") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
		throw new DSEException(DSEException.critical, null, "Exception caught while mapping data from the child processor to the parent processor: " + e.getMessage()) ;
	}


	// if no format was defined in the transition, we attempt to obtain one from
	// from the current state of the parent
	try
	{
		if(formatTag == null)
		{
			formatTag = ((DSEHtmlState) parent.getCurrentState()).getOutputFormatTag();
		}
	}
	catch(ClassCastException ignored) {}
	catch (Exception e)
	{
		if (Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "An exception was caught while getting the mapping format to be applied from the child processor to the parent processor");
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
	}


	// map the data
	try
	{
		if(formatTag != null)
		{
			doDataMapping(formatTag, child.getContext(), parent.getContext());
		}
	}
	catch(Exception e)
	{
		if (Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "An exception was caught while mapping data from the child processor to the parent processor: ");
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
		throw new DSEException(DSEException.critical, null, "Exception caught while mapping data from the child processor to the parent processor: " + e.getMessage()) ;
	}
}
/**
 * Checks to see if the processor supplied as argument has a parent.
 * 
 * @return boolean
 * @param processor com.ibm.dse.automaton.html.DSEHtmlProcessor
 */
protected boolean processorHasParent(DSEHtmlProcessor processor)
{
	String parentId = processor.getParentId() ;


	if(!isEmpty(parentId)) return true ;
	else return false ;
}
/**
 * Checks to see if the processor supplied as argument has a parent.
 * 
 * @return boolean
 * @param processor com.ibm.dse.automaton.html.DSEHtmlProcessor
 */
protected boolean processorHasParent(Processor processor)
{
	String parentId = null;
	
	if(processor instanceof DSEHtmlProcessor){
		parentId = ((DSEHtmlProcessor)processor).getParentId() ;
	} else {
		try {
			parentId = (String)((DSEProcessor)processor).getContext().getValueAt("dse_parentId");
		}
		catch(Exception e)
		{}
	}


	if(!isEmpty(parentId)) return true ;
	else return false ;
}
/**
 * Gets the transition context from the processor context (which is only stored there if there is a 
 * skipValidation transition), chains the transition context to the processor context, and removes the 
 * transition context from the processor context. This method is used only when a HtmlTransition 
 * with a true skipValidation value is executed. 
 *
 * @return boolean
 */
public boolean replaceCtxIfNeeded()
{
	try
	{
		// Get the trCtx, stored in the procCtx if exists
		Context trCtx = (Context) getContext().getValueAt("dse_trCtx");
		// Chain the trCtx to the procCtx
		trCtx.chainTo( this.getContext() );
		// Remove the trCtx from the procCtx (this reference will not be needed any longer)
		getContext().removeAt("dse_trCtx");
		// Perform the real replacement of the contexts
		setContext(trCtx);


		return true;
	}
	catch (DSEObjectNotFoundException e)
	{
		// Just return false, there is no need to replace the context
		return false;
	}
	catch (DSEInvalidRequestException e)
	{
		// This exception only can be thrown from the chainTo method. Means that the
		// trCtx is the root context or that it has already a parent ctx. In both cases,
		// the transition is bad defined as skipValidation, so stop the process.
		return false;
	}
}
/**
 * Sets the reply page. This method does not need to add any automaton-specific information
 * because the processor is ended. If the application wants to begin a new
 * processor from this page, it must add the state field itself.
 * 
 * @param id java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 */
public void setFinalResponseId(String id) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.base.DSEObjectNotFoundException, DSEInvalidRequestException
{
	getContext().removeAt(HtmlConstants.PROCESSORID);
	getContext().removeAt(HtmlConstants.PROCESSORSTATE);
	if ( id == null )
	{
		// If no id is passed, the default final page is used.
		try
		{
			getContext().setValueAt(HtmlConstants.REPLYPAGE, (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFINALPAGEKEY) );
		}
		// If there is no default final page defined, the home page is used
		catch (DSEObjectNotFoundException e)
		{
			getContext().setValueAt(HtmlConstants.REPLYPAGE, (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSHOMEPAGEKEY) );
		}
	}
	else
	{
		getContext().setValueAt(HtmlConstants.REPLYPAGE, id);
	}
}
/**
 * Sets the processor object to be managed.
 *
 * @param processor com.ibm.dse.automaton.Processor
 */
public void setProcessor(com.ibm.dse.automaton.Processor processor)
{
	this.processor = processor ;
	if(this.processor != null) setContext(this.processor.getContext()) ;
}
/**
 * Sets the response ID to be set into the HTML Client toolkit.
 * 
 * @param id java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 */
public void setResponseId(String id) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.base.DSEObjectNotFoundException
{
	// response id is set by setting a value for reply page
	getContext().setValueAt(HtmlConstants.REPLYPAGE, id) ;
	getContext().setValueAt(HtmlConstants.PROCESSORSTATE, getProcessor().getCurrentState().getName()) ;
	getContext().setValueAt(HtmlConstants.PROCESSORID, getProcessor().getProcessorId()) ;
}
/**
 * Sets the status of this object.
 * 
 * @param newStatus java.lang.String
 */
public void setStatus(java.lang.String newStatus) {
	status = newStatus;
}
/**
 * This is a utility method for creating and executing a subprocessor.
 * A processor object is created and initialized based on the processor
 * name supplied as argument to this function. The current processor being
 * managed by the processor operation is saved, then replaced with the sub
 * processor. Data is optionally passed from the parent processor to the
 * subprocessor using data mapping formats defined in the current state of
 * the parent processor. Execution is then passed off to the subprocessor,
 * with an event that corresponds to its start state.
 * 
 * @param processName java.lang.String
 * @exception com.ibm.dse.base.DSEException
 */
public void startSubProcessor(String processorName) throws com.ibm.dse.base.DSEException
{
	Processor parent = getProcessor() ;


	// create a sub-processor
	Processor child = createSubProcessor(processorName, parent) ;


	// copy required fields from parent context to child context
	try
	{
		Context parentContext = parent.getContext() ;
		Context childContext = child.getContext() ;


		String sessId = (String) parentContext.getValueAt(HtmlConstants.SESSIONID) ;
		DataField sessionid = new DataField() ;
		sessionid.setName(HtmlConstants.SESSIONID) ;
		sessionid.setValue(sessId) ;
		childContext.addElement(sessionid) ;


		String applId = (String) parentContext.getValueAt(HtmlConstants.APPLICATIONID) ;
		DataField applicationId = new DataField() ;
		applicationId.setName(HtmlConstants.APPLICATIONID) ;
		applicationId.setValue(applId) ;
		childContext.addElement(applicationId) ;		


		DataField replypageid = new DataField() ;
		replypageid.setName(HtmlConstants.REPLYPAGE) ;
		childContext.addElement(replypageid) ;


		DataField procid = new DataField() ;
		procid.setName(HtmlConstants.PROCESSORID) ;
		procid.setValue(child.getProcessorId()) ;
		childContext.addElement(procid) ;


		DataField state = new DataField() ;
		state.setName(HtmlConstants.PROCESSORSTATE) ;
		state.setValue(child.getCurrentState().getName()) ;
		childContext.addElement(state) ;


		if(!(child instanceof DSEHtmlProcessor))
		{
			DataField parentId = new DataField();
			parentId.setName("dse_parentId");
			parentId.setValue(parent.getProcessorId());
			childContext.addElement(parentId);
		}
		
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "An error occured while copying system data between the parent and child processors: ") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
		throw new DSEException(DSEException.critical, null, "Exception caught while copying system data from the parent processor to the child processor: " + e.getMessage()) ;
	}
	// map data from the parent processor to the sub-processor
	try
	{
		if(((DSEHtmlState) parent.getCurrentState()).getInputFormatTag() != null)
		{ DataMapperFormat format = (DataMapperFormat) FormatElement.getExternalizer().convertTagToObject(((DSEHtmlState) parent.getCurrentState()).getInputFormatTag());
		  if(format != null)
		  {  doDataMapping(format, parent.getContext(), child.getContext()) ;
			 FormatExternalizer.putIntoCache(format);				
		  }
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "An error occured while mapping data between the parent and child processors: ") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
		throw new DSEException(DSEException.critical, null, "Exception caught while mapping data from the parent processor to the child processor: " + e.getMessage()) ;
	}

	/** <HSBC> **/
	// Map data from previous action context to this action context
	// and clear previous action context and format afterward
	if (parent instanceof InterStateFormatEnabledProcessor) {
		InterStateFormatSupport.startFormat((InterStateFormatEnabledProcessor) parent, child.getContext());
		InterStateFormatSupport.clearInfoForFormat((InterStateFormatEnabledProcessor) parent);
	}
	/** </HSBC> **/

	// save the current processor
	if(!(getApplicationId().equals(HtmlConstants.IGNORE))){
			HtmlProcessorManager.getUniqueHTMLInstance().saveProcessor(getApplicationId(), (DSEHtmlProcessor) getProcessor()) ;
		}else{
			HtmlProcessorManager.getUniqueHTMLInstance().saveProcessor(getSessionId(), (DSEHtmlProcessor) getProcessor()) ;
		}


	// Put the processor into the cache to be reused by other requests
	try	{com.ibm.dse.automaton.ProcessorExternalizer.putIntoCache((DSEHtmlProcessor) getProcessor());}
	catch (java.io.IOException ioe)
	{	if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "It has not been possible to store the processor in cache: " + ioe.toString());
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), ioe);
		}		
	}						


	// swap current processor for the sub processor
	setProcessor(child) ;


	execute("start") ;
}
/**
 * Requests that the processor (which this processor operation is
 * handling) update its context with the data supplied in the collection supplied
 * as argument. It returns a boolean indicating whether the update was successful
 * or not.
 * Before updating the context, this method checks if the request is legal according to  
 * consequentiality (back button support).
 *
 * @return boolean
 * @exception com.ibm.dse.base.types.DSETypeException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 */
public boolean updateContext(com.ibm.dse.base.KeyedCollection collection) throws com.ibm.dse.base.types.DSETypeException, DSEObjectNotFoundException, DSEInvalidArgumentException
{
	// If the switch is not legal, don't update the context.
	if( getStatus().equals(ILLEGALSWITCH) )
		return true; // Just go on. The proc will not be executed.
	
	String eventName = (String) collection.getValueAt(HtmlConstants.NEXTEVENTNAME);
	eventName = eventName.substring(eventName.indexOf(".")+1);
	
	String sessionID = (String) collection.getValueAt(HtmlConstants.SESSIONID);
	
	if(!eventName.startsWith("dse_lang_")) {
		return ((DSEHtmlProcessor) getProcessor()).updateContext(collection, sessionID) ;
	} else {
		return true;
	}
	
}
/**
 * Requests that the processor (which this processor operation is
 * handling) update its context with the data supplied in the collection supplied
 * as argument. It returns a boolean indicating whether the update was successful
 * or not.
 * Before updating the context, this methodchecks if the request is legal according to  
 * consequentiality (back button support).
 * @param kcReqData com.ibm.dse.base.KeyedCollection
 * @param convType java.lang.String 
 * @return boolean
 * @exception com.ibm.dse.base.types.DSETypeException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 */
public boolean updateContext(com.ibm.dse.base.KeyedCollection collection, String convType) throws com.ibm.dse.base.types.DSETypeException, DSEObjectNotFoundException, DSEInvalidArgumentException
{
	// This protocol is used by the HtmlRequestHandler to use the "html" conversion type for the input data
	// This class is internally using the hardcoded "html" conversion type in DSEHtmlProcessor.
	return updateContext(collection);
}
public void updateLocale(String localeString) throws DSEObjectNotFoundException,DSEInvalidArgumentException{
	
	//This method will update the locale with the info passed
	//The string should of the style EN_en
	getProcessor().getContext().setValueAt(HtmlConstants.LOCALE,new java.util.Locale(localeString.substring(0,2), localeString.substring(3,5)));
}
// Add for PMR27673 begin
	/**
	 * Returns the processorSaved.
	 * @return boolean
	 */
public boolean isProcessorSaved() {
		return ProcessorSaved;
	}

	/**
	 * Sets the processorSaved.
	 * @param processorSaved The processorSaved to set
	 */
public void setProcessorSaved(boolean processorSaved) {
		ProcessorSaved = processorSaved;
	}
// PMR27673 end
}
