package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import javax.servlet.http.HttpSession;
import com.ibm.dse.base.*;
import com.ibm.dse.automaton.*;
import com.ibm.dse.cs.html.*;
/**
 * This class manages creating, loading, and deleting of HTML processors.
 * HTML processors can be persisted to and restored from storage and this
 * class provides support for doing this. It also provides the traditional
 * support for creating processors.
 *
 * @copyright(c) Copyright IBM Corporation 2000,2003
 */
public class HtmlProcessorManager extends ProcessorManager
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2003 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	private static HtmlProcessorManager instance = null;
	/**
	 * Static object used for synchronizing the access to the "instance" attribute 
	 * in the getUniqueHTMLInstance method.
	 **/
	protected static java.lang.Object lockObject = new Object();
	/**
	 * Indicates whether the server configuration features session affinity.
	 * This attribute is specifical for HttpSession persistence.
	 */
	protected boolean sessionAffinity;
	/**
	 * Indicates whether the server is configured to work under persistence
	 * conditions.
	 */
	protected boolean sessionPersistence;
	
	/** <HSBC> **/
	public static String STATE_CONTEXT_ID_SUBFIX = "_STATE";
	/** </HSBC> **/

/**
 * Constructor for the HtmlProcessorManager class. Sets the values
 * for sesison persistence.
 */
public HtmlProcessorManager() 
{
	super();
	setSessionPersistence();
	setSessionAffinity();
}
/**
 * Returns the context table for the session ID supplied as argument.
 * 
 * @return com.ibm.dse.base.Vector
 * @param sessionId java.lang.String
 */
public static java.util.Hashtable getContextTable(String sessionId)
{
	java.util.Hashtable contextTable = null ;

	try
	{
		contextTable = (java.util.Hashtable) Context.getCurrentContextForSessionOrApplication(sessionId).getValueAt("contextTable") ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Low, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Low, Trace.Information, Settings.getTID(), "Context table for session "+sessionId+" not found. Creating a new one") ;
		}		
		contextTable = new java.util.Hashtable(3) ;
		try
		{
			DataElement element = new DataField() ;
			element.setName("contextTable") ;
			element.setValue(contextTable) ;
			Context.getCurrentContextForSessionOrApplication(sessionId).addElement(element) ;
		}
		catch(Exception e2)
		{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when adding the context table to the session: "+sessionId) ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e2) ;
			}			
		}
	}

	return contextTable ;
}
/**
 * Retrieves the persistent data for the supplied processor ID.
 * 
 * @return com.ibm.dse.automaton.html.HtmlProcessorInfo
 * @param processorId java.lang.String
 */
public com.ibm.dse.automaton.html.HtmlProcessorInfo getInfo(String sessionId, String processorId)
{
	HtmlProcessorInfo info = null ;

	if (getSessionPersistence())
	{
		info = getPersistentInfo(sessionId, processorId);
	}
	else
	{
		try
		{
			java.util.Hashtable procTable = getProcessorInfoTable(sessionId) ;
			info = (HtmlProcessorInfo) procTable.get(processorId) ;
		}
		catch(Exception e)
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getInfo: No persistent data could be loaded for given processor id: " + processorId) ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;			
			}
		}
	}
	return info ;
}
/**
 * Retrieves the persistent context for the supplied session ID and context ID.
 * 
 * @return com.ibm.dse.automaton.html.HtmlProcessorInfo
 * @param processorId java.lang.String
 */
public Context getPersistentContext(String sessionId, String contextId)
{
	Context procCtx = null ;

	try
	{
		HttpSession sessionObject = ( HttpSession ) Context.getSessionObjectForSession( sessionId );
		procCtx = ( Context ) sessionObject.getAttribute( contextId );

	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getContext: No persistent context could be loaded for given session id: " + sessionId + " and context id: " + contextId);
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e);
		}
	}

	return procCtx;
}

/**
 * <HSBC>
 * Retrieves the persistent state context for the supplied session ID and statae context tag ID.
 * 
 * @return Context
 * @param processorId java.lang.String
 */
public Context getPersistentStateContext(String sessionId, String stateContextTagID)
{
	Context stateContext = null;

	try
	{
		HttpSession sessionObject = ( HttpSession ) Context.getSessionObjectForSession( sessionId );
		Object tagOrContext = sessionObject.getAttribute( stateContextTagID );
		
		if (tagOrContext != null) {
			if (tagOrContext instanceof Tag) {
				stateContext = (Context) Context.getExternalizer().convertTagToObject((Tag) tagOrContext);
			} else {
				stateContext = (Context) tagOrContext;
			}
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getStateContextTag: No persistent state context tag could be loaded for given session id: " + sessionId + " and state context tag id: " + stateContextTagID);
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e);
		}
	}

	return stateContext;
}

/**
 * Retrieves the persistent data for the supplied processor ID.
 * It is specific for a session persistent environment. Takes the data from
 * the session object (DB) instead of memory.
 * 
 * @return com.ibm.dse.automaton.html.HtmlProcessorInfo
 * @param processorId java.lang.String
 */
public com.ibm.dse.automaton.html.HtmlProcessorInfo getPersistentInfo(String sessionId, String processorId)
{
	HtmlProcessorInfo info = null ;

	try
	{
		HttpSession sessionObject = ( HttpSession ) Context.getSessionObjectForSession( sessionId );
		//java.util.Hashtable procTable = getProcessorInfoTable(sessionId) ;
		info = ( HtmlProcessorInfo ) sessionObject.getAttribute( processorId );
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getInfo: No persistent data could be loaded for given processor id: " + processorId) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;			
		}
	}
	return info ;
}
/**
 * Returns a subSet of the processor registry that contains 
 * all the processors which are running under a given session.
 * 
 * @return java.util.Hashtable
 * @param sessionId java.lang.String
 */
public java.util.Vector getProcessorIdsInSession(String sessionId)
{
	java.util.Vector procsInSession = new java.util.Vector();

	if ( !getProcessorInfoTable(sessionId).isEmpty() )
	{
		// Each session could have associated several different processors.
		java.util.Enumeration processorsId = getProcessorInfoTable(sessionId).keys();
		while (processorsId.hasMoreElements())
		{
			// Add the processorId to the vector.
			procsInSession.addElement((String) processorsId.nextElement());
		}
	}
	return procsInSession;
}
/**
 * Returns the processor info table for the session ID supplied as argument.
 * 
 * @return com.ibm.dse.base.Vector
 * @param sessionId java.lang.String
 */
public static java.util.Hashtable getProcessorInfoTable(String sessionId)
{
	java.util.Hashtable procInfoTable = null ;

	try
	{
		procInfoTable = (java.util.Hashtable) Context.getCurrentContextForSessionOrApplication(sessionId).getValueAt(HtmlConstants.PROCINFOTABLE) ;
	}
	catch(Exception e) 
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Low, Trace.Information))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Low, Trace.Information, Settings.getTID(), "Processors info table for session "+sessionId+" not found. Creating a new one") ;
		}				
		procInfoTable = new java.util.Hashtable(8) ;
		try
		{
			DataElement element = new DataField() ;
			element.setName(HtmlConstants.PROCINFOTABLE) ;
			element.setValue(procInfoTable) ;
			Context.getCurrentContextForSessionOrApplication(sessionId).addElement(element) ;			
		}
		catch(Exception e2)
		{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when adding the processors info table to the session: "+sessionId) ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e2) ;
			}						
		}
	}
	return procInfoTable ;
}
/**
 * Returns the processor info table for the session ID supplied as argument.
 * 
 * @return com.ibm.dse.base.Vector
 * @param sessionId java.lang.String
 */
public static java.util.Hashtable getProcessorInfoTableForCheckSession(String sessionId)
{
	java.util.Hashtable procInfoTable = null ;

	try
	{
		procInfoTable = (java.util.Hashtable) Context.getCurrentContextForSessionOrApplication(sessionId).getValueAt(HtmlConstants.PROCINFOTABLE) ;
	}
	catch(Exception e) 
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Low, Trace.Debug))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Low, Trace.Debug, Settings.getTID(), "Processors info table for session "+sessionId+" not found.") ;
		}				
		
	}
	return procInfoTable ;
}
/**
 * Returns the value of the <I>sessionAffinity</I> property.
 * This method is specific for HttpSession persistence.
 * @return boolean
 */
public boolean getSessionAffinity(){
	return sessionAffinity;
}
/**
 * Returns the value of the <I>sessionPersistence</I> property.
 * @return boolean
 */
public boolean getSessionPersistence() {
	return sessionPersistence;
}
/**
 * Returns a singleton instance for the HTML Processor
 * Manager. It should be called instead of getUniqueInstance(), which
 * returns a singleton for the the superclass.
 * 
 * @return com.ibm.dse.automaton.ProcessorManager
 */
public static HtmlProcessorManager getUniqueHTMLInstance()
{
	// First time this method is called, the singleton is created
	if(instance == null)
	{
		// Ensure that only one instance will be created
		synchronized (lockObject)
		{
			if (instance == null) instance = new HtmlProcessorManager();
		}
	}

	return instance ;
}
/**
 * Restores a virtual processor object. The supplied processor ID
 * is used to obtain the stored version of the processor object. This data is
 * then used to recreate the processor object. If the data does not exist for the
 * supplied processor ID, this method returns a null object.
 *
 * @return com.ibm.dse.automaton.html.DSEHtmlProcessor
 * @param processId java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException.
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 */
public com.ibm.dse.automaton.html.DSEHtmlProcessor loadProcessor(String sessionId, String processorId) throws DSEInvalidArgumentException, DSEObjectNotFoundException
{
	if ( getSessionPersistence() ) return loadProcessorPersistent( sessionId, processorId );
	else return loadProcessorNonPersistent( sessionId, processorId );
}
/**
 * Restores the virtual processor object associated with the session ID 
 * from the tables stored in memory.
 *
 * @return com.ibm.dse.automaton.html.DSEHtmlProcessor
 * @param processId java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException.
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 */
public com.ibm.dse.automaton.html.DSEHtmlProcessor loadProcessorNonPersistent(String sessionId, String processorId) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.base.DSEObjectNotFoundException
{
	DSEHtmlProcessor processor = null ;
	HtmlProcessorInfo info = null;

	// create and initialize the processor from saved information
	try
	{
		info = getInfo(sessionId, processorId);

		if ( info == null )
			throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve processor information for sessionId: " + sessionId + " and processorId: "+ processorId );
		
		String processorName = info.getProcessorName();

		processor = (DSEHtmlProcessor) createProcessorFromCache(processorName);
		// Consider this processor as a running processor, so add it to the registry
		processorRegistry.put(processorId, processor);

		processor.setName(processorName) ;
		processor.setProcessorId(processorId) ;
		processor.setStatus(info.getProcessorStatus()) ;
		processor.setParentId(info.getParentId()) ;
		// These two attributes will identify the parent if the child proc changes the link to a different appSession
		processor.setParentSessionId(info.getParentSessionId());
		processor.setParentAppSessionId(info.getParentAppSessionId());

		HtmlStateInfo stateInfo = info.getCurrentStateInfo() ;
		processor.changeState(stateInfo.getStateName(), stateInfo.isIgnoresEntryActions()) ;
		try
		{
			((DSEHtmlState) processor.getCurrentState()).setIgnoreReturnType(stateInfo.isIgnoresReturnType()) ;
		}
		catch(Exception e) {
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when updating the ignoreReturnType for the processor state: "+processor) ;
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
				}
			}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while attempting to load processor for given process id: " + processorId) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
		}
		throw new DSEInvalidArgumentException(DSEException.critical, null, "Exception caught while attempting to load processor for given process id: " + processorId + ": " + e.toString()) ;
	}

	// once we have loaded the processor, we have to set its context
	Context context = null ;
	try
	{
		java.util.Hashtable contextTable = getContextTable(sessionId) ;
		context = (Context) contextTable.get(processorId) ;
		processor.setContext(context) ;
		
		/** <HSBC> **/
		Object tagOrContext = contextTable.get(processorId + STATE_CONTEXT_ID_SUBFIX);
		if (tagOrContext != null) {		
			Context stateContext = null;
			
			if (tagOrContext instanceof Tag) {
				// Try to load the state context tag and recreate the state context
				stateContext = (Context) Context.getExternalizer().convertTagToObject((Tag) tagOrContext);
			} else {
				stateContext = (Context) tagOrContext;
			}
						
			stateContext.chainTo(context);
			// clear the entry from table after retrieve
			contextTable.remove(processorId + STATE_CONTEXT_ID_SUBFIX);
		}
		/** </HSBC> **/
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "The processor was successfully loaded, but an error was encountered while setting its context.") ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
		}
	}

	// chain the processor to the session context
	try
	{
		//**<HSBC> DSEHtmlProcessorOperation.chainToSessionContext(processor.getContext(), sessionId) ;
		DSEHtmlProcessorOperation.chainToSessionContext(processor.getProcessorContext(), sessionId) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "Could not chain processor to session context. Ensure that the processor context" +
				" definition in the external files is not chained. Processor context = "+processor.getContext()) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), e) ;
		}
		throw new DSEInvalidArgumentException(DSEException.critical, null, "Exception caught while loading processor. Could not chain processor to session context: " + e.toString()) ;
	}
	//updating the time-stamp
	info.setTimeStamp(new Long(System.currentTimeMillis()));
	return processor ;
}
/**
 * Restores a virtual processor object. The info is
 * obtained from the session object (DB) instead of memory.
 *
 * @return com.ibm.dse.automaton.html.DSEHtmlProcessor
 * @param processId java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException.
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 */
public com.ibm.dse.automaton.html.DSEHtmlProcessor loadProcessorPersistent(String sessionId, String processorId) throws DSEInvalidArgumentException, DSEObjectNotFoundException
{
	DSEHtmlProcessor processor = null ;
	HtmlProcessorInfo info = getPersistentInfo(sessionId, processorId);

	if ( info == null )
		throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve processor information for sessionId: " + sessionId + " and processorId: "+ processorId );

	// create and initialize the processor from saved information
	try
	{
		String processorName = info.getProcessorName();

		processor = (DSEHtmlProcessor) createProcessorFromCache(processorName) ;
		// Consider this processor as a running processor, so add it to the registry
		processorRegistry.put(processorId, processor);

		processor.setName(processorName) ;
		processor.setProcessorId(processorId) ;
		processor.setStatus(info.getProcessorStatus()) ;
		processor.setParentId(info.getParentId()) ;
		// These two attributes will identify the parent if the child proc changes the link to a different appSession
		processor.setParentSessionId(info.getParentSessionId());
		processor.setParentAppSessionId(info.getParentAppSessionId());

		HtmlStateInfo stateInfo = info.getCurrentStateInfo() ;
		processor.changeState(stateInfo.getStateName(), stateInfo.isIgnoresEntryActions()) ;
		try
		{
			((DSEHtmlState) processor.getCurrentState()).setIgnoreReturnType(stateInfo.isIgnoresReturnType()) ;
		}
		catch(Exception e) {
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
				{
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when updating the ignoreReturnType for the processor state: "+processor) ;
					Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
				}
			}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while attempting to load processor for given process id: " + processorId) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
		}
		throw new DSEInvalidArgumentException(DSEException.critical, null, "Exception caught while attempting to load processor for given process id: " + processorId + ": " + e.toString()) ;
	}

	// once the processor is loaded, need to set its context
	Context context = getPersistentContext( sessionId, info.getContextId() );
	if ( context == null )
		throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve a context for sessionId: " + sessionId + " and processorId: "+ processorId );

	/** <HSBC> **/
	// clear the entry from table after retrieve
	//**** removePersistentContext( sessionId, info.getContextId());
	
	// Try to load the state context 
	Context stateContext = getPersistentStateContext(sessionId, processorId + STATE_CONTEXT_ID_SUBFIX);
	if (stateContext != null) {
		try {
			
			stateContext.chainTo(context);
			
		} catch (Exception e) {
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug)){
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "Could not chain state contect to processor context.  State Context name: " + stateContext.getName()) ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), e) ;
			}
			throw new DSEInvalidArgumentException(DSEException.critical, null, "Exception caught while chaining state context to processor context: " + e.toString()) ;
		}
		// clear the entry from table after retrieve
	    removePersistentStateContextTag(sessionId, processorId + STATE_CONTEXT_ID_SUBFIX);
	}
	/** </HSBC> **/

	processor.setContext( context );

	// chain the processor to the session context
	try
	{
		DSEHtmlProcessorOperation.chainToSessionContext(processor.getContext(), sessionId) ;
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), "Could not chain processor to session context. Ensure that the processor context" +
				" definition in the external files is not chained. Processor context = "+processor.getContext()) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Debug, Settings.getTID(), e) ;
		}
		throw new DSEInvalidArgumentException(DSEException.critical, null, "Exception caught while loading processor. Could not chain processor to session context: " + e.toString()) ;
	}

	//Updating the time-stamp info
	info.setTimeStamp(new Long(System.currentTimeMillis()));	
	return processor ;
}
/**
 * This is equivalent to calling removeProcessor(String, String, boolean)
 * 
 * @param sessionId java.lang.String
 * @param processorId java.lang.String
 */
public void removeProcessor(String sessionId, String processorId)
{
	this.removeProcessor(sessionId, processorId, false) ;
}
/**
 * Unregisters a processor object by removing its saved data
 * from storage. If the boolean argument is true, entries for all of
 * the processor's parents are also removed. Depending on the session
 * persistence configuration, the method looks for the processor either in the memory
 * table or in the session object (persistence).
 * 
 * @param sessionId java.lang.String
 * @param processorId java.lang.String
 * @param removeParentEntries boolean
 */
public void removeProcessor(String sessionId, String processorId, boolean removeParentEntries)
{
	if ( getSessionPersistence() ) removeProcessorPersistent( sessionId, processorId, removeParentEntries );
	else removeProcessorNonPersistent( sessionId, processorId, removeParentEntries );
}
/**
 * Unregisters a processor object by removing its saved data
 * from storage. If the boolean argument is true, entries for all of
 * the processor's parents are also removed.
 * 
 * @param sessionId java.lang.String
 * @param processorId java.lang.String
 * @param removeParentEntries boolean
 */
public void removeProcessorNonPersistent(String sessionId, String processorId, boolean removeParentEntries)
{
	if(processorId == null) return ;

	// get tables
	java.util.Hashtable procTable = getProcessorInfoTable(sessionId) ;
	java.util.Hashtable contextTable = getContextTable(sessionId) ;

	// get parent id, if any
	HtmlProcessorInfo info = (HtmlProcessorInfo) procTable.get(processorId) ;

	// if no information exists for this processor id, then return
	if(info == null)
	{// remove the processor from the running processors table (processorRegistry) if there exists it
		removeProcessor(processorId);
		return ;
	}	

	String parentId = info.getParentId() ;

	// remove current entry
	procTable.remove(processorId) ;
	contextTable.remove(processorId) ;

	/*<HSBC>*/
	//**** clear the entry from table after retrieve
	contextTable.remove(processorId + STATE_CONTEXT_ID_SUBFIX);
	/*</HSBC>*/

	// remove parent entry
	if(removeParentEntries) removeProcessor(sessionId, parentId, removeParentEntries) ;

	// remove the processor from the running processors table (processorRegistry) if there exists it
	removeProcessor(processorId);
}
/**
 * Unregisters a processor object by removing its saved data
 * from storage. If the boolean argument is true, entries for all of
 * the processor's parents are also removed.
 * 
 * @param sessionId java.lang.String
 * @param processorId java.lang.String
 * @param removeParentEntries boolean
 */
public void removeProcessorPersistent(String sessionId, String processorId, boolean removeParentEntries)
{
	if ( processorId == null ) return ;

	HttpSession sessionObject = null;

	try
	{
		sessionObject = ( HttpSession ) Context.getSessionObjectForSession( sessionId );
	}
	catch (DSEException e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Session object for session: "+sessionId+" not found. The sessionPersistence value has set to true, but not able to find the session object.") ;
			}				
	}
	
	// get parent id, if any
	HtmlProcessorInfo info = ( HtmlProcessorInfo ) sessionObject.getAttribute( processorId );

	// if no information exists for this processor id, then return
	if ( info == null ) 
	{// remove the processor from the running processors table (processorRegistry) if there exists it
		removeProcessor(processorId);
		return ;
	}

	String parentId = info.getParentId() ;

	// remove current entry
	sessionObject.removeAttribute(processorId) ;

	// Update the session object's processor registry
	/*java.util.Vector processorReg = ( java.util.Vector ) sessionObject.getValue( "processorRegister" );
	processorReg.removeElement( processorId );
	sessionObject.putValue( "processorRegister", processorReg );*/

	// remove parent entry
	if(removeParentEntries) removeProcessor(sessionId, parentId, removeParentEntries) ;

	// remove the processor from the running processors table (processorRegistry) if there exists it
	removeProcessor(processorId);
	
	/*<HSBC>*/
	//**** clear the entry from table after retrieve
        removePersistentStateContextTag(sessionId, processorId + STATE_CONTEXT_ID_SUBFIX);
	/*</HSBC>*/
	
}
public void removeProcessorsInSession(String sessionId)
{
	java.util.Vector processorKeys = getProcessorIdsInSession(sessionId);

	if ( processorKeys.isEmpty() ) 
	{
		// If there are no processors associated to this session just trace and return.
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Warning))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Warning, Settings.getTID(), "An attempt has been made to remove the processors associated with session: " + sessionId + " but this session has no processors running.");
		}		
		return;
	}

	String procId = null;
	for ( java.util.Enumeration enum = processorKeys.elements(); enum.hasMoreElements(); )
	{
		procId = (String) enum.nextElement();
		removeProcessor(sessionId, procId);
	}
}

/**
 * <HSBC>
 * Remove the persisted context
 */
public void removePersistentContext(String sessionId, String contextId)
{
	try {
		
		HttpSession sessionObject = ( HttpSession ) Context.getSessionObjectForSession( sessionId );
		sessionObject.removeAttribute( contextId );

	} catch(Exception e) {
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.removeContext: No persistent context could be removed for given session id: " + sessionId + " and context id: " + contextId);
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e);
		}
	}
}
/**
 * <HSBC>
 * Remove the persisted state context tag
 */
public void removePersistentStateContextTag(String sessionId, String stateContextTagID)
{
	try {
		
		HttpSession sessionObject = ( HttpSession ) Context.getSessionObjectForSession( sessionId );
		sessionObject.removeAttribute( stateContextTagID );

	} catch(Exception e) {
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.removeStateContextTag: No persistent state context tag could be removed for given session id: " + sessionId + " and state context tag id: " + stateContextTagID);
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e);
		}
	}
}

/**
 * Aborts all the processors that are running and 
 * removes their references in the processorRegistry. Both the running 
 * and not running processors are stored in the ProcessorInfoTable, and the 
 * contexts associated in the ContextTable are removed as well.
 * Each session has an associated processors table, so this table is
 * needed to store all the process for each session.
 */
public void resetProcessorManager() 
{
	if ( getSessionPersistence() ) resetProcessorManagerPersistent();
	else resetProcessorManagerNonPersistent();
}
/**
 * Aborts all the processors that are running and 
 * removes their references in the processorRegistry.
 */
public void resetProcessorManagerNonPersistent() 
{
	// Get an enumeration with all the stored sessions.
	java.util.Enumeration sessions =null;
	try
	{
		sessions = Context.getSessionTable().getEnumeration();
	}
	catch(DSEInvalidRequestException e)
	{
		if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
		{
			Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(), "Cannot reset the HtmlProcessorManager. Problems trying to get the sessions table.");
			Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(), e);
		}
	}
	
	// Go through all the sessions table.
	String sessionId = null;
	while (sessions.hasMoreElements())
	{
		try
		{
			sessionId = ( (SessionEntry) sessions.nextElement() ).getSessionId();
		} 
		catch(DSEInvalidRequestException e)
		{
			if (Trace.doTrace(Processor.COMPID,Trace.Low,Trace.Debug)) 
			{
				Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(), "Cannot reset the HtmlProcessorManager. Problems trying to get the session id from a SessionEntry.");
				Trace.trace(Processor.COMPID,Trace.Low,Trace.Debug,Settings.getTID(), e);
			}
		}

		if ( !getProcessorInfoTable(sessionId).isEmpty() )
		{
			// Each session could have associated several different processors.
			java.util.Enumeration processorsId = getProcessorInfoTable(sessionId).keys();
			String processorId = null;
			while (processorsId.hasMoreElements())
			{
				processorId = (String) processorsId.nextElement();

				// Remove from the tables both the processor and the context.
				removeProcessor(sessionId, processorId);
			}
		}
	}

	// The HTML specifical reseting tasks are done.
	// Now abort the running processors and remove them from the processors registry.
	super.resetProcessorManager();
}
/**
 * Gets a registry of the processors ran under this session.
 * 
 */
public void resetProcessorManagerPersistent() 
{
	if (Trace.doTrace(Processor.COMPID,Trace.High,Trace.Error)) 
	{
		Trace.trace(Processor.COMPID,Trace.High,Trace.Error,Settings.getTID(), "Cannot reset the HtmlProcessorManager while using persistence.");
	}
}
/**
 * Saves the information required to reinitialize
 * the processor to its current state.
 *
 * @param sessionId java.lang.String
 * @param processor com.ibm.dse.automaton.html.DSEHtmlProcessor
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.base.DSEException
 */
public void saveProcessor(String sessionId, DSEHtmlProcessor processor) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.base.DSEException
{
	if ( getSessionPersistence() ) saveProcessorPersistent( sessionId, processor);
	else saveProcessorNonPersistent( sessionId, processor);

	// Processor instance is not put into the cache because the invoker thread
	// may still need the instance to perform additional stuff.
	// The invoker of this method is responsible to put the processor into the
	// cache when it is safe to do it. Bear in mind that once the processor is in
	// the cache it can be reused by other threads belonging to concurrent requests

}
/**
 * Saves the information required to reinitialize
 * the processor to its current state.
 *
 * @param sessionId java.lang.String
 * @param processor com.ibm.dse.automaton.html.DSEHtmlProcessor
 * @param processorName java.lang.String
 * @param processorState java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.base.DSEException
 */
public void saveProcessorNonPersistent(String sessionId, DSEHtmlProcessor processor) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.base.DSEException
{
	try
	{
		// From now on does not need to store an instance in the processorRegistry
		// This is a registry of running processors.
		removeProcessor(processor.getProcessorId());
		
		java.util.Hashtable procTable = getProcessorInfoTable(sessionId) ;
		java.util.Hashtable contextTable = getContextTable(sessionId) ;

		HtmlStateInfo stateInfo = new HtmlStateInfo() ;
		State state = processor.getCurrentState() ;
		stateInfo.setStateName(state.getName()) ;
		try
		{
			stateInfo.setIgnoresEntryActions(((DSEState) state).isIgnoreEntryActions()) ;
			stateInfo.setRunningFlag(((DSEState) state).isRunningModeFlag()) ;
		}
		catch(Exception e) {
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when setting parameters to the stateInfo") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}
		}

		try
		{
			stateInfo.setIgnoresReturnType(((DSEHtmlState) state).isIgnoreReturnType()) ;
		}
		catch(Exception e) {
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when setting the ignoreReturnType to the stateInfo") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
			}
		}

		HtmlProcessorInfo info = new HtmlProcessorInfo();
		info.setProcessorName(processor.getName());
		info.setProcessorStatus(processor.getStatus());
		info.setParentId(processor.getParentId());
		// Next two attributes will identify the parent if the child proc changes the link to a different appSession
		info.setParentSessionId(processor.getParentSessionId());
		info.setParentAppSessionId(processor.getParentAppSessionId());
		info.setCurrentStateInfo(stateInfo);
		info.setProcessorTimeout(processor.getProcessorTimeout());
		info.setTimeStamp(new Long(System.currentTimeMillis()));
		info.setId(processor.getProcessorId());
		info.setExpired(false);

		procTable.put(processor.getProcessorId(), info) ;

		Object delValField=processor.getContext().getKeyedCollection().getElements().get(HtmlConstants.DELVALASSESSMENT);
		if ((delValField !=null) && (((String)((DataField)delValField).getValue()).equals("true")))
		{
			// if true take the parent context unchain it and store it in the context table
			//** <HSBC> processor.getContext().getParent().unchain();
			//** <HSBC> contextTable.put(processor.getProcessorId(), processor.getContext().getParent());

			processor.getProcessorContext().getParent().unchain();
			contextTable.put(processor.getProcessorId(), processor.getProcessorContext().getParent());
		}
		else
		{
			// Before saving the processor context, need to unchain it, and before is necessary to unchain its children
			//** <HSBC> for (Vector children = processor.getContext().getChildren(); children.elements().hasMoreElements();) {
			for (Vector children = processor.getProcessorContext().getChildren(); children.elements().hasMoreElements();) {
				((Context) children.elements().nextElement()).unchain();	
			}		
			//** <HSBC> processor.getContext().unchain();
			//** <HSBC> contextTable.put(processor.getProcessorId(), processor.getContext());
			processor.getProcessorContext().unchain();
			contextTable.put(processor.getProcessorId(), processor.getProcessorContext());
		}
		
			/** <HSBC> **/
		// If state context is available, store the context tag instead of the context
		if (state instanceof DSEHtmlState) {
			DSEHtmlState htmlState = (DSEHtmlState) state;
			if (htmlState.getContext() != null) {
				// Store the context tag if the context is not asked to be keep alive
				if (htmlState.isKeepContext()) {
					contextTable.put(processor.getProcessorId() + STATE_CONTEXT_ID_SUBFIX, htmlState.getContext());
				} else {
					contextTable.put(processor.getProcessorId() + STATE_CONTEXT_ID_SUBFIX, htmlState.getContextTag());
				}
		
				// chain back only the state context
				if (htmlState.getContext().getParent() == null) {
					htmlState.getContext().chainTo(processor.getProcessorContext());
					
					DataField field = new DataField();
					field.setName(Constants.StateContextInd);
					field.setValue(Boolean.TRUE);
					htmlState.getContext().addElement(field);
				}
			}
			
			htmlState.setContext(null);
		}
	/** </HSBC> **/
		
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "The processor was not successfully saved: " + e.toString());
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}
		throw new DSEException(DSEException.critical, null, "Exception caught while saving processor: " + e.getMessage()) ;
	}
}
/**
 * Saves the information required to reinitialize
 * the processor to its current state.
 *
 * @param sessionId java.lang.String
 * @param processor com.ibm.dse.automaton.html.DSEHtmlProcessor
 * @param processorName java.lang.String
 * @param processorState java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.base.DSEException
 */
public void saveProcessorPersistent(String sessionId, DSEHtmlProcessor processor) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.base.DSEException
{
	try
	{
		// From now on does not need to store an instance in the processorRegistry
		// This is a registry of running processors.
		removeProcessor(processor.getProcessorId());
		
		// Get the session object where the processor is going to be stored.
		HttpSession sessionObject = ( HttpSession ) Context.getSessionObjectForSession( sessionId );

		HtmlStateInfo stateInfo = new HtmlStateInfo();
		State state = processor.getCurrentState();
		stateInfo.setStateName(state.getName());
		try
		{
			stateInfo.setIgnoresEntryActions(((DSEState) state).isIgnoreEntryActions()) ;
			stateInfo.setRunningFlag(((DSEState) state).isRunningModeFlag()) ;
		}
		catch(Exception e) {
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when setting parameters to the stateInfo") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}
		}

		try
		{
			stateInfo.setIgnoresReturnType(((DSEHtmlState) state).isIgnoreReturnType()) ;
		}
		catch(Exception e) 
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when setting the ignoreReturnType to the stateInfo") ;
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;
			}
		}

		HtmlProcessorInfo info = new HtmlProcessorInfo() ;
		info.setProcessorName(processor.getName()) ;
		info.setProcessorStatus(processor.getStatus()) ;
		info.setParentId(processor.getParentId()) ;
		
		info.setParentSessionId(processor.getParentSessionId());
		info.setParentAppSessionId(processor.getParentAppSessionId());
		
		info.setCurrentStateInfo(stateInfo) ;
		info.setProcessorTimeout(processor.getProcessorTimeout());
		info.setTimeStamp(new Long(System.currentTimeMillis()));
		info.setId(processor.getProcessorId());
		info.setExpired(false);

		sessionObject.setAttribute( processor.getProcessorId(), info );

		Object delValField=processor.getContext().getKeyedCollection().getElements().get(HtmlConstants.DELVALASSESSMENT);
		if (delValField !=null)
		{
			String delValFlag=(String)((DataField)delValField).getValue();
			if (delValFlag.equals("true"))
			{	// if true take the parent context unchain it and store it in the context table
				
				//** <HSBC> processor.getContext().getParent().unchain();
				processor.getProcessorContext().getParent().unchain();
				
				String contextId = JavaExtensions.getAlphaUniqueCode();

				//** <HSBC> sessionObject.setAttribute( contextId, processor.getContext().getParent());
				sessionObject.setAttribute( contextId, processor.getProcessorContext().getParent());

				info.setContextId(contextId);
			}
		}
		else
		{
			// Before saving the processor context, need to unchain it, and before is necessary to unchain its children
			//** <HSBC> for (Vector children = processor.getContext().getChildren(); children.elements().hasMoreElements();) 
			for (Vector children = processor.getProcessorContext().getChildren(); children.elements().hasMoreElements();) 
			{
				((Context) children.elements().nextElement()).unchain();
			}
			// To save the processor context a new unique identifier is created.
			
			//** <HSBC>processor.getContext().unchain();
			processor.getProcessorContext().unchain();
			
			String contextId = JavaExtensions.getAlphaUniqueCode();

			//** <HSBC> sessionObject.setAttribute( contextId, processor.getContext() );
			sessionObject.setAttribute( contextId, processor.getProcessorContext() );
			
			info.setContextId(contextId);
		}

		/** <HSBC> **/
		// If state context is available, store the context tag instead of the context
		if (state instanceof DSEHtmlState) {
			DSEHtmlState htmlState = (DSEHtmlState) state;
			if (htmlState.getContext() != null) {
				// Store the context tag if the context is not asked to be keep alive
				if (htmlState.isKeepContext()) {
					sessionObject.setAttribute(processor.getProcessorId() + STATE_CONTEXT_ID_SUBFIX, htmlState.getContext());
				} else {
					sessionObject.setAttribute(processor.getProcessorId() + STATE_CONTEXT_ID_SUBFIX, htmlState.getContextTag());
				}
				// chain back only the state context and set indicator
				if (htmlState.getContext().getParent() == null) {
					htmlState.getContext().chainTo(processor.getProcessorContext());
					
					DataField field = new DataField();
					field.setName(Constants.StateContextInd);
					field.setValue(Boolean.TRUE);
					htmlState.getContext().addElement(field);
				}
				
				htmlState.setContext(null);
			}
			
		}
		/** </HSBC> **/
				
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "HTMLProcessorManager.saveProcessor: The processor was not successfully saved: " + e.toString()) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}
		throw new DSEException(DSEException.critical, null, "Exception caught while saving processor: " + e.getMessage()) ;
	}
}
/**
 * Gets from the initialization file (dse.ini) the value of the tag <I>initialization.sessionAffinity</I>, and
 * sets the value of the attribute <I>sessionAffinity</I>.
 * If its value is "false" then the WSBCC assumes that does not exist 
 * session affinity. By default is "true".
 * This method is specifical for HttpSession persistence.
 */
protected void setSessionAffinity()
{
	String	affinity;

	try
	{
		affinity = (String) Settings.getSettings().getValueAt("initialization.sessionAffinity");
	}
	catch (DSEObjectNotFoundException e)
	{
		setSessionAffinity(true);
		return;
	}

	if ( affinity!=null && affinity.equals(Constants.False) )
		setSessionAffinity(false);
	else 
		setSessionAffinity(true);
}
/**
 * Sets the value of the attribute <I>sessionAffinity</I>.
 * This method is specifical for HttpSession persistence.
 * @param newSessionPersistence boolean
 */
public void setSessionAffinity(boolean sessAffinity){
	sessionAffinity=sessAffinity;
}
/**
 * Gets from the initialization file (dse.ini) the value of the 
 * <I>initialization.sessionPersistence</I> tag, and
 * sets the value of the <I>sessionPersistence</I> property.
 * If its value is true, the toolkit assumes that the
 * server is not configured to persist sessions. By default, the property is set to false.
 * This method is specifically for HttpSession persistence.
 */
protected void setSessionPersistence()
{
	String	persistence;

	try
	{
		persistence = (String) Settings.getSettings().getValueAt("initialization.sessionPersistence");
	}
	catch (DSEObjectNotFoundException e)
	{
		setSessionPersistence(false);
		return;
	}

	if ( persistence!=null && persistence.equals(Constants.True) )
		setSessionPersistence(true);
	else 
		setSessionPersistence(false);
}
/**
 * Sets the value of the <I>sessionPersistence</I> property.
 * @param newSessionPersistence boolean
 */
public void setSessionPersistence(boolean newSessionPersistence) {
	sessionPersistence = newSessionPersistence;
}
}
