package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2002
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.*;
import com.ibm.dse.automaton.html.TCInfo;
/**
 * This class represents a persistent storage entity for an HTML
 * processor state.
 * 
 * @copyright(c) Copyright IBM Corporation 2000,2002
 */
public class HtmlStateInfo implements java.io.Serializable
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2002 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	private boolean ignoresReturnType = false;
	private boolean ignoresEntryActions = false;
	private java.lang.String stateName = null;
	private boolean runningFlag = true;
	protected Hashtable transitionContexts = null;
/**
 * HtmlStateInfo constructor.
 */
public HtmlStateInfo() {
	super();
}
/** 
 * Stores the data element name to be added to the transiton context for the specified transition. 
 * @param String - The transition name
 * @param String - The name of the data element to be added
 */ 
public void addDataElementToTransitionContext(String transitionName, String dataName)
{
	try{
		if (transitionContexts==null) transitionContexts=new Hashtable(1);
		String fullTransitionName=getStateName()+"."+transitionName;
		TCInfo tcInfo=(TCInfo)transitionContexts.get(fullTransitionName);
		if (tcInfo!=null) 
		{
			tcInfo.addAddElements(dataName);
		}
		else
		{
			tcInfo=new TCInfo();
			tcInfo.addAddElements(dataName);
			transitionContexts.put(fullTransitionName, tcInfo);
		}
	} catch (java.lang.Throwable e) {
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "Unable to add DataElement "+dataName+" in the transition  "+transitionName) ;
		}
	}
}
/** 
 * Stores the data element names to be created for transiton context for the specified transition 
 * with the set of dataNames.
 * @param String - The transition name
 * @param Vector - A vector with the data element names to be added
 */ 
public void createTransitionContext(String transitionName, Vector dataNames) throws DSEException
{
	if (transitionContexts==null) transitionContexts=new Hashtable(1);
	String fullTransitionName=getStateName()+"."+transitionName;
	TCInfo tcInfo=(TCInfo)transitionContexts.get(fullTransitionName);
	if (tcInfo!=null) 
	{
		tcInfo.deleteRemoveElements();
		tcInfo.deleteAddElements();
		tcInfo.addCreateElements(dataNames);
	}
	else
	{
		tcInfo=new TCInfo();
		tcInfo.addCreateElements(dataNames);
		transitionContexts.put(fullTransitionName, tcInfo);
	}
}
/**
 * Returns the state name property.
 * 
 * @return java.lang.String
 */
public java.lang.String getStateName() {
	return stateName;
}
public TCInfo getTCInfo(String transitionName) {
	if (transitionContexts!=null)
		return (TCInfo)transitionContexts.get(getStateName()+"."+transitionName); 
	else return null;
}
/**
 * Gets the value for the parameter named paramName for a transitionName of this state
 * Returns null if the parameter is not defined for the transaction
 * @param String - The transitionName
 * @param String - The parameter name 
 * @return String - The parameter value 
 */ 
public String getTransitionParameter(String transitionName, String paramName)
{
	if (transitionContexts==null) return null;
	else
	{
		String fullTransitionName=getStateName()+"."+transitionName;
		TCInfo tcInfo=(TCInfo)transitionContexts.get(fullTransitionName);
		if (tcInfo==null) return null;
		else
		{
			return tcInfo.getParameter(paramName);
		}
	}
}
/**
 * Returns the entry actions attribute of this class.
 * 
 * @return boolean
 */
public boolean isIgnoresEntryActions() {
	return ignoresEntryActions;
}
/**
 * Returns the ignore return type property.
 * 
 * @return boolean
 */
public boolean isIgnoresReturnType() {
	return ignoresReturnType;
}
/**
 * Returns the <i>isRunning</i> property.
 * 
 * @return boolean
 */
public boolean isRunningFlag() {
	return runningFlag;
}
/** 
 * Stores the data element name to be removed from the transiton context for the specified transition. 
 * @param String - The transition name
 * @param String - The name of the data element to be removed
 */ 
public void removeDataElementFromTransitionContext(String transitionName, String dataName)
{
	if (transitionContexts==null) transitionContexts=new Hashtable(1);
	String fullTransitionName=getStateName()+"."+transitionName;
	TCInfo tcInfo=(TCInfo)transitionContexts.get(fullTransitionName);
	if (tcInfo!=null) 
	{
		tcInfo.addRemoveElements(dataName);
	}
	else
	{
		tcInfo=new TCInfo();
		tcInfo.addRemoveElements(dataName);
		transitionContexts.put(fullTransitionName, tcInfo);
	}
}
/**
 * Sets the entry actions property.
 * 
 * @param newIgnoresEntryActions boolean
 */
public void setIgnoresEntryActions(boolean newIgnoresEntryActions) {
	ignoresEntryActions = newIgnoresEntryActions;
}
/**
 * Sets the ignore return type property.
 * 
 * @param newIgnoresReturnType boolean
 */
public void setIgnoresReturnType(boolean newIgnoresReturnType) {
	ignoresReturnType = newIgnoresReturnType;
}
/**
 * Sets the <i>isRunning</i> property.
 * 
 * @param newRunningFlag boolean
 */
public void setRunningFlag(boolean newRunningFlag) {
	runningFlag = newRunningFlag;
}
/**
 * Sets the state name property.
 * 
 * @param newStateName java.lang.String
 */
public void setStateName(java.lang.String newStateName) {
	stateName = newStateName;
}
/** 
 * Sets a parameter given by paramName with a value given by paramValue for a transitionName of this state
 * @param String - The transitionName
 * @param String - The parameter name 
 * @param String - The parameter value 
 */ 
public void setTransitionParameter(String transitionName, String paramName, String paramValue)
{
	if (transitionContexts==null) transitionContexts=new Hashtable(1);
	String fullTransitionName=getStateName()+"."+transitionName;
	TCInfo tcInfo=(TCInfo)transitionContexts.get(fullTransitionName);
	if (tcInfo!=null) 
	{
		tcInfo.setParameter(paramName, paramValue);
	}
	else
	{
		tcInfo=new TCInfo();
		tcInfo.setParameter(paramName, paramValue);
		transitionContexts.put(fullTransitionName, tcInfo);
	}
}
}
