package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.*;

/**
 * This class extends the behavior of TransitionDescriptor to provide support
 * for features unique to the HomeBanking HTML Client.
 *
 * @copyright(c) Copyright IBM Corporation 2000
 */
public class HtmlTransitionDescriptor extends com.ibm.dse.automaton.TransitionDescriptor
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	protected com.ibm.dse.base.Context context = null;
	protected java.lang.String consequentiality = DSEHtmlProcessor.CONSEQ_UNDEFINED;
	protected com.ibm.dse.base.Tag contextTag = null;
	public com.ibm.dse.base.Tag inputFormatTag = null;
	public com.ibm.dse.base.Tag outputFormatTag = null;

	/**
	 * Name of the consequentiality attribute.
	 */
	protected final String CONSEQ_ATT_NAME = "conseq";
	
	/**
	 * Name of the skipValidation attribute.
	 */
	protected final String SKIP_ATT_NAME = "skipValidation";
	
	/**
	 * Means that does not want to perform any validation task during this transition.
	 * By default this is true.
	 */
	protected boolean skipValidation = false;
	/**
	 * Means that the validation assesment is delegated for this transition to a later state.
	 * By default this is false.
	 */
	protected boolean delegateValAssessment = false;
	
	/** <HSBC> **/	
	public Tag interStateFmt = null;
	/** </HSBC> **/
	
/**
 * HtmlTransitionDescriptor constructor.
 */
public HtmlTransitionDescriptor() {
	super();
}
/**
 * Returns the consequentiality of this transition. It will be one
 * of the following values from DSEHTMLProcessor: CONSEQ_UNDEFINED, CONSEQ_TRUE,
 * or CONSEQ_FALSE.
 * 
 * @return java.lang.String
 */
public java.lang.String getConsequentiality()
{
	return consequentiality;
}
/**
 * Returns the context for this HTML transition.
 *
 * @note This method uses lazy instantiation to create its context, based
 * @note on the context tag stored from the initializeFrom() method.
 *
 * @return com.ibm.dse.base.Context
 */
public com.ibm.dse.base.Context getContext()
{
	if (context == null)
	{
		try
		{
			setContext((Context) Context.getExternalizer().convertTagToObject(getContextTag()));
		}
		catch (Exception e)
		{
			if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Information))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Information, Settings.getTID(), "Could not create a context instance for transition: " + this.getName()) ;
			}
		}
	}

	return context;
}
/**
 * Returns the context tag corresponding to the context defined
 * for this object.
 * 
 * @return com.ibm.dse.base.Tag
 */
public com.ibm.dse.base.Tag getContextTag() {
	return contextTag;
}
/**
 * Returns the value of the delegateValAssessment property.
 * 
 * @return boolean
 */
public boolean getDelegateValAssessment()
{
	return delegateValAssessment;
}
/**
 * Returns the input format tag property.
 * 
 * @return com.ibm.dse.base.Tag
 */
public com.ibm.dse.base.Tag getInputFormatTag() {
	return inputFormatTag;
}
/**
 * Returns the output format tag property.
 * 
 * @return com.ibm.dse.base.Tag
 */
public com.ibm.dse.base.Tag getOutputFormatTag() {
	return outputFormatTag;
}
/**
 * Initializes the state of this transition from externalized definitions.
 *
 * @note HtmlTransitionDescriptors have a context, which is lazy initialized in the
 * @note getContext() method of this class. To accomplish this, the context tag is
 * @note obtained in this method, and stored in the class for future use.
 *
 * @return java.lang.Object
 * @param aTag com.ibm.dse.base.Tag
 * @exception java.io.IOException
 * @exception com.ibm.dse.base.DSEException
 */
public Object initializeFrom(Tag aTag) throws java.io.IOException, com.ibm.dse.base.DSEException
{
	super.initializeFrom(aTag) ;

	// The default behaviour is for a transition not to specify consequentiality
	setConsequentiality(DSEHtmlProcessor.CONSEQ_UNDEFINED) ;

	// get context attribute
	String name = null, value = null ;
	TagAttribute attribute = null ;
	Hashtable h=new Hashtable();
	for(java.util.Enumeration e = aTag.getAttrList().elements() ; e.hasMoreElements() ; )
	{
		attribute = (TagAttribute) e.nextElement() ;
		name = (String) attribute.getName() ;
		value = (String) attribute.getValue() ;
		if(name.equals(CONSEQ_ATT_NAME))
		{
			if(value.equals(Constants.True))
			{
				setConsequentiality(DSEHtmlProcessor.CONSEQ_TRUE) ;
			}
			else if(value.equals(Constants.False))
			{
				setConsequentiality(DSEHtmlProcessor.CONSEQ_FALSE) ;
			}
			else throw new DSEException(DSEException.harmless, null, "Undefined value specified for conseq attribute in tag definition: " + value) ;
		}
		
		if(name.equals(SKIP_ATT_NAME))
		{
			if(value.equals(Constants.True))
			{
				setSkipValidation(true);
			}
			else if(value.equals(Constants.False))
			{
				setSkipValidation(false);
			}
			else throw new DSEException(DSEException.harmless, null, "Undefined value specified for skipValidation attribute in tag definition: " + value) ;
		}
		if (name.equals(DSEHtmlProcessor.DELEGATEVALASSESSMENT))
		{ 
			if(value.equals(Constants.True))
			{	
				setDelegateValAssessment(true);
			}
			else if(value.equals(Constants.False))
			{
				setDelegateValAssessment(false);
			}
			else throw new DSEException(DSEException.harmless, null, "Undefined value specified for delValidationAssesment attribute in tag definition: " + value) ;	
		}
	}
	Tag nextSubTag;
	java.util.Enumeration attributes;
	TagAttribute refname=null;
	Tag format=null;

	try
	{
		int nTags = aTag.getSubTags().size();
		for (int i = 0; i<nTags; i++)
		{
			nextSubTag = (Tag)(aTag.getSubTags().elementAt(i));
			if (nextSubTag.getName()!=null)
			{
				if (nextSubTag.getName().equals(Constants.RefFormat))
				{
					format=nextSubTag.getSubTag(0);//only there is one subtag
					attributes = nextSubTag.getAttrList().elements();
					while (attributes.hasMoreElements())
					{
						attribute = (TagAttribute) attributes.nextElement();
						if (attribute.getName().equals(Constants.Name))
						{
							if (attribute.getValue().equals(Constants.InputMapFmt))
							{	
								setInputFormatTag(format);
			
							}
							else if (attribute.getValue().equals(Constants.OutputMapFmt))
							
							{	
								setOutputFormatTag(format);
							}
							/** <HSBC> **/
							// Set InterStateFmt value if defined
							else if (attribute.getValue().equals(DSEHtmlState.INTER_STATE_FMT)) 
							{
								setInterStateFormatTag(format);
							}
							/** </HSBC> **/
						}
					}
				}
				else if (nextSubTag.getName().equals(com.ibm.dse.base.Constants.Context))
				{
					setContextTag(nextSubTag) ;
				}	
			}//end if null
		}//end for
	}
	catch(Exception e){
		throw new java.io.IOException(e.toString());
	}
	
	return this ;
}
/**
 * Gets the skipValidation attribute of this transition.
 * @return boolean
 */
public boolean isSkipValidation() {
	return skipValidation;
}
/**
 * Sets the consequentiality of this transition. It should be one
 * of the following values from DSEHTMLProcessor: CONSEQ_UNDEFINED, CONSEQ_TRUE,
 * or CONSEQ_FALSE.
 * 
 * @param newConsequentiality java.lang.String
 */
public void setConsequentiality(java.lang.String newConsequentiality)
{
	consequentiality = newConsequentiality;
}
/**
 * Sets the context property.
 *
 * @param newContext com.ibm.dse.base.Context
 */
public void setContext(com.ibm.dse.base.Context newContext) {
	context = newContext;
}
/**
 * Stores the context tag corresponding to the context attribute
 * of this class.
 * 
 * @param newContextTag com.ibm.dse.base.Tag
 */
public void setContextTag(com.ibm.dse.base.Tag newContextTag) {
	contextTag = newContextTag;
}
/**
 * Sets the delegateValAssessment property.
 * 
 * @param newDelValAssessment boolean
 */

public void setDelegateValAssessment(boolean newDelValAssessment)
{
	delegateValAssessment=newDelValAssessment;
}
/**
 * Sets the input format tag property.
 * 
 * @param newInputFormatTag com.ibm.dse.base.Tag
 */
public void setInputFormatTag(com.ibm.dse.base.Tag newInputFormatTag) {
	inputFormatTag = newInputFormatTag;
}
/**
 * Sets the output format tag property.
 * 
 * @param newOutputFormatTag com.ibm.dse.base.Tag
 */
public void setOutputFormatTag(com.ibm.dse.base.Tag newOutputFormatTag) {
	outputFormatTag = newOutputFormatTag;
}
/**
 * Sets the skipValidation property.
 * @param newSkipValidation boolean
 */
protected void setSkipValidation(boolean newSkipValidation) {
	skipValidation = newSkipValidation;
}

/**
 * <HSBC>
 * set the State out format
 */
public void setInterStateFormatTag(Tag newInterStateFormatTag) {
	interStateFmt = newInterStateFormatTag;
}


/**
 * <HSBC>
 * Returns inter state format
 */
public Tag getInterStateFormatTag() {
	return interStateFmt;
}

}
