package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.*;
import com.ibm.dse.automaton.* ;
/**
 * This interface provides an abstract definition of a processor operation.
 * A processor operation is basically a manager for processor objects.
 *
 * @copyright(c) Copyright IBM Corporation 2000
 */
public interface ProcessorOperation {

/**
 * Close this processor operation, performing housekeeping tasks.
 *
 * @exception com.ibm.dse.base.DSEInvalidRequestException
 * @exception com.ibm.dse.base.DSEDSEObjectNotFoundException
 */
public void close() throws com.ibm.dse.base.DSEInvalidRequestException, com.ibm.dse.base.DSEObjectNotFoundException ;
/**
 * Closes the current processor object being managed by the processor
 * operation
 * 
 * @param processor com.ibm.dse.automaton.Processor
 */
public void closeProcessor(Processor processor) ;
/**
 * Provides the entry point for execution of the processor object currently
 * being managed by this processor operation. The supplied event name is used
 * as input, for execution of the processor state machine.
 *
 * @param eventName java.lang.String
 * @exception com.ibm.dse.base.DSEException
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 */
public abstract void execute(String eventName) throws com.ibm.dse.base.DSEException, com.ibm.dse.base.DSEInvalidArgumentException;
/**
 * Returns the processor object currently being managed by this processor
 * operation
 *
 * @return com.ibm.dse.automaton.Processor
 */
public abstract com.ibm.dse.automaton.Processor getProcessor();
/**
 * Returns an appropriate response id in response to execution
 * of the processor object currently being managed.
 * 
 * @return java.lang.String
 * @exception com.ibm.dse.base.DSEException
 */
public String getResponseId() throws com.ibm.dse.base.DSEException ;
/**
 * Returns the status of the processor operation.
 * 
 * @return java.lang.String
 */
public java.lang.String getStatus() ;
/**
 * This method is called to initialize a processor object based on data passed in the
 * keyed collection. The processor object initialized should be set as the current
 * processor being managed by the processor operation.
 *
 * @param coll com.ibm.dse.base.KeyedCollection
 * @exception com.ibm.dse.base.DSEException
 */
public void initializeProcessor(KeyedCollection coll) throws com.ibm.dse.base.DSEException ;
/**
 * Sets the processor object currently being managed by this processor
 * operation
 * @param processor com.ibm.dse.automaton.Processor
 */
public abstract void setProcessor(com.ibm.dse.automaton.Processor processor);
/**
 * Sets an appropriate response id corresponding to the execution of the
 * currently managed processor.
 * 
 * @param id java.lang.String
 * @exception com.ibm.dse.base.DSEInvalidArgumentException
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 */
public void setResponseId(String id) throws com.ibm.dse.base.DSEInvalidArgumentException, com.ibm.dse.base.DSEObjectNotFoundException ;
/**
 * Sets the status of the processor operation.
 * 
 * @param newStatus java.lang.String
 */
public void setStatus(java.lang.String newStatus) ;
/**
 * Updates the context of the processor object currently being managed
 * by this processor operation, from the data supplied in the keyed collection.
 *
 * @return boolean
 * @param collection com.ibm.dse.base.KeyedCollection
 * @exception com.ibm.dse.base.DSEObjectNotFoundException
 * @exception com.ibm.dse.base.DSEException
 */
public abstract boolean updateContext(KeyedCollection collection) throws com.ibm.dse.base.DSEObjectNotFoundException, com.ibm.dse.base.DSEException ;
}
