package com.ibm.dse.automaton.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2002
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.Hashtable; 
import com.ibm.dse.base.Vector; 
/**
 * This class represents a persistent storage of the transition context
 * to be created dynamically for a given transition of an HTML
 * processor state.
 * 
 * @copyright(c) Copyright IBM Corporation 2002
 */ 
 
public class TCInfo implements java.io.Serializable {
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2002 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
/** The array of names of the DataElements to be added into the transition context **/
protected String [] addElements = null;
/** The addElements length **/
protected int addElementsLength = 0;
/** The addElements maximum length **/
protected final int addElementsMAXLength = 50;
/** The array of names of the DataElements to be created in the transition context **/
protected String [] createElements = null;
/** The createElements length **/
protected int createElementsLength = 0;
/** The createElements maximum length **/
protected final int createElementsMAXLength = 50;
/** The array of names of the DataElements to be removed from the transition context **/
protected String [] removeElements = null;
/** The removeElements length **/
protected int removeElementsLength = 0;
/** The removeElements maximum length **/
protected final int removeElementsMAXLength = 50;
/** Stores the parameterName  parameterValue pairs to be assigned to the delegateValAssessment attribute of the transition once this is restored **/
protected Hashtable transitionParameters = null;
/** 
 * Adds a new dataName in the addElements array 
 * @param String - The data name
 */ 
public void addAddElements(String dataName)
{
	if (addElements==null) 
	{
		addElements=new String[addElementsMAXLength];
		addElements[0]=dataName;
	    addElementsLength=1;		
	}
	else
	{	
		addElements[addElementsLength]=dataName;
	    addElementsLength++;		
	}
}
/** 
 * Sets the createElements array with a new set of dataNames 
 * @param Vector - The vector of data names
 */ 
public void addCreateElements(Vector dataNames)
{
	int size=dataNames.size();
	int index=0;
	createElements=null;
	String [] newElements= new String[createElementsMAXLength];
	while (index<size)
	{
		newElements[index]=(String)dataNames.elementAt(index);
		index=index+1;
	}
	createElements=newElements;
	createElementsLength=size;
}
/** 
 * Adds a new element in the removeElements array 
 * @param String - The dataName to be added
 */ 
public void addRemoveElements(String dataName)
{
	if (removeElements==null) 
	{
		removeElements=new String[removeElementsMAXLength];
		removeElements[0]=dataName;
		removeElementsLength=1;
	}
	else
	{	
		removeElements[removeElementsLength]=dataName;
		removeElementsLength++;
	}
}
/** 
 * Deletes the data names contained in the addElements array 
 */ 
public void deleteAddElements()
{
   addElements=null;
   addElementsLength=0;		
}
/** 
 * Deletes the data names contained in the createElements array 
 */ 
public void deleteCreateElements()
{
	createElements=null;
    createElementsLength=0;		
}
/** 
 * Deletes the data names contained in the removeElements array 
 */ 

public void deleteRemoveElements()
{
	removeElements=null;
    removeElementsLength=0;		
}
/** 
 * Gets the addElements array 
 * @return String [] 
 */ 
public String [] getAddElements()
{
	return addElements;	
}
/** 
 * Gets the addElements array size
 * @return int 
 */ 
public int getAddElementsLength()
{
	return addElementsLength;	
}
/** 
 * Gets the createElements array 
 * @return String [] 
 */ 
public String [] getCreateElements()
{
	return createElements;	
}
/** 
 * Gets the createElements array size
 * @return int 
 */ 
public int getCreateElementsLength()
{
	return createElementsLength;	
}
/** 
 * Returns the value of a parameter from the transitionParameters if this parameters
 * exists. Otherwise returns null.
 * @param String - the name of the parameter
 * @return String - the value of the parameter
 */ 
public String getParameter(String name)
{
	if (transitionParameters==null) return null;
	else return ((String)transitionParameters.get(name));
}
/** 
 * Gets the removeElements array 
 * @return String [] 
 */ 
public String [] getRemoveElements()
{
	return removeElements;	
}
/** 
 * Gets the removeElements array 
 * @return int 
 */ 
public int getRemoveElementsLength()
{
	return removeElementsLength;	
}
/** 
 * Adds a name value pair to the transitionParameters
 * @param String - the name of the paramer
 * @param String - the value 
 */ 
public void setParameter(String name, String value)
{
	if (transitionParameters==null) transitionParameters=new Hashtable(1);
	transitionParameters.put(name, value);
}
}
