package com.ibm.dse.cs.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.NoSuchElementException;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;
import com.ibm.dse.automaton.html.DSEHtmlProcessor;
import com.ibm.dse.automaton.html.HtmlProcessorInfo;
import com.ibm.dse.automaton.html.HtmlStateInfo;
import com.ibm.dse.automaton.html.HtmlProcessorManager;
import com.ibm.dse.base.*;
import com.ibm.dse.base.types.PropertyDescription;
import com.ibm.dse.cs.html.*;


/**
 * This class implements the JspContextServices interface.
 * <P>It receives the operation context from the request and
 * provides various interfaces to access the context information.
 * @copyright(c) Copyright IBM Corporation 2000, 2003.
 */
public class DSEJspContextServices implements JspContextServices
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2003 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	private static java.util.ResourceBundle resResources = java.util.ResourceBundle.getBundle("htmlclient");  //$NON-NLS-1$
	/** The operation context. **/
	protected Context operationContext = null;
	/** The session context. **/
	protected Context sessionContext = null;
	/** The current operation exception **/
	protected DSEException dseexception = null;
	/** The base web path string **/
	protected String baseWebPath = null;
	/** The locale (extra path) string. **/
	protected String extraPathInfo = null;
	/** The session id string **/
	protected String sessionID = null;
	/** The error marker value **/
	protected String errorMarker = "<font color=darkred>X</font>";//$NON-NLS-1$
	/** The error decorator "prefix" value **/
	protected String errorPrefix = "<font color=darkred>";//$NON-NLS-1$
	/** The error decorator "suffix" value **/
	protected String errorSuffix = "</font>";//$NON-NLS-1$
	/** The validation errors list **/
	protected String[] errsVal = null;
	/** The cross-validation errors list **/
	protected String[] errsXVal = null;
	/** The JSP format errors list **/
	protected Vector errsJsp = null;
	/** The pageId for the displaying page **/
	protected String pageId = null;
	/** The application session id string **/
	protected String applicationID = null;
	/** The application session context **/
	protected Context applicationSessionContext = null;
	/** The processor name **/
	protected String processorName = null;
	/** The processor ID **/
	protected String processorID = null;
    /** the Web Application URI **/
    protected String webApplURI = null;
    /** the Web Application URI plus base path **/
    protected String baseWebApplURIPath = null;

	
/**
 * DSEJspContextServices default constructor.
 */
public DSEJspContextServices() {
	super();
}
/** 
 * Adds a data element in the the transition context for the specified transition
 * @param String - The transition name
 * @param String - The name of the data element to be added
 */ 
public void addDataElementToTransitionContext ( String transitionName, String dataName) throws DSEException
{
	com.ibm.dse.automaton.html.HtmlProcessorInfo info = null ;
 	com.ibm.dse.automaton.html.HtmlStateInfo stateInfo = null;
	try
	{
		info = HtmlProcessorManager.getUniqueHTMLInstance().getInfo(sessionID, processorID);
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getInfo: No persistent data could be loaded for given processor id: " + processorID) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;			
		}
	}

	if ( info == null )	throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve processor information for sessionId: " + sessionID + " and processorId: "+ processorID );

	String theProcessorName = info.getProcessorName();
	stateInfo = info.getCurrentStateInfo() ;
	if (stateInfo!=null) stateInfo.addDataElementToTransitionContext(transitionName, dataName);
	else throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve currentState information for sessionId: " + sessionID + " and processorId: "+ processorID );
}
/**
 * Useful service for converting a String into it's
 * equivalent double value.
 * @return double - The converted <I>double</I> value.
 */
public double asDouble ( String strval )
{
	try { return Double.valueOf ( strval ).doubleValue ( ); }
	catch ( NumberFormatException xcp ) { }
	return 0.0;
	
} // end asDouble ( String strval )
/**
 * Useful service for converting a String into it's
 * equivalent float value.
 * @return float - The converted <I>float</I> value.
 */
public float asFloat ( String strval )
{
	try { return Float.valueOf ( strval ).floatValue ( ); }
	catch ( NumberFormatException xcp ) { }
	return 0.0f;
	
} // end asFloat ( String strval )
/**
 * Useful service for converting a String into it's
 * equivalent integer value.
 * @return int - The converted <I>int</I> value.
 */
public int asInt ( String strval )
{
	try { return Integer.parseInt ( strval ); }
	catch ( NumberFormatException xcp ) { }
	return 0;
	
} // end asInt ( String strval )
/**
 * Useful service for converting a String into it's
 * equivalent long value.
 * @return long - The converted <I>long</I> value.
 */
public long asLong ( String strval )
{
	try { return Long.parseLong ( strval ); }
	catch ( NumberFormatException xcp ) { }
	return 0;
	
} // end asLong ( String strval )
/** 
 * Creates a transiton context for the specified transition 
 * with the set of dataNames
 * @param String - The transition name
 * @param Vector - A vector with the data element names to be added
 */ 
public void createTransitionContext(String transitionName, com.ibm.dse.base.Vector dataNames) throws DSEException
{
 	com.ibm.dse.automaton.html.HtmlProcessorInfo info = null ;
 	com.ibm.dse.automaton.html.HtmlStateInfo stateInfo = null;
	try
	{
		info = HtmlProcessorManager.getUniqueHTMLInstance().getInfo(sessionID, processorID);
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getInfo: No persistent data could be loaded for given processor id: " + processorID) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;			
		}
	}


	if ( info == null )
		throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve processor information for sessionId: " + sessionID + " and processorId: "+ processorID );


	// get the processor currentState info
		String theProcessorName = info.getProcessorName();
		stateInfo = info.getCurrentStateInfo() ;
		if (stateInfo!=null) stateInfo.createTransitionContext( transitionName, dataNames);
		else throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve currentState information for sessionId: " + sessionID + " and processorId: "+ processorID );
}
/**
 * Returns the current application session identifier string
 * as an HTML hidden field.
 * @return String - The <I>application id</I> field html.
 */
public String getApplicationIdHtmlField ( )
{
	return "<input type=\"hidden\" name=\"" + HtmlConstants.APPLICATIONID +//$NON-NLS-1$
			"\" value=\"" + applicationID + "\" />";//$NON-NLS-2$//$NON-NLS-1$
	
} // end getSessionIdHtmlField ( )
/**
 * Returns the current application identifier string
 * as a URL parameter.
 * @return String - The <I>application id</I> parameter.
 */
public String getApplicationIdParameter ( )
{
	return HtmlConstants.APPLICATIONID + "=" + applicationID;//$NON-NLS-1$
	
} // end getSessionIdParameter ( )
/**
 * Return the application session context for the page which
 * initialized this bean.
 * @return Context - The <I>application session context</I> value.
 */
public Context getApplicationSessionContext ( )
{
	return applicationSessionContext;
	
} // end getSessionContext ( )
/**
 * Returns the current application session identifier string "as-is".
 * @return String - The <I>application id</I> value.
 */
public String getApplicationSessionId() {
	
	
	return applicationID;
}
/**
 * Gets the baseWebApplURIPath
 * @return Returns a String
 */
public String getBaseWebApplURIPath() {

    return baseWebApplURIPath;
}
/**
 * Obtain the relative path for jsp resources.
 * <P>Relative to the web app server web path
 * @return String - The <I>base web path</I> value.
 */
public String getBaseWebPath ( )
{
	return baseWebPath;
	
} // end getBaseWebPath ( )
/**
 * Get the operation context for the page which initialized this bean.
 * @return Context - The <I>operation context</I> value.
 */
public Context getContext ( )
{
	return operationContext;
	
} // end getContext ( )
/**
 * Return the current list of cross validation errors.
 * @return String[] - The <I>error list array</I> value.
 */
public String[] getCrossValidationErrorList ( )
{
	if (errsXVal == null) 
	{
		errsXVal = (String[]) operationContext.tryGetValueAt ( HtmlConstants.ERRORMESSAGES );
		if( errsXVal == null )
		{
			errsXVal = new String[0];
		}
		
	} 
	
	return errsXVal;
	
} // end getCrossValidationErrorList ( )
/**
 * Returns the named data element from the operation context.
 * <P>If the element is not present in the context, null
 * is returned.
 * @return DataElement - The <I>data element</I> value.
 */
public DataElement getElement ( String name )
{
	DataElement value=null;
	try {
		value = operationContext.tryGetElementAt(name);
		if ( value == null )
		{
			if (applicationSessionContext!=null)
			{
				value = applicationSessionContext.tryGetElementAt(name);
				if(value == null)
				{
					value=sessionContext.tryGetElementAt(name);
				}
			}
			else
				value=sessionContext.tryGetElementAt(name);
		}
		// if element not found in ctx hierarchy, print trace
		if(value == null )
		{
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Information))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Data element not found in context hierarchy: "+name) ;
			//Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), exc) ;			
		}
		}
	}
	catch (Exception exc)
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Information))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "Data element not found in context hierarchy: "+name) ;
			//Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), exc) ;			
		}
	}
	return value;
	
} // end getElement ( String )
/**
 * Insert the method's description here.
 * @return com.ibm.dse.base.DataElement
 * @param name java.lang.String
 * @param useSess boolean
 */
protected DataElement getElement ( String name, boolean useSess )
{
	Context ctx = (useSess) ? sessionContext : operationContext;
	DataElement de;


	try { de = ctx.getElementAt ( name ); }
	catch ( Exception xcp )
	{
		setJspFormatError ( xcp, name, useSess );
		de = null;
		
	} // end catch general purpose exceptions
	
	return de;
	
} // end getElement ( String, boolean )
/**
 * Returns the error info object associated to the named field in the context.
 * @return ErrorInfo
 */
public ErrorInfo getErrorInfo (String name)
{
	ErrorInfo errorInfo = null;
	DataElement value=null;
	try{
		
	value = operationContext.tryGetElementAt(name);
	if ( value == null )
	{
	
		
	  if (applicationSessionContext!=null)
	  {
			value = applicationSessionContext.tryGetElementAt(name);
			if( value==null)
			{
				value = sessionContext.tryGetElementAt(name);
			}
		}
		else
			value = sessionContext.tryGetElementAt(name);
		
		
	 }
		
	if( value != null )
	{
		errorInfo = value.getErrorInfo();
	}
	
	}catch(Exception e){}
	
	return errorInfo;
}
/**
 * Return the error page name
 * @return String - The <I>error page</I> html.
 */
public String getErrorPageHtml ( String name )
{
	return "<input type=\"hidden\" name=\"" + HtmlConstants.ERRORPAGE + "\" value=\"" + name + "\" />";//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$


} // end getErrorPageHtml ( String )
/**
 * If the toolkit placed an exception in the 
 * HttpServletRequest with key name <b>dseexception</b>,
 * then it can be saved here when the bean is initialized.
 * @return DSEException - The <I>exception</I> value.
 */
public DSEException getException ( )
{
	return dseexception;
	
} // end getException ( )
/**
 * Return message text cooresponding to the current exception.
 * @return String - The <I>exception</I> message.
 */
public String getExceptionMessage ( )
{
	String msg = null;


	if (dseexception != null)
	{
		String mcode = dseexception.getCode ( );
		
		if (mcode.equals ( HtmlConstants.PENDINGEXCEPTIONCODE ))
			msg = resResources.getString("HTML005"); //$NON-NLS-1$


		if (mcode.equals ( HtmlConstants.AGINGEXCEPTIONCODE ))
			msg = resResources.getString("HTML006"); //$NON-NLS-1$
		
	} // end if there is an exception outstanding


	return (msg != null) ? "<hr /><b><i>" + msg + "</i></b><hr />" : "";//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$


} // end getExceptionMessage ( )
/**
 * Return the current error marker only if the given
 * field is in error.
 * <P>Returns a blank string if no error exists.
 * @return String - The <I>field error marker</I> value.
 */
public String getFieldErrorMarker ( String name )
{
	return (isFieldInError ( name )) ? errorMarker : "";//$NON-NLS-1$
	
} // end getFieldErrorMarker ( String )
/**
 * Returns the label associated with the given field
 * in the context.
 * @return String - The <I>field label</I> value.
 */
public String getFieldLabel ( String name )
{
	String val;
	
	try
	{
		DataElement de = getElement ( name );
		val = de.getDescription ( );
		
	} catch ( Exception xcp ) { val = new String ( "" ); }//$NON-NLS-1$
	
	return (String) val;
	
} // end getFieldLabel ( String name )
/**
 * Return the field error message text at the specified index.
 * <P>This method returns a blank string if the field
 * has no errors.
 * @return String - The <I>validation error</I> value.
 */
public String getFieldValidationError ( String name, int idx )
{
	String eval = "";//$NON-NLS-1$
	
	DataElement de = getElement ( name );

	if (de!=null && de.getErrorInfo ( ) != null)
	{
			
		String[] errorMsgs = translateErrors(de.getErrorInfo());
		eval = errorMsgs[idx];
			
	} // end if error information exists for this field
		
	return eval;
	
} // end getFieldValidationError ( String, int )
/**
 * Return the field error message text at the specified
 * index decorated with prefix and suffix HTML strings.
 * <P>This method returns a blank string if the field
 * has no errors.
 * @return String - The <I>decorated error</I> value.
 */
public String getFieldValidationErrorDecorated ( String name, int idx )
{
	String eval = "";//$NON-NLS-1$
	
	DataElement de = getElement ( name );

	if (de!=null && de.getErrorInfo ( ) != null)
	{
			
		String[] errorMsgs = translateErrors(de.getErrorInfo());
		eval = errorMsgs[idx];
			
		if (eval.length ( ) > 0)
				eval = errorPrefix + eval + errorSuffix;
					
	} // end if error information exists for this field
		
	return eval;
	
} // end getFieldValidationError ( String, int )
/**
 * Retrieve the entire error list for the field.
 * <P>An array of zero size is returned if there
 * are no errors associated with the field.
 * <P>null is returned if there is not an element
 * matching with the name provided as a parameter.
 * @return String[] - The <I>field error</I> list.
 */
public String[] getFieldValidationErrorList ( String name )
{
	ErrorInfo err = null;
	
	DataElement de = getElement ( name );
	if (de==null) return null;
	else if ((err = de.getErrorInfo ( ))==null) return new String[0];

	String[] errs = translateErrors(err);
	
	return errs;
	
} // end getFieldValidationErrorList ( String name )
/**
 * Returns the value of the wrong data typed by the user.
 * This data is considered to be wrong according to the validation process.
 * The typed data is stored in the ErrorInfo object associated to the named field
 * @return String
 */
public String getInputDataInError (String name)
{
	String wrongData = "";
	DataElement value = null;
	try 
	{
		value = operationContext.tryGetElementAt(name);
		if( value == null )
		{
			value = sessionContext.tryGetElementAt(name);
		}
		if (value != null )
		{
			wrongData = (String)value.getErrorInfo().getInputObject();
		}


	} catch ( Exception e) {
		
	}
	return wrongData;
}
/**
 * Returns the list of Jsp Format Errors that were found
 * during Jsp Processing.
 * @return String[] - The <I>format error</I> list.
 */
public String[] getJspFormatErrorList ( )
{
	String[] aerrs;
	
	if (errsJsp == null) aerrs = new String[0];
	else
	{
		aerrs = new String[errsJsp.size ( )]; int idx = 0;


		for ( Enumeration enu = errsJsp.elements ( ); enu.hasMoreElements ( ); )
		{
			aerrs[idx++] = (String) enu.nextElement ( );
			
		} // end for loop converting error list to string array
		
	} // end if there is indeed a JSP error list
	
	return aerrs;
	
} // end getJspFormatErrorList ( )
/**
 * Emit the Html to the JspWriter stream to create a link with given text to
 * a given JSP saved in a given web path (relative to the base JSP path for the
 * application server). The link will include the session id using URL rewriting.
 * @return String - The <I>link</I> html.
 */
public String getLinkToPage ( String webPath, String pageName, String linkText )
{
	return  "<a href=" + baseWebPath + webPath + pageName + //$NON-NLS-1$
			"?" + HtmlConstants.SESSIONID + "=" + getSessionId ( ) +//$NON-NLS-2$//$NON-NLS-1$
			">" + linkText + "</a>";//$NON-NLS-2$//$NON-NLS-1$
	
} // end getLinkToPage ( String, String, String )
/**
 * Obtain any extra information passed on the call to establish the session.
 * For example, if the URL was invoked as:
 * <br>http://.../CSEstablishSessionServlet/en
 * <br>then the <b>extra path info</b> is the string <b>en</b>.
 * @return String - The <I>extra path</I> info.
 */
public String getLocaleId ( )
{
	return extraPathInfo;
	
} // end getLocaleId ( )
/**
 * Returns a string containing the HTML for all of the
 * HTML Client required fields using the current processor name
 * gotten from the table of stored processors.
 * @return String - The <I>required fields</I> html.
 */
public String getRequiredHtmlFields ()
{
	return getRequiredHtmlFields("",null);//$NON-NLS-1$
} // end getRequiredHtmlFields ()
/**
 * Returns a string containing the HTML for all of the
 * HTML Client required fields using the passed parameters 
 * to fill in the user specifiable information.
 * If no opName is specified, assume that this page is being
 * displayed within a processor flow, so the processor name is
 * got from the table of stored processors.
 * @param parentctx java.lang.String
 * @param opname java.lang.String
 * @return String - The <I>required fields</I> html.
 */
public String getRequiredHtmlFields ( String parentCtx, String opName)
{
	StringBuffer resp = new StringBuffer();


	try
	{
		// If no opName is specified, assume that this page is being displayed within a processor flow. Look for the processor name.
		if ( opName == null || opName.equals("") )//$NON-NLS-1$
		{
			if ( getValue(HtmlConstants.PROCINFOTABLE) == null ) //session persistence is used
				opName = ( (com.ibm.dse.automaton.html.HtmlProcessorInfo) ( (javax.servlet.http.HttpSession) Context.getSessionObjectForSession(sessionID) ).getAttribute( (String) getValue(HtmlConstants.PROCESSORID) )).getProcessorName();
			else
				opName = ( (com.ibm.dse.automaton.html.HtmlProcessorInfo) ( ( (Hashtable) getValue(HtmlConstants.PROCINFOTABLE) ).get( (String) getValue(HtmlConstants.PROCESSORID) ) ) ).getProcessorName();
		}
	
		resp.append( getSessionIdHtmlField ( ) ).append("\n");//$NON-NLS-1$
		resp.append( getApplicationIdHtmlField()).append("\n");
		resp.append("<input type=\"hidden\" name=\"").append(HtmlConstants.OPERATIONNAME).append("\" value=\"").append( opName ).append("\" />\n");//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		resp.append("<input type=\"hidden\" name=\"").append(HtmlConstants.PAGEID).append("\" value=\"").append(pageId).append("\" />");//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$


		if ( !(getValue(HtmlConstants.PROCESSORID)==null) && !( (String) getValue(HtmlConstants.PROCESSORID) ).equals("") ) //$NON-NLS-1$
		{
			resp.append("\n");//$NON-NLS-1$
			resp.append("<input type=\"hidden\" name=\"").append(HtmlConstants.PROCESSORSTATE).append("\" value=\"").append(getValue(HtmlConstants.PROCESSORSTATE)).append("\" />\n");//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
			resp.append("<input type=\"hidden\" name=\"").append(HtmlConstants.PROCESSORID).append("\" value=\"").append(getValue(HtmlConstants.PROCESSORID)).append("\" />");//$NON-NLS-3$//$NON-NLS-2$//$NON-NLS-1$
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(com.ibm.dse.automaton.html.DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(com.ibm.dse.automaton.html.DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Could not render necessary fields to the JSP.") ;//$NON-NLS-1$
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;						
		}
	}
	
	return resp.toString();
}
/**
 * Returns a string containing the HTML for all of the
 * HTML Client required fields using the current processor name
 * gotten from the table of stored processors.
 * The returned String is for use as part of an URL.
 * The format is the needed to be sent via the GET method:
 * <I>param1=value1&param2=value2&...&paramN=valueN</I>
 * @return String - The <I>required fields</I> html in the format needed to send using the GET method.
 */
public String getRequiredHtmlFieldsForGETMethod ()
{
	return getRequiredHtmlFieldsForGETMethod ("",null);//$NON-NLS-1$
}
/**
 * Returns a string containing the HTML for all of the
 * HTML Client required fields using the passed parameters 
 * to fill in the user specifiable information.
 * If no opName is specified, assume that this page is being
 * displayed within a processor flow, so the processor name is
 * got from the table of stored processors.
 * The returned String is for use as part of an URL.
 * The format is the needed to be sent via the GET method:
 * <I>param1=value1&param2=value2&...&paramN=valueN</I>
 * @param parentctx java.lang.String
 * @param opname java.lang.String
 * @return String - The <I>required fields</I> html in the format needed to send using the GET method.
 */
public String getRequiredHtmlFieldsForGETMethod ( String parentCtx, String opName)
{
	StringBuffer resp = new StringBuffer();


	try
	{
		// If no opName is specified, assume that this page is being displayed within a processor flow. Look for the processor name.
		if ( opName == null || opName.equals("") )//$NON-NLS-1$
		{
			if ( getValue(HtmlConstants.PROCINFOTABLE) == null ) //session persistence is used
				opName = ( (com.ibm.dse.automaton.html.HtmlProcessorInfo) ( (javax.servlet.http.HttpSession) Context.getSessionObjectForSession(sessionID) ).getAttribute( (String) getValue(HtmlConstants.PROCESSORID) )).getProcessorName();
			else
				opName = ( (com.ibm.dse.automaton.html.HtmlProcessorInfo) ( ( (Hashtable) getValue(HtmlConstants.PROCINFOTABLE) ).get( (String) getValue(HtmlConstants.PROCESSORID) ) ) ).getProcessorName();
		}
	
		resp.append(HtmlConstants.SESSIONID).append("=").append(sessionID).append("&");//$NON-NLS-2$//$NON-NLS-1$
		resp.append(HtmlConstants.APPLICATIONID).append("=").append(applicationID).append("&");
		resp.append(HtmlConstants.OPERATIONNAME).append("=").append(opName).append("&");//$NON-NLS-2$//$NON-NLS-1$
		resp.append(HtmlConstants.PAGEID).append("=").append(pageId);//$NON-NLS-1$


		if ( !(getValue(HtmlConstants.PROCESSORID)==null) && !( (String) getValue(HtmlConstants.PROCESSORID) ).equals("") ) //$NON-NLS-1$
		{
			resp.append("&");//$NON-NLS-1$
			resp.append(HtmlConstants.PROCESSORSTATE).append("=").append(getValue(HtmlConstants.PROCESSORSTATE)).append("&");//$NON-NLS-2$//$NON-NLS-1$
			resp.append(HtmlConstants.PROCESSORID).append("=").append(getValue(HtmlConstants.PROCESSORID));//$NON-NLS-1$
		}
	}
	catch(Exception e)
	{
		if(Trace.doTrace(com.ibm.dse.automaton.html.DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(com.ibm.dse.automaton.html.DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Could not render necessary fields to the JSP. Problems will occur: " + e.getMessage()) ;//$NON-NLS-1$
		}
	}


	return resp.toString();
}
/**
 * Return the session context for the page which
 * initialized this bean.
 * @return Context - The <I>session context</I> value.
 */
public Context getSessionContext ( )
{
	return sessionContext;
	
} // end getSessionContext ( )
/**
 * Returns the named data element from the session context.
 * <P>If the element is not present in the context, null is returned.
 * @return DataElement - The <I>data element</I> value.
 */
public DataElement getSessionElement ( String name )
{
	return getElement ( name, true );
	
} // end getSessionElement ( String )
/**
 * Returns the current session identifier string "as-is".
 * @return String - The <I>session id</I> value.
 */
public String getSessionId ( )
{
	return sessionID;
	
} // end getSessionId ( )
/**
 * Returns the current session identifier string
 * as an HTML hidden field.
 * @return String - The <I>session id</I> field html.
 */
public String getSessionIdHtmlField ( )
{
	return "<input type=\"hidden\" name=\"" + HtmlConstants.SESSIONID +//$NON-NLS-1$
			"\" value=\"" + sessionID + "\" />";//$NON-NLS-2$//$NON-NLS-1$
	
} // end getSessionIdHtmlField ( )
/**
 * Returns the current session identifier string
 * as a URL parameter.
 * @return String - The <I>session id</I> parameter.
 */
public String getSessionIdParameter ( )
{
	return HtmlConstants.SESSIONID + "=" + sessionID;//$NON-NLS-1$
	
} // end getSessionIdParameter ( )
/**
 * Returns the specified value from the session context as a string.
 * <P>Handles exceptions by returning a null string value.
 * @return String - The <I>session string</I> value.
 */
public String getSessionStringValue ( String name )
{
	return getStringValue ( name, true );
	
} // end getSessionStringValue ( String name )
/**
 * Returns the named value from the session context in its
 * natural format.
 * <P>If the element is not present in the context,
 * null is returned.
 * @return String - The <I>session</I> value.
 */
public Object getSessionValue ( String name )
{
	return getValue ( name, true );
	
} // end getSessionValue ( String )
/**
 * Returns the specified value from the operation context as a string.
 * If the element is not present in the operation context it is looked 
 * for in the session context.
 * <P>Handles exceptions by returning a null string value.
 * @return String - The <I>string</I> value.
 */
public String getStringValue ( String name )
{
	Object value = new String("");//$NON-NLS-1$
	DataElement de = null;
	try 
	{
		de = operationContext.tryGetElementAt(name);
		if ( de == null )
		{
			de = sessionContext.getElementAt(name);
		}


		PropertyDescription pd = de.getDescriptor();
	
		if (pd != null)
			value = pd.formatToString( de.getValue(), "default" );//$NON-NLS-1$
		else
			value = de.getValue().toString();
	}
	catch (Exception exc) 
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Warning))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Warning, Settings.getTID(), "Dataname not found in context hierarchy: "+name);//$NON-NLS-1$
		}
		setJspFormatError( exc, name );
	}
	return (String) value;
}
/**
 * Returns the specified value from the operation context as a string depending on the "use session" condition
 * 
 * @return java.lang.String
 * @param ctx com.ibm.dse.base.Context
 * @param name java.lang.String
 * @deprecated Replaced by <code>getStringValue(String)</code>.
 */

protected String getStringValue ( String name, boolean useSess )
{
	Context ctx = (useSess) ? sessionContext : operationContext;
	String val;
	
	try
	{
		DataElement de = ctx.getElementAt ( name );
		PropertyDescription pd = de.getDescriptor ( );
		val = (pd != null)
			? pd.formatToString ( de.getValue ( ), "default" )//$NON-NLS-1$
			: de.getValue ( ).toString ( );
		
	} // end try block


	catch ( Exception xcp )
	{
		setJspFormatError ( xcp, name, useSess );
		val = new String ( "" );//$NON-NLS-1$
		
	} // end catch general purpose exceptions
	
	return (String) val;
	
} // end getStringValue ( String, boolean )
/**
 * Returns all validation errors in a single list array.
 * @return String[] - The <I>validation error</I> list.
 */
public String[] getValidationErrorList ( )
{
	Context aContext;
	if (errsVal == null)
	{
		Vector verr = new Vector();
		Hashtable nestedElements = operationContext.getKeyedCollection ( ).calculeNestedQualifiedElements();
		Enumeration enu1 = (Enumeration)nestedElements.keys();
		while(enu1.hasMoreElements( ))
		{
			DataElement de = (DataElement) nestedElements.get(enu1.nextElement ( ));
			ErrorInfo err = de.getErrorInfo ( );
			if(err != null){
				verr.addElement(translateErrors(err));
			}
		}

		aContext = sessionContext;
		while (!(aContext == null)) {
			Hashtable nestedElements2 = aContext.getKeyedCollection ( ).calculeNestedQualifiedElements();
			Enumeration enu2 = (Enumeration)nestedElements2.keys(); 
			while(enu2.hasMoreElements ( ))
			{
				DataElement de = (DataElement) nestedElements2.get(enu2.nextElement ( )); 
				ErrorInfo err = de.getErrorInfo ( );
				if (err != null)
				{
					verr.addElement(translateErrors(err));
				}
			}
				
			aContext = aContext.getParent();
		}
		
		int index = 0; int idx = 0;
		
		//errsVal = new String[verr.size ( )]; int idx = 0;
		for ( Enumeration enu = verr.elements ( ); enu.hasMoreElements ( ); )
		{
			index = index +((String[])enu.nextElement( )).length;
		}
		errsVal = new String[index];
		for ( Enumeration enu = verr.elements ( ); enu.hasMoreElements ( ); )
		{
			String[] aux = (String[])enu.nextElement();
			for(int i=0;i<aux.length;i++){		
				errsVal[idx+i]=aux[i];
			}
			idx=idx+aux.length; 
		}
	}
	return errsVal;
}
/**
 * Returns the named value from the operation context
 * in its natural format. If the element is not present in
 * the operation context, it is looked for in the session
 * context.
 * <P>If the element is not present in the context hierarchy,
 * null is returned.
 * @return Object - The <I>context</I> value.
 */
public Object getValue ( String name )
{
	Object value = null;//operationContext.getValue("dse_allErrors")
	try 
	{
		try 
		{ 
			value=operationContext.getValueAt(name);
		}
		catch (DSEObjectNotFoundException e) 
		{
			if (applicationSessionContext!=null)
			{
				try
				{
					value=applicationSessionContext.getValueAt(name);
				}
				catch (DSEObjectNotFoundException ex)
				{
					value=sessionContext.getValueAt(name);
				}
			}
			else
				value=sessionContext.getValueAt(name);
		}
	}
	catch (Exception exc) 
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Warning))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Warning, Settings.getTID(), "Dataname not found in context hierarchy: "+name);//$NON-NLS-1$
		}
		setJspFormatError ( exc, name );
	}
	return value;
}
/**
 * Returns the named value from the operation context
 * 
 * @return java.lang.Object
 * @param name java.lang.String
 * @param useSess boolean
 * @deprecated Replaced by <code>getValue(String)</code>.
 */


protected Object getValue ( String name, boolean useSess )
{
	Context ctx = (useSess) ? sessionContext : operationContext;
	Object val;
	
	try { val = ctx.getValueAt ( name ); }
	catch ( Exception xcp )
	{
		setJspFormatError ( xcp, name, useSess );
		val = null;
		
	} // end catch general purpose exceptions
	
	return val;
	
} // end getValue ( String, boolean )
/**
 * Gets the webApplURI
 * @return Returns a String
 */
public String getWebApplURI() {
 
      return webApplURI;
}
/**
 * The HttpServletRequest is supposed to have references to the operation
 * context and exceptions placed by the toolkit during handling of
 * the request. Use this information to initialize the bean for use by the
 * JSP that includes it.
 *
 * @param request javax.servlet.http.HttpServletRequest
 */
public void initialize ( HttpServletRequest request ) throws DSEException
{
	setOperationAndSessionContext ( request );

    setWebAttributes ( request );
	try
	{
		setException ( (DSEException) request.getAttribute ( HtmlConstants.EXCEPTION ) );
	}
	catch (ClassCastException e) 
	{
		Exception exc = (Exception) request.getAttribute ( HtmlConstants.EXCEPTION );
		setException ( new DSEException ( DSEException.critical, null, exc.getMessage() ) );
	}
	
} // end initialize ( HttpServletRequest )
/**
 * Returns the "hidden" status of the named field in the context.
 * @return boolean - The <I>field hidden</I> indicator.
 */
public boolean isFieldHidden ( String name )
{
	try
	{
		return getElement(name).getState().isHidden();
	}
	catch ( Exception xcp ) 
	{
		return false; 
	}


} // end isFieldHidden ( String name );
/**
 * Returns the error status of the named field in the context.
 * @return boolean - The <I>field in error</I> indicator.
 */
public boolean isFieldInError ( String name )
{
	boolean iserr;
	
	try 
	{
		DataElement de = getElement ( name );
		iserr = (de.getErrorInfo ( ) != null);
		
	} catch ( Exception xcp ) { iserr = false; }
	
	return iserr;
	
} // end isFieldInError ( String name );
/**
 * Returns the "input mandatory" status of the named field in the context.
 * @return boolean - The <I>field "is mandatory"</I> indicator.
 */
public boolean isFieldMandatory ( String name )
{
	try 
	{
		return getElement(name).getState().isMandatory();
	}
	catch ( Exception xcp )
	{
		return false; 
	}
} // end isFieldMandatory ( String name );
/**
 * Returns the "read only" status of the named field in the context.
 * @return boolean - The <I>field "read only"</I> indicator.
 */
public boolean isFieldReadOnly ( String name )
{
	try 
	{
		return getElement(name).getState().isReadOnly();
	} 
	catch ( Exception xcp ) 
	{
		return false;
	}
} // end isFieldReadOnly ( String name );
/**
 * Returns whether there has been any error during the validation process
 * @return boolean
 */
public boolean isThereAnyError()
{
	try {
		return ((Boolean) operationContext.getValueAt(HtmlConstants.ERRORFLAG)).booleanValue();
	} catch (Exception e) {
		if(Trace.doTrace(com.ibm.dse.automaton.html.DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(com.ibm.dse.automaton.html.DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Internal field 'dse_ErrorFlag' not found in operation context") ;//$NON-NLS-1$
			}
		return false;}
	
}
/** 
 * Removes a data element from the the transition context for the specified transition
 * @param String - The transition name
 * @param String - The name of the data element to be removed
 */ 
public void removeDataElementFromTransitionContext ( String transitionName, String dataName) throws DSEException
{
	com.ibm.dse.automaton.html.HtmlProcessorInfo info = null ;
 	com.ibm.dse.automaton.html.HtmlStateInfo stateInfo = null;
	try
	{
		info = HtmlProcessorManager.getUniqueHTMLInstance().getInfo(sessionID, processorID);
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getInfo: No persistent data could be loaded for given processor id: " + processorID) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;			
		}
	}

	if ( info == null )	throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve processor information for sessionId: " + sessionID + " and processorId: "+ processorID );

	String theProcessorName = info.getProcessorName();
	stateInfo = info.getCurrentStateInfo() ;
	if (stateInfo!=null) stateInfo.removeDataElementFromTransitionContext(transitionName, dataName);
	else throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve currentState information for sessionId: " + sessionID + " and processorId: "+ processorID );

}
/**
 * Sets up the error decorator prefix.
 * <P>This prefix is prepended to any error
 * information returned in a "decorated" error string.
 * @return void - Nothing
 */
public void setErrorDecorationPrefix ( String pfx )
{
	errorPrefix = pfx;
	return;
	
} // end setErrorDecorationPrefix ( String )
/**
 * Sets up the error decorator suffix.
 * <P>This suffix is appended to any error
 * information returned in a "decorated" error string.
 * @return void - Nothing
 */
public void setErrorDecorationSuffix ( String sfx )
{
	errorSuffix = sfx;
	return;
	
} // end setErrorDecorationSuffix ( String )
/**
 * Sets up the error marker value.
 * <P>This is the string that is returned by the
 * "getFieldErrorMarker" method if the field is
 * found to be in error.
 * @return void - Nothing
 */
public void setErrorMarker ( String newmarker )
{
	errorMarker = newmarker;
	return;
	
} // end setErrorMarker ( String newmarker )
/**
 * If the toolkit placed an exception in the HttpServletRequest with key name
 * <b>dseexception</b>, then it can be saved here when the bean is initialized.
 * 
 * @param newException com.ibm.dse.base.DSEException
 */
protected void setException ( DSEException newException )
{
	dseexception = newException;
	
} // end setException ( DSEException )
/**
 * Updates the JspFormatErrorList with information about the error occurred.
 * 
 * @param text java.lang.String
 */
protected void setJspFormatError ( Exception xcp, String name )
{
	String emsg = null;


	if (xcp instanceof DSEObjectNotFoundException)
		emsg = resResources.getString("HTML003") + "\" field: " + name ; //$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$ //$NON-NLS-1$


	else if (!(xcp instanceof NullPointerException))
		emsg = resResources.getString("HTML004") + "\" field: " + name ; //$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$ //$NON-NLS-1$


	if (emsg != null) setJspFormatError ( emsg );
	return;
	
}
/**
 * Updates the JspFormatErrorList with information about the error occurred.
 * 
 * @param text java.lang.String
 * @deprecated Replaced by <code>setJspFormatError(Exception ,String)</code>.
 */

protected void setJspFormatError ( Exception xcp, String name, boolean useSess )
{
	String emsg = null;
	String ctxn = (useSess) ? "session" : "operation";//$NON-NLS-2$//$NON-NLS-1$


	if (xcp instanceof DSEObjectNotFoundException)
		emsg = resResources.getString("HTML003") + "\" field: " + name + "\", context: "+ ctxn ; //$NON-NLS-5$ //$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$ //$NON-NLS-1$


	else if (!(xcp instanceof NullPointerException))
		emsg = resResources.getString("HTML004") + "\" field: " + name + "\", context: " +  ctxn;  //$NON-NLS-5$ //$NON-NLS-4$//$NON-NLS-3$//$NON-NLS-2$ //$NON-NLS-1$


	if (emsg != null) setJspFormatError ( emsg );
	return;
	
}
/**
 * Updates the JspFormatErrorList with information about the error occurred.
 *
 * @param text java.lang.String
 */
protected void setJspFormatError ( String text )
{
	if (errsJsp == null) errsJsp = new Vector ( );
	errsJsp.addElement ( text );
	return;
	
}
/**
 * Save the operation context, and use it to obtain the session context, which is saved as well.
 * 
 * @param newOperationContext com.ibm.dse.base.Context
 */
protected void setOperationAndSessionContext ( HttpServletRequest req ) throws DSEException
{
	// Attempt to retrieve any pre-existing operation context from
	// the incoming request.  If there is an operation context in 
	// effect then get the session identifier from it.
	operationContext = (Context) req.getAttribute ( HtmlConstants.OPERATIONCONTEXT );
	if (operationContext != null){
		sessionID = (String) operationContext.getValueAt ( HtmlConstants.SESSIONID );
		applicationID = (String) operationContext.getValueAt ( HtmlConstants.APPLICATIONID );
		try {
		processorID=(String) operationContext.getValueAt ( HtmlConstants.PROCESSORID);
		} catch (DSEException e) {
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			//Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Process ID not found in operation context") ;//$NON-NLS-1$
		}	
		}
		
	}
	else
	{
		// If no operation context exists then we create a default
		// empty one and get the session identifier from the request.
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Error, Settings.getTID(), "The operation context in request is null. Setting a new empty one") ;//$NON-NLS-1$
		}		
		operationContext = new Context();
		operationContext.setName("dse_operationContext");//$NON-NLS-1$
		sessionID = req.getParameter ( HtmlConstants.SESSIONID ); //CSConstants.SESSIONIDKEY );
		applicationID = req.getParameter ( HtmlConstants.APPLICATIONID );


	} // end if there is no existing operation context


	// If the session id is null, throw an exception.
	if (sessionID==null)
	{
	Throwable t = (Throwable) req.getAttribute(HtmlConstants.EXCEPTION);
		if (t==null)
			throw new DSEException(DSEException.critical,"Session id lost"," The response could not be posted to client because there is no sessionId neither in the operation context nor in the request object.");//$NON-NLS-2$//$NON-NLS-1$
		else
			throw new DSEException(DSEException.critical,"Session id lost",t.getMessage());//$NON-NLS-1$
	}
	
	// The operation context is supposed to include the sessionId or applicatioId too.
	// Use it now to obtain the session context or application session context too.
	
	try
	{	
		if (applicationID == null || applicationID.equals("-1"))
			applicationID = HtmlConstants.IGNORE;
		else
			applicationSessionContext = Context.getCurrentContextForSession(sessionID,applicationID);

		sessionContext = Context.getCurrentContextForSession ( sessionID );
		try {
			// Get the base web path either from the appSession context, the session context or the op context
			if(!(applicationID.equals(HtmlConstants.IGNORE)))
				baseWebPath = (String) applicationSessionContext.getValueAt(HtmlConstants.JSPBASEPATH);
			else if(sessionContext != null)
				baseWebPath = (String) sessionContext.getValueAt(HtmlConstants.JSPBASEPATH);
			else
				baseWebPath = (String) operationContext.getValueAt(HtmlConstants.JSPBASEPATH);
		}
		catch (DSEObjectNotFoundException e) {
			try {
				// If it has not been found in any of the contexts try to get it directly from Settings
				baseWebPath = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFILEPATHKEY);
			}
			catch(DSEException exc) {
				throw new DSEInvalidRequestException(DSEException.critical,HtmlConstants.SESSIONID,"It has not been possible to get a valid baseWebPath. Check the attribute: "+HtmlConstants.SETTINGSFILEPATHKEY+" in the configuration file.");
			}
		}
	}
	catch(DSEException exc)
	{
		// The session id does not exist create a specifical exception and throw it.
			try{
				baseWebPath = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFILEPATHKEY);
			}catch(DSEException exce){
				throw new DSEInvalidRequestException(DSEException.critical,HtmlConstants.SESSIONID,"Invalid session id: "+sessionID+". Run the CSEstablishSessionServlet to obtain a valid session");
			}
		
	}

	

	// Also, the session context has the extra path info saved in it. Retain this for
	// later use.
	try
	{
		if(!(applicationID.equals(HtmlConstants.IGNORE)))
			extraPathInfo = (String) applicationSessionContext.getValueAt(HtmlConstants.EXTRAPATHINFO);
		else if(sessionContext != null)
			extraPathInfo = (String) sessionContext.getValueAt(HtmlConstants.EXTRAPATHINFO);
		else
			extraPathInfo = (String) operationContext.getValueAt(HtmlConstants.EXTRAPATHINFO);
	}
	catch(DSEObjectNotFoundException dseobj)
	{
		// No extra path info... do nothing. Could an expired session.
	}

	// Get the pageId for the displaying page. If the session ctx does not have it stored,
	// we must get a new one from the one stored with the session
	try
	{
		if(!(applicationID.equals(HtmlConstants.IGNORE)))
			pageId = (String) applicationSessionContext.getValueAt(HtmlConstants.PAGEID);
		else if(sessionContext != null)
			pageId = (String) sessionContext.getValueAt(HtmlConstants.PAGEID);
	}
	catch ( DSEObjectNotFoundException noPageId )
	{
		try
		{
			pageId = RequestValidationServices.getNextPageId(sessionID);
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug))
				Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug, Settings.getTID(), "Page id not present in operation context. Using page id from RequestValidationServices.");//$NON-NLS-1$
		}
		catch(Exception exc)
		{
			//Do nothing. Probably the session has expired.
		}
	}
} // end setOperationAndSessionContext ( HttpServletRequest )
/** 
 * Sets a parameter given by paramName with a value given by paramValue for a transitionName of this state
 * @param String - The transitionName
 * @param String - The parameter name 
 * @param String - The parameter value 
 */ 
public void setTransitionParameter(String transitionName, String paramName, String paramValue) throws DSEObjectNotFoundException
{
	com.ibm.dse.automaton.html.HtmlProcessorInfo info = null ;
 	com.ibm.dse.automaton.html.HtmlStateInfo stateInfo = null;
	try
	{
		info = HtmlProcessorManager.getUniqueHTMLInstance().getInfo(sessionID, processorID);
	}
	catch(Exception e)
	{
		if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error))
		{
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "HtmlProcessorManager.getInfo: No persistent data could be loaded for given processor id: " + processorID) ;
			Trace.trace(DSEHtmlProcessor.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), e) ;			
		}
	}


	if ( info == null )
		throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve processor information for sessionId: " + sessionID + " and processorId: "+ processorID );


	// get the processor currentState info
	stateInfo = info.getCurrentStateInfo() ;
	if (stateInfo!=null) stateInfo.setTransitionParameter(transitionName, paramName, paramValue);
	else throw new DSEObjectNotFoundException(DSEException.critical, DSEHtmlProcessor.COMPID, "It has not been possible to retrieve currentState information for sessionId: " + sessionID + " and processorId: "+ processorID );
}
/**
 * This method sets the URI attributes from the request
 * @param request javax.servlet.http.HttpServletRequest
 */

public void setWebAttributes ( HttpServletRequest request ) throws DSEException
{
       //Getting the URL from the request
       String uri = request.getRequestURI();
       String server = request.getServerName();
       int portInt = request.getServerPort();
       String ctxRoot = request.getContextPath();
       
       Integer port = new Integer(portInt);
              
       //Setting the webApplURI
       webApplURI = "http://"+server+":"+port.toString()+ctxRoot;
       
       //Setting the baseWebApplURIPath
       int index2 = uri.lastIndexOf("/");
       
       baseWebApplURIPath = uri.substring(0,index2);
       
}
/**
 * Translate the field error message
 * @param errInfo The errorInfo of the dataElement
 * @return String[] Array of translated messages
 */

public String[] translateErrors(ErrorInfo errInfo){
	
	
	int i = 0;
	java.util.Locale locale = (java.util.Locale)getValue(HtmlConstants.LOCALE);
		
	Enumeration enum = errInfo.keys();
	String[] msgArray = new String[errInfo.size()];
	while(enum.hasMoreElements()){
		
		String code = (String)enum.nextElement();
		String msg = errInfo.get(code,locale);
		msgArray[i] = msg;
		i++;
	}
	
	return msgArray;
}
} // end class DSEJspContextServices
