package com.ibm.dse.cs.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import java.io.*;
import java.net.SocketException;

import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.dse.clientserver.*;
import com.ibm.dse.base.*;
import com.ibm.dse.cs.servlet.CSConstants;

import com.ibm.dse.automaton.InterStateFormatEnabledProcessor;
import com.ibm.dse.automaton.InterStateFormatSupport;
import com.ibm.dse.automaton.html.DSEHtmlProcessorOperation;

/**
 * Forward responses from the Html Client component to the client browser.
 * Responses are forwarded to either the reply page or the error page as
 * specified by the completed operation context or by information saved
 * in the HttpServletRequest by the toolkit.
 * @copyright(c) Copyright IBM Corporation 2000, 2003.
 */
public class HtmlPresentationHandler extends DSEPresentationHandler
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2003 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
	private static java.util.ResourceBundle resResources = java.util.ResourceBundle.getBundle("htmlclient");  //$NON-NLS-1$
/**
 * HtmlPresentationHandler default constructor.
 */
public HtmlPresentationHandler() {
	super();
}
/**
 * This method adds toolkit related fields to a context.
 * 
 */
public void addFieldTo(Context ctx, String name, Object value) throws Exception
{
	DataField df = new DataField();
	df.setName(name);
	df.setValue(value);
	ctx.addElement(df);
}
/**
 * The session context requires some data to be added to it.
 * 
 * @param sessionId java.lang.String
 * @param req HttpServletRequest 
 */
protected void addRequiredDataToSessionCtx(String sessionId, HttpServletRequest req)
{
	//------
	// Place the value from Settings (dse.ini) specified for filePath. This path
	// will be used as a relative path from the web root directory to locate
	// web resources (ie JSP files).
	//------
	try
	{
		// add the sessionId
		Context seCtx = Context.getCurrentContextForSessionOrApplication(sessionId);
		addFieldTo(seCtx,HtmlConstants.SESSIONID,sessionId);

		// add the jspBasePath and the sessionExtraPathInfo
		String jspBasePath = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFILEPATHKEY);
		String sessionExtraPathInfo = req.getPathInfo();

		if (sessionExtraPathInfo==null)
		{
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug, Settings.getTID(), "There is no path information. The JspBasePath used is: "+jspBasePath);
			}
		}
		else	
			jspBasePath = jspBasePath + sessionExtraPathInfo.substring(1) + "/";
		
		addFieldTo(seCtx,HtmlConstants.JSPBASEPATH,jspBasePath);

		addFieldTo(seCtx,HtmlConstants.EXTRAPATHINFO,sessionExtraPathInfo);
	}
	catch (Exception e)
	{if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Error))
		{Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "Exception adding required infrastructure fields to the session context.");
		 Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;}
	}
}
/**
 * Obtains a RequestDispatcher for the replyPageName passed in and forwards
 * the response to this page. Once forwarded the page, this method flushes 
 * the buffer in order to force the launching of an exception if other request
 * from same client is in process. Doing so double clicks are detected. The entry 
 * in the pendingRequestsTable is updated to let further requests notice that they 
 * are product of abnormal navigation.
 * 
 * @param replyPageName java.lang.String The name of the page to be forwarded
 * @param req javax.servlet.HttpServletRequest Request object
 * @param res javax.servlet.HttpServletResponse Response object
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
protected void forward(String replyPageName, HttpServletRequest req, HttpServletResponse res, ChannelContext channelContext)
{
	try
	{
		// Forward to the next page as passed by the caller.
		ServletContext sc = ((HttpServlet) (channelContext.getChannelDriver())).getServletConfig().getServletContext();
		sc.getRequestDispatcher(replyPageName).forward(req, res);
		res.flushBuffer();
		
		/** <HSBC> **/
		clearUp(req);
		/** </HSBC> **/
		
		// If arrived until here without launching an IOException and the current request
		// is product of a double click, need to mark it as answered, to allow the deletion of PRT entry
		// if current request is not double click, marking does nothing. If the request is for the establish session servlet, the request must not be marked, though
		try
		{
			getRVS(channelContext).markAsAnswered(channelContext);
		}
		catch (DSEException e)
		{
			if (Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Information))
				Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Information, Settings.getTID(), "Not able to mark the request as answered, if it is a session establish request or a logoff request this message must be ignored. Exception: " + e);
		}
	}
	catch (ServletException se)
	{
		try
		{
			getRVS(channelContext).markAsAnswered(channelContext);
			if (replyPageName.endsWith((String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSERRORPAGEKEY)))
				handleException(channelContext, se);
			else
				processException(channelContext, se);
		}
		catch (Exception ex)
		{
			handleException(channelContext, se);
		}
	}
	catch (IOException io)
	{
		if (io instanceof SocketException)
		{
			// Got a double click, there is someone else waiting execution from same client
			// Mark the entry in the pending table to let know the waiting one that it has been already executed.
			// After that, just finish the execution normally.
			try
			{
				getRVS(channelContext).markAsNotAnswered(channelContext);
			}
			catch (DSEException dse)
			{
				if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
					Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception marking the request as not answered: " + dse);
			}
		}
		else
		{
			try
			{
				getRVS(channelContext).markAsAnswered(channelContext);
				if (replyPageName.endsWith((String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSERRORPAGEKEY)))
					handleException(channelContext, io);
				else
					processException(channelContext, io);
			}
			catch (Exception ex)
			{
				handleException(channelContext, io);
			}
		}
	}
	
	// add for IY61130 start
	catch (Exception se)
	{
		try
		{
			getRVS(channelContext).markAsAnswered(channelContext);
			if (replyPageName.endsWith((String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSERRORPAGEKEY)))
				handleException(channelContext, se);
			else
				processException(channelContext, se);
		}
		catch (Exception ex)
		{
			handleException(channelContext, se);
		}
	}
	// add for IY61130 end
}
/**
 * Returns the RequestValidationServices instance associated with the current session.
 * @return RequestValidationServices RequestValidationServices object associated with given session
 * @param cc com.ibm.dse.clientserver.ChannelContext ChannelContext used to get the session
 * @throws DSEException Either it is not possible to retrieve a sessionId from the channelContext or there is no session associated with the sessionId.
 */
protected RequestValidationServices getRVS(ChannelContext cc) throws DSEException
{
	String sessionId = getSessionId(cc);
	if (sessionId!=null)
		return (RequestValidationServices) Context.getSession(sessionId).getValueAt(HtmlConstants.RVS);
	else
		throw new DSEInvalidRequestException(DSEException.critical, HtmlConstants.SESSIONID, "ChannelContext does not contain ChannelSession or it does not contain any sessionId.");
}
/**
 * Obtains the session id from the channel context.
 * Returns the sessionId or null if there is no sessionId associated with this ChannelContext.
 * @return java.lang.String
 * @param aChannelContext com.ibm.dse.clientserver.ChannelContext
 */
protected String getSessionId(ChannelContext aChannelContext)
{
	String sessionId = null;
	if (aChannelContext.getChannelSession() instanceof HttpSession)
	{
		sessionId = ((HttpSession) aChannelContext.getChannelSession()).getId();
	}
	else if (aChannelContext.getChannelSession() instanceof ChannelSession)
	{
		sessionId = ((ChannelSession) aChannelContext.getChannelSession()).getId();
	}
	return sessionId;
}

/**
 * <HSBC>
 * Clean up context and session after forwarded JSP
 */
protected void clearUp(HttpServletRequest req) {
	
	// Check for state context and remove it
	try {
		Context ctxt = (Context) req.getAttribute(HtmlConstants.OPERATIONCONTEXT);
		Boolean ind = (Boolean) ctxt.getValueAt(Constants.StateContextInd);
		if (ind != null && ind.equals(Boolean.TRUE)) {
			ctxt.unchain();
		}
	} catch (Exception e) {
		// DO Nothing
	}
	
	// Remove operation context after forwarded JSP
	req.removeAttribute(HtmlConstants.OPERATIONCONTEXT);
}

/**
 * When an error occurs attempting to return to the client browser,
 * it may not be possible to return the error. If we have access to an
 * HtmlServletResponse, we will use it. Otherwise we just record the
 * error to the System.err stream.
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 * @param e java.lang.Exception
 */
public void handleException(ChannelContext channelContext, Exception e)
{

	// See if we have a response to use.
	HttpServletResponse res = (HttpServletResponse) channelContext.getChannelResponse();
	PrintStream out = null;
	
	if (res != null)
	{
		try
		{
			PrintWriter htmlOut = res.getWriter();
			htmlOut.println("<h2>" + resResources.getString("HTML002") + "</h2>");//$NON-NLS-3$ //$NON-NLS-2$//$NON-NLS-1$
			htmlOut.println ("<br>" + "HtmlPresentationHandler:handleException():");//$NON-NLS-2$//$NON-NLS-1$
			htmlOut.println ("<br><br>" + resResources.getString("HTML000")+e.toString()); //$NON-NLS-2$//$NON-NLS-1$
			htmlOut.println ("<br><hr>" + resResources.getString("HTML001") + "<br>");//$NON-NLS-3$ //$NON-NLS-2$//$NON-NLS-1$
			e.printStackTrace(htmlOut);
		}
		catch (IOException exc){
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when writing to the output stream") ;//$NON-NLS-1$
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), exc) ;			
			}			
		}
		
	}
	else
	{
		out = System.err;
		out.println ("HtmlPresentationHandler:handleException():");//$NON-NLS-1$
		out.println ("Exception message: "+e.toString());//$NON-NLS-1$
		out.println ("Exception stack trace:");//$NON-NLS-1$
		e.printStackTrace(out);
	}
	 
	
}
/**
 * Obtain the error page name from DSE.INI settings and forward the
 * response to that page.
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 * @param e java.lang.Exception
 */
public void processException(ChannelContext channelContext, Exception exception)
{
	
	String sessionId = null;
	DataField jspBasePath = null;
	HttpServletRequest req = (HttpServletRequest) channelContext.getChannelRequest();
	HttpServletResponse res = (HttpServletResponse) channelContext.getChannelResponse();

	// Place the exception object in the request so the error page has access to
	// it.
	req.setAttribute(HtmlConstants.EXCEPTION, exception);

	try
	{
		// Obtain the error page name from the Settings (dse.ini)
		String errorPage = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSERRORPAGEKEY);

		
		Context theContext = (Context)req.getAttribute(HtmlConstants.OPERATIONCONTEXT);
		
		//------
		// Obtain the jspBasePath, which is relative to the web application web root,
		// and apply it to the name of the file which is to be displayed.
		//------
		if (theContext != null)
		{
			sessionId = (String) theContext.getValueAt(HtmlConstants.SESSIONID);
		}
		else
		{
			sessionId = req.getParameter(HtmlConstants.SESSIONID);
		}

		if (sessionId != null)
		{
			Context sessionContext = null;
			try
			{
				sessionContext = Context.getCurrentContextForSessionOrApplication(sessionId);
			}
			catch (DSEObjectNotFoundException e)
			{
				try{
					
					if(theContext != null){
						jspBasePath = (DataField)theContext.getElementAt(HtmlConstants.JSPBASEPATH);
					}else{
						jspBasePath = new DataField();
						jspBasePath.setValue(Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFILEPATHKEY));
					}
					
				}catch(Exception exc){
					handleException(channelContext, exception);
					return;
				}
			}
			
			try{
				if(sessionContext != null){
					jspBasePath = (DataField)sessionContext.getElementAt(HtmlConstants.JSPBASEPATH);
				}else{
					if(theContext != null){
						jspBasePath = (DataField)theContext.getElementAt(HtmlConstants.JSPBASEPATH);
					}
				}
			}catch(DSEObjectNotFoundException exc){
				//Maybe this error occurrs in the startup sequence and the JSPBASEPATH is not yet
				//in the session context so probably is in the operation context
				jspBasePath = (DataField)theContext.getElementAt(HtmlConstants.JSPBASEPATH);
				addRequiredDataToSessionCtx(sessionId,req);
			}
				

			String jspFullName = (String)jspBasePath.getValue()+errorPage;
			
			
			forward(jspFullName, req, res, channelContext);
		}
		else
		{
			handleException (channelContext, exception);
		}
	}
	catch (Exception e2)
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when processing an exception") ;//$NON-NLS-1$
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e2) ;			
		}		
		handleException(channelContext, e2);
	}
}
/**
 * Obtain the name of the page to display and forward the request to that page.
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 * @param operation com.ibm.dse.base.ServerOperation 
 */
public void processReply(ChannelContext channelContext, com.ibm.dse.base.ServerOperation operation) {

	DataElement jspName = null, jspBasePath = null;
	Context theContext = null, sessionContext = null;

	HttpServletRequest req = (HttpServletRequest) channelContext.getChannelRequest();
	HttpServletResponse res = (HttpServletResponse) channelContext.getChannelResponse();

	try
	{
		/** <HSBC> **/
		// Clear Action Context from Processor
		if (operation instanceof DSEHtmlProcessorOperation) {
			if (((DSEHtmlProcessorOperation) operation).getProcessor() instanceof InterStateFormatEnabledProcessor) {
				InterStateFormatSupport.clearInfoForFormat((InterStateFormatEnabledProcessor) ((DSEHtmlProcessorOperation) operation).getProcessor());
			}
		}
		/** </HSBC> **/
		
		if (operation != null) 
		{
			theContext = operation.getContext();
			// Get the name to forward to...
			jspName = theContext.getElementAt(HtmlConstants.REPLYPAGE);
		}
		else 
		{
			// Some exceptions may result in the operCtx already set
			// in the request. DSETypeExceptions cause this to occur, with
			// a call to this method with a null operation.
			theContext = (Context) req.getAttribute(HtmlConstants.OPERATIONCONTEXT);
			try 
			{
				jspName = theContext.getElementAt(HtmlConstants.ERRORPAGE);
			}
			catch (DSEException e) 
			{
				DSEException dseXcp = (DSEException) req.getAttribute(HtmlConstants.TYPEEXCEPTION);
				processException(channelContext, dseXcp);
				return;
			}
		}

		//------
		// Obtain the jspBasePath, which is relative to the web application web root,
		// and apply it to the name of the file which is to be displayed.
		// By default it is stored in the session ctx, but if we are returning from
		// an op of ending session, this op has the responsibility of copying the field
		// in the op context.
		// If the op ctx does not have this data, it is caught from Settings, but it
		// will not have the extra path info added from the establish session.
		//------
		String sessionId = (String) theContext.getValueAt(HtmlConstants.SESSIONID);
		String applicationId = (String) theContext.getValueAt(HtmlConstants.APPLICATIONID);
		try
		{
			if(!(applicationId.equals(HtmlConstants.IGNORE))){
				sessionContext = Context.getCurrentContextForSession(sessionId,applicationId);
			}else{
				sessionContext = Context.getCurrentContextForSession(sessionId);
			}

			if (sessionContext != null) {
				jspBasePath = (DataField) sessionContext.getElementAt(HtmlConstants.JSPBASEPATH);
			} else {
				// Maybe the startup operation couldn't be executed and so the session context is not created
				// then the jspBasePath has been saved in the operation context
				jspBasePath = theContext.getElementAt(HtmlConstants.JSPBASEPATH);
			}
		}
		catch (DSEObjectNotFoundException noSession)
		{
			try
			{
				// If out of session, the operation is responsible of storing the base path in the op ctx
				jspBasePath = (DataField) theContext.getElementAt(HtmlConstants.JSPBASEPATH);
			}
			catch (DSEObjectNotFoundException e)
			{
				// If the session has no base path info, look for it in the Settings. The extrapathinfo will get lost.
				jspBasePath = new DataField();
				jspBasePath.setName(HtmlConstants.JSPBASEPATH);
				jspBasePath.setValue(Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFILEPATHKEY));
			}
		}
		
		
		
		String jspFileName = (String) jspName.getValue();
		if (jspFileName == null || jspFileName.trim().equals(""))//$NON-NLS-1$
		{
			jspFileName = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSHOMEPAGEKEY);
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "No reply page has been found. Displaying the Home Page.") ;//$NON-NLS-1$
			}
		}
			
		String jspFullName = (String) jspBasePath.getValue() + jspFileName;
		// req.getAttribute(HtmlConstants.OPERATIONCONTEXT)
		forward(jspFullName, req, res, channelContext);
		
	}
	catch (Exception e) 
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when processing the reply to the client") ;//$NON-NLS-1$
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}		
		processException(channelContext, e);
	}
}
}
