package com.ibm.dse.cs.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import java.io.*;
import java.util.*;
import javax.servlet.http.*;

import com.ibm.dse.base.*;
import com.ibm.dse.base.types.*;
import com.ibm.dse.clientserver.*;
import com.ibm.dse.cs.servlet.*;
import com.ibm.dse.automaton.html.*;
import java.util.Vector;
/**
 * Handle requests that are made from html clients. The DSE.INI file 
 * for the server specifies this class name as the handler for 
 * html requests. One instance of it is created and held by the 
 * HandlerRegistry for use by all html clients.
 * @copyright(c) Copyright IBM Corporation 2000, 2003.
 */
public class HtmlRequestHandler extends DSERequestHandler
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2003 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$
/**
 * HtmlRequestHandler constructor comment.
 */
public HtmlRequestHandler() {
	super();
}
/**
 * Adds to the session context the information needed to manage the processor's consequentiality.
 * 
 * @param sessionId String
 * @param kcReqData com.ibm.dse.base.KeyedCollection 
 */
protected void addConseqInfoToSessionCtx(String sessionId, KeyedCollection kcReqData)
{
	try
	{
		// add the sessionId
		Context seCtx = Context.getCurrentContextForSession(sessionId);
		// add pageId
		Integer pageId = new Integer(0);
		try
		{
			pageId = new Integer((kcReqData.getValueAt(HtmlConstants.PAGEID)).toString());
		}
		catch (Exception e)
		{
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Warning))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Warning, Settings.getTID(), "Page id not found in received data, using a default page id of 0");
			}
		}
		// add page id
		addFieldTo(seCtx, HtmlConstants.PAGEID, pageId.toString());
	}


	catch (Exception e)
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when adding pageId and callingPageId to the session context") ;
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}		
	}
}
/**
 * Adds to the application session context the information needed to manage the processor's consequentiality.
 * 
 * @param sessionId String
 * @param applicationId String
 * @param kcReqData com.ibm.dse.base.KeyedCollection 
 */
protected void addConseqInfoToSessionCtx(String sessionId, String applicationId, KeyedCollection kcReqData)
{
	try
	{
		// add the sessionId
		Context seCtx = Context.getCurrentContextForSession(sessionId,applicationId);
		// add pageId
		Integer pageId = new Integer(0);
		try
		{
			pageId = new Integer((kcReqData.getValueAt(HtmlConstants.PAGEID)).toString());
		}
		catch (Exception e)
		{
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Warning))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Warning, Settings.getTID(), "Page id not found in received data, using a default page id of 0");
			}
		}
		// add page id
		addFieldTo(seCtx, HtmlConstants.PAGEID, pageId.toString());
	}


	catch (Exception e)
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when adding pageId and callingPageId to the session context") ;
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}		
	}
}
/**
 * Adds to the operation context error information upgrading the 
 * existent array of error messages or creating it if it does not exist.
 * @param oper com.ibm.dse.base.DSEServerOperation The operation to upgrade
 * @param message java.lang.String The message to add to the operation's context
 * @throws DSEInvalidArgumentException
 * @throws DSEInvalidRequestException
 */
protected void addErrorInfo(DSEServerOperation oper, String message) throws DSEInvalidArgumentException, DSEInvalidRequestException
{
	try
	{
		if (oper.getValueAt(HtmlConstants.ERRORMESSAGES) == null)
			addFieldTo(oper.getContext(), HtmlConstants.ERRORMESSAGES, new String[0]);
		
		// Add the error message to the errormessages attribute of the operation's context
		String[] errors = (String[]) oper.getValueAt(HtmlConstants.ERRORMESSAGES);
		String[] errorsModified = new String[errors.length+1];

		// Add the information already existing in the operation to the newly created array
		for (int i=0; i<errors.length; i++)
		{
			errorsModified[i] = errors[i];
		}
		errorsModified[errors.length] = message;
		oper.setValueAt(HtmlConstants.ERRORMESSAGES, errorsModified);
	}
	catch (DSEObjectNotFoundException xcp)
	{
		// A field for error messages not found in the operation context. Adding a new one...
		String[] errors = { message };
		addFieldTo(oper.getContext(), HtmlConstants.ERRORMESSAGES, errors);
	}
}
/**
 * This method creates and adds a toolkit related field to the context passed in.
 * @param ctx com.ibm.dse.base.Context The context where to add the new field
 * @param name java.lang.String The name of the field to create
 * @param value java.lang.Object The value of the field to create
 * @throws Exception
 */
public void addFieldTo(Context ctx, String name, Object value) throws DSEInvalidRequestException
{
	DataField df = new DataField();
	df.setName(name);
	df.setValue(value);
	ctx.addElement(df);
}
/**
 * The operation context requires some data to be added to it.
 * @param opCtx com.ibm.dse.base.Context
 * @param kcReqData com.ibm.dse.base.KeyedCollection 
 */
protected void addRequiredDataToOperationCtx(Context opCtx, KeyedCollection kcReqData)
{
	try
	{
		String applicationId = HtmlConstants.IGNORE;
		// add replyPage
		addFieldTo(opCtx, HtmlConstants.REPLYPAGE, "");
		// add operationName
		addFieldTo(opCtx, HtmlConstants.OPERATIONNAME, "");
		// add parentContextName
		addFieldTo(opCtx, HtmlConstants.PARENTCONTEXTNAME, "");
		// add processorId
		addFieldTo(opCtx, HtmlConstants.PROCESSORID, "");
		// add processorId
		addFieldTo(opCtx, HtmlConstants.PROCESSORSTATE, "");
		// add errorMessages
		addFieldTo(opCtx, HtmlConstants.ERRORMESSAGES, new String[0]);
		// add validation errors flag
		addFieldTo(opCtx, HtmlConstants.ERRORFLAG, new Boolean(false));
		
		// add the sessionId
		String sessionId = (String) kcReqData.getValueAt(HtmlConstants.SESSIONID);
		addFieldTo(opCtx,HtmlConstants.SESSIONID, sessionId);


		// add the applicationId
		try { applicationId = (String) kcReqData.getValueAt(HtmlConstants.APPLICATIONID);} catch (Exception e) {};
		addFieldTo(opCtx,HtmlConstants.APPLICATIONID, applicationId);
		
		// add the errorPage
		String errorPage = "";
		try
		{
			errorPage = (String) kcReqData.getValueAt(HtmlConstants.ERRORPAGE);
		}
		catch (Exception e)
		{ // no error page specified
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Information))
			Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Information, Settings.getTID(), "No error page specified in JSP");
		}
		addFieldTo(opCtx, HtmlConstants.ERRORPAGE, errorPage);
	}
	catch (Exception exc)
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when adding the required infastructure data to the operation context") ;
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), exc) ;			
		}		
	}
}
/**
 * The session context requires some data to be added to it.
 * @param sessionId java.lang.String
 * @param req HttpServletRequest 
 */
protected void addRequiredDataToSessionCtx(String sessionId, HttpServletRequest req)
{
	//------
	// Place the value from Settings (dse.ini) specified for filePath. This path
	// will be used as a relative path from the web root directory to locate
	// web resources (ie JSP files).
	//------
	try
	{
		// add the sessionId
		Context seCtx = Context.getCurrentContextForSessionOrApplication(sessionId);
		addFieldTo(seCtx,HtmlConstants.SESSIONID,sessionId);


		// add the jspBasePath and the sessionExtraPathInfo
		String jspBasePath = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFILEPATHKEY);
		String sessionExtraPathInfo = req.getPathInfo();


		if (sessionExtraPathInfo==null)
		{
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug, Settings.getTID(), "There is no path information. The JspBasePath used is: "+jspBasePath);
			}
		}
		else	
			jspBasePath = jspBasePath + sessionExtraPathInfo.substring(1) + "/";
		
		addFieldTo(seCtx,HtmlConstants.JSPBASEPATH,jspBasePath);


		addFieldTo(seCtx,HtmlConstants.EXTRAPATHINFO,sessionExtraPathInfo);
		
		//NLS managing
		try{
		
			seCtx.getValueAt(HtmlConstants.LOCALE);
		
		}catch(DSEObjectNotFoundException objnf){
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Debug, Settings.getTID(), "No locale defined, using the request locale or default ");
			}
			setLocale(req,seCtx);
		}	
	
		// 
	}
	catch (Exception e)
	{if(Trace.doTrace(HtmlConstants.COMPID, Trace.Medium, Trace.Error))
		{Trace.trace(HtmlConstants.COMPID, Trace.Medium, Trace.Error, Settings.getTID(), "Exception adding required infrastructure fields to the session context.");
		 Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;}
	}
}
/**
 * Returns true if any of the DataElements associated to this context has validation errors.
 * @return boolean
 * @param trCtx com.ibm.dse.base.Context
 */
protected boolean areTransitionCtxErrors(Context trCtx)
{
	if (trCtx!=null)
	{
		DataElement de = null;
		ErrorInfo err = null;


		for ( Enumeration enu = trCtx.getKeyedCollection().getNestedElements().elements(); enu.hasMoreElements ( ); )
		{
			de = (DataElement) enu.nextElement ( );
			err = de.getErrorInfo ( );
			if (err != null) return true;
		}
	}
	return false;	
}
/**
 * Chains the operation or processor context to the right parent. It will be either
 * an application session context, an application sub session context or any other 
 * predefined parent of the operation context.
 * @param oper com.ibm.dse.base.DSEServerOperation
 * @param sessionId java.lang.String
 * @param applicationId java.lang.String
 * @throws DSEInvalidRequestException There is no context associated with given sessionId or applicationId
 */
protected void chainOperation(DSEServerOperation oper, String sessionId, String applicationId) throws DSEInvalidRequestException
{
	Context aContext;
	
	if (oper.getParent() == null)
	{
		if (!(applicationId.equals(HtmlConstants.IGNORE)))
		{
			try
			{
				aContext = Context.getCurrentContextForSession(sessionId, applicationId);
			}
			catch (DSEObjectNotFoundException e)
			{
				// The session id or application id does not exist. Throw an specifical exception.
				throw new DSEInvalidRequestException(DSEException.critical, HtmlConstants.SESSIONID, "Invalid session id or application id: " + sessionId + ". Run the CSEstablishSessionServlet to obtain a valid session");
			}
		}
		else
		{
			try
			{
				aContext = Context.getCurrentContextForSession(sessionId);
			}
			catch (DSEObjectNotFoundException e)
			{
				// The session id does not exist. Throw an specifical exception.
				throw new DSEInvalidRequestException(DSEException.critical, HtmlConstants.SESSIONID, "Invalid session id: " + sessionId + ". Run the CSEstablishSessionServlet to obtain a valid session");
			}
		}
		oper.chainTo(aContext);
	}
}
/**
 * Updates channel context's request data with the appropriate Locale. It is got either from
 * the information sent from client or from the system's default.
 * @param channelContext
 * @exception Exception
 */
public void checkForLocale(ChannelContext channelContext) throws Exception{
	
	KeyedCollection kColl = channelContext.getRequestData();
	try{
		Locale locale = null;
		String stringLocale = (String)kColl.getValueAt(HtmlConstants.LOCALEDATA);
		
		if(stringLocale != null){
		
			String language = stringLocale.substring(0,stringLocale.indexOf("_"));
			String country = stringLocale.substring(stringLocale.indexOf("_")+1,stringLocale.indexOf("_")+3);
		
			locale = new Locale(language,country);
		} else {
			locale = Locale.getDefault();
		}
		kColl.setValueAt(HtmlConstants.LOCALEDATA,locale);
		
	}catch(DSEObjectNotFoundException objnf){
				//Do nothing no locale info is sended.
	}catch(Exception e){
		
		if (Trace.doTrace(CSConstants.COMPID, Trace.High, Trace.Error)) {
				String tid = Settings.getTID();
				Trace.trace(CSConstants.COMPID,Trace.High,Trace.Error, tid, "Some problem occurred when setting the locale");
			}
	}
}
/**
 * Creates an appropriate error message depending on the error type.
 * The error message that will be added to the operation can be customized through dse.ini.
 * This method reads the Settings to assign the right error message depending on the type of error.
 * If there is no customized error message a default one is assigned. There is a different default 
 * message for each different type of error.
 * @param type String the type of abnormal navigation (pending, aging or unconsistent processor)
 */
protected String createErrorMessage(String type)
{
	String message = null;
	try
	{
		if (type.equals(HtmlConstants.PENDINGEXCEPTIONCODE))
			message = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSDOUBLECLICKMESSAGE);
		else if (type.equals(HtmlConstants.AGINGEXCEPTIONCODE))
			message = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSAGINGMESSAGE);
		else if (type.equals(HtmlConstants.INCONSISTENTPROCESSOREXCEPTIONCODE))
			message = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSINCONSISTENTPROCESSORMESSAGE);
	}
	catch (DSEObjectNotFoundException e)
	{
		// No specific message set, setting a predefined one
		if (type.equals(HtmlConstants.PENDINGEXCEPTIONCODE))
			message = "The following request has been received while another request coming from the same page and within the same session is being processed. This request has been considered a double click.";
		else if (type.equals(HtmlConstants.AGINGEXCEPTIONCODE))
			message = "The following request has been received only a short time after another one coming from the same page and within the same session has been processed. This request has been considered a double click.";
		else if (type.equals(HtmlConstants.INCONSISTENTPROCESSOREXCEPTIONCODE))
			message = "Two concurrent requests coming from same source asking for different processor events have been detected, second request has been discarded.";
	}
	return message;
}
/**
 * Performs maintenance of the pending requests table stored in the 
 * requestValidationServices instance associated with this session.
 * Deletes the entry if noone is waiting as a double click, leaves it
 * in the table updating its timestamp if there is minRequestResubmitTime
 * defined and closes the operation if anyone else will need it.
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public void endRequest(ChannelContext channelContext)
{
	try
	{
		getRVS(channelContext).beginAgingRequest(channelContext);
	}
	catch (DSEObjectNotFoundException onf)
	{
		if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Information))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Information, Settings.getTID(), "It has not been possible to close the operation. If current operation is a logoff operation, this message should be ignored. Exception: "+onf);
		}
	}
	catch (Exception e)
	{
		if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when beginAgingRequest is performed. ChannelContext: " + channelContext);
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}
		try
		{
			((PresentationHandler) getPresentationHandler(channelContext)).processException(channelContext, e);
			getRVS(channelContext).deletePendingRequest(getRVS(channelContext).getCompositeKey(channelContext));
		}
		catch (DSENoRegisteredPresentationHandlerException dsee)
		{
			if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception trying to get the registered presentation handler. ChannelContext: " + channelContext);
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
			}
		}
		catch (DSEException exc)
		{
			if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception trying to delete the pending request. ChannelContext: " + channelContext);
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
			}
		}
	}
}
/**
 * Closes the operation.
 * @param anOperation com.ibm.dse.base.ServerOperation
 */
public void endSessionRequest(ServerOperation anOperation)
{
	try
	{
		anOperation.close();
	}
	catch (Exception e)
	{
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when closing the startUpOp operation: "+anOperation);
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}			
	}
}
/**
 * When a server is set up with the cookies flag in DSE.INI set to
 * false, then a DSEChannelSession object is used rather than
 * an HttpSession. The required actions to create, execute,
 * and return an operation using a DSEChannelSession are performed here.
 *
 * @note Due to dependencies on data set up in this method, care should be taken
 * @note in changing the ordering of logic within this code. In particular, if
 * @note the addRequiredDataToOperationCtx(Context, KeyedCollection) call is moved
 * @note above the creation of the server operation, the code will break. Care should
 * @note exercised in moving this method around.
 *
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public ServerOperation executeRequest(ChannelContext channelContext) throws Exception
{
	boolean isProcessorOp = true ;
	int operationId = 0;
	String applicationId;
	HttpServletRequest request = (HttpServletRequest)channelContext.getChannelRequest();

	// There are several data fields that are required to execute the request. These come
	// from the form hidden fields that was posted to make the request. They are saved in a
	// keyed collection held in the channel context, which we obtain next.
	KeyedCollection requestData = channelContext.getRequestData();

	// Get the data fields necessary to execute the operation.
	String operationName = (String) requestData.getValueAt(HtmlConstants.DATAOPERATIONNAME);
	String sessionId = (String) requestData.getValueAt(HtmlConstants.DATASESSIONIDKEY);

	// Security Check
	checkExecutionPermission(sessionId, operationName, channelContext);	

	try
	{
		applicationId = (String) requestData.getValueAt(HtmlConstants.DATAAPPLICATIONIDKEY);
	}
	catch (Exception e) 
	{
		applicationId = HtmlConstants.IGNORE;
	}

	try
	{
		String callingPageId = (String) requestData.getValueAt(HtmlConstants.DATAPAGEID);
		String nen = (String) requestData.getValueAt(HtmlConstants.DATANEXTEVENTNAME);
	}
	catch (Exception e) {isProcessorOp=false;}

	// Place the session id in the request just in case an exception occurs
	// and there is no operation context to carry it along.
	request.setAttribute (HtmlConstants.SESSIONID, sessionId);
	request.setAttribute (HtmlConstants.APPLICATIONID, applicationId);

	//------
	// Now we need to update the operation context with data from the form which was added by the
	// user. First, get the keyed collection with this data.	
	//------
	KeyedCollection kcReqData = (KeyedCollection) requestData.getElementAt(HtmlConstants.DATAKEY);

	// Load the designated operation from the external source.
	DSEServerOperation oper = readOperation(operationName, isProcessorOp, kcReqData, sessionId);
	// Upgrade the PendingTable entry with the op.
	getRVS(channelContext).addOpToPendingRequest(channelContext, oper);
	// Chain server operation to its context
	chainOperation(oper, sessionId, applicationId);

	// Update both operation and session context with the data needed by the toolkit
	addRequiredDataToOperationCtx(oper.getContext(), kcReqData);
	if (!(applicationId.equals(HtmlConstants.IGNORE)))
	{
		addConseqInfoToSessionCtx(sessionId, applicationId, kcReqData);
	}
	else
	{
		addConseqInfoToSessionCtx(sessionId, kcReqData);
	}

	// Will check if the request have locale info
	checkForLocale(channelContext);

	// Update the operation context with the request data posted from the form.
	if (oper.updateContext((KeyedCollection) kcReqData, DSEHtmlProcessor.HTML)==false)
	{
		manageValidationErrors(oper, request, kcReqData);
		// Set the operation to null, it will make the processRequest method to 
		// know that there have occurred validation errors
		oper = null;
	}
	else
	{
		// There are no validation errors, execute the operation/processor
		if (oper instanceof DSEHtmlProcessorOperation)
		{
			request.setAttribute (HtmlConstants.OPERATIONCONTEXT, ((DSEHtmlProcessorOperation)oper).getProcessor().getContext());
		}
		else request.setAttribute (HtmlConstants.OPERATIONCONTEXT, oper.getContext());


		if(!isProcessorOp)
		{
			oper.execute();
		}
		else
		{
			String eventName = (String) kcReqData.getValueAt(HtmlConstants.NEXTEVENTNAME) ;
			((DSEHtmlProcessorOperation)oper).execute(eventName) ;
			
			((DSEHtmlProcessorOperation)oper).replaceCtxIfNeeded();
			request.setAttribute (HtmlConstants.OPERATIONCONTEXT, ((DSEHtmlProcessorOperation)oper).getProcessor().getContext());
		}
	}

	// Update the timestamp of the current session
	updateTimestamp(sessionId, applicationId, oper, channelContext);
	
	return oper;
}
/**
 * When a session is established, we need to call an operation which sets the context
 * to hold data that is saved in the session.
 *
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public ServerOperation executeSessionRequest(ChannelContext channelContext) throws Exception
{
	HttpServletRequest req = (HttpServletRequest) channelContext.getChannelRequest();


	//------
	// Obtain an all alpha session id!
	//------
	String sessionId = getSessionId(channelContext);


	//------
	// Get the name of the operation which will establish a context that is used
	// for this session's data, then read the operation from the external files.
	//------
	String establishSessionOp = (String) Settings.getSettings().getValueAt(HtmlConstants.SETTINGSSTARTUPOPKEY);
	DSEServerOperation operation = (DSEServerOperation) DSEServerOperation.readObject(establishSessionOp);


	//------
	// Save the sessionId and pageId in the operation context.
	//------
	Context opCtx = operation.getContext();
	// add the reply page: Obtain the start page name from the Settings (dse.ini) and
	// set the response page name in the context. 
	addFieldTo(opCtx, HtmlConstants.REPLYPAGE, (String)Settings.getSettings().getValueAt(HtmlConstants.SETTINGSHOMEPAGEKEY));


	// add errorMessages
	addFieldTo(opCtx, HtmlConstants.ERRORMESSAGES, new String[0]);
	
	//------
	// Run the operation which establishes and saves the session context.
	//------
	addFieldTo(opCtx,HtmlConstants.SESSIONID,sessionId);


	//------
	// Prepare the operation context for the application session id
	//------
	addFieldTo(opCtx,HtmlConstants.APPLICATIONID,HtmlConstants.IGNORE);


	//------
	// Save the error page for the initialization step in case of error
	// occurred in the validation time
	//------
	addFieldTo(opCtx, HtmlConstants.ERRORPAGE, (String)Settings.getSettings().getValueAt(HtmlConstants.SETTINGSERRORPAGEKEY));


	//------
	// Save the basePath from the settings
	//------
	addFieldTo(opCtx, HtmlConstants.JSPBASEPATH, (String)Settings.getSettings().getValueAt(HtmlConstants.SETTINGSFILEPATHKEY));


	//-------
	// Create the extrapath field in the context
	//-------
	addFieldTo(opCtx, HtmlConstants.EXTRAPATHINFO,"");
	
	//-------
	// Create the locale field in the context
	//-------
	setLocale(req,opCtx);


	//------
	// Now we need to update the operation context with data from the form which was added by the
	// user. First, get the keyed collection with this data.	
	//------
	KeyedCollection kcReqData = (KeyedCollection) channelContext.getRequestData().getElementAt(HtmlConstants.DATAKEY);
	if(operation.updateContext(kcReqData,DSEHtmlProcessor.HTML) != false){


	//------
	// Save a reference to the operation context in the request to make it available to the JSP.
	//------	
	( (HttpServletRequest)channelContext.getChannelRequest() ).setAttribute (HtmlConstants.OPERATIONCONTEXT, operation.getContext());
	operation.runInSession(sessionId);
	addRequiredDataToSessionCtx(sessionId, req);
	
	} else {


		// Some problem occured with the validation of the received data
		operation = null;
		req.setAttribute(HtmlConstants.OPERATIONCONTEXT,opCtx);
	
	}
	
	
	//------
	// Update the timestamp of the current session and check the session table for expired sessions
	//------
	if (usingCookies(channelContext)) {
		Context.setTimeStampForSession(sessionId, null);
		} else {
			Context.setTimeStampForSession(sessionId, new Long(System.currentTimeMillis()));
		}	

	// New for making RVS one instance per session instead of a static common one
	SessionEntry se = Context.getSession(sessionId);
	DataField rvs = new DataField();
	rvs.setName(HtmlConstants.RVS);
	rvs.setValue(new RequestValidationServices());
	se.addElement(rvs);	

	return operation;
}
/**
 * Returns the RequestValidationServices instance associated with the current session.
 * If the session does not exist an exception is thrown, if the session does not have a requestValidationServices 
 * instance associated, a new one is created and returned, because it might be cause of a distributed environment.
 * In distributed environments, some of the features of abnormal nabvigation will not work.
 * @return RequestValidationServices RequestValidationServices object associated with given session
 * @param cc com.ibm.dse.clientserver.ChannelContext ChannelContext used to get the session
 * @throws DSEException Either it is not possible to retrieve a sessionId from the channelContext or there is no session associated with the sessionId.
 */
protected RequestValidationServices getRVS(ChannelContext cc) throws DSEException
{
	String sessionId = getSessionId(cc);
	if (sessionId!=null)
	{
		// check if a RVS already exists for this session, if not, it could mean we are in a distributed environment, trace and create a new one
		SessionEntry se = Context.getSession(sessionId);

		if (se.tryGetValueAt(HtmlConstants.RVS)==null)
		{
			DataField df = new DataField();
			df.setName(HtmlConstants.RVS);
			df.setValue(new RequestValidationServices());
			se.addElement(df);	

			if (Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Debug))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Debug, Settings.getTID(), "Current session does not have a RequestValidationServices instance associated. If the application runs under a distributed environment this is the normal behavior. sessionId: " + sessionId);
			}
		}
		
		return (RequestValidationServices) se.getValueAt(HtmlConstants.RVS);		
	}
	else
	{
		throw new DSEInvalidRequestException(DSEException.critical, HtmlConstants.SESSIONID, "ChannelContext does not contain ChannelSession or it does not contain any sessionId.");
	}
}
/**
 * Obtain the session id from the channel context.
 * Returns the sessionId or null if there is no sessionId associated with this ChannelContext.
 * @return java.lang.String
 * @param aChannelContext com.ibm.dse.clientserver.ChannelContext
 */
protected String getSessionId(ChannelContext aChannelContext)
{
	String sessionId = null;
	if (aChannelContext.getChannelSession() instanceof HttpSession)
	{
		sessionId = ((HttpSession) aChannelContext.getChannelSession()).getId();
	}
	else if (aChannelContext.getChannelSession() instanceof ChannelSession)
	{
		sessionId = ((ChannelSession) aChannelContext.getChannelSession()).getId();
	}
	return sessionId;
}
/**
 * Perform the necessary steps to manage validation errors. These steps are the following:
 * <i>Update the operation context's errorFlag to true
 * <i>Set the operation or processor context to the request in order to give access to it from the PresentationHandler
 * <br>
 * If the request tries to execute a processor instead of an operation, following two steps are performed:
 * <i>Update operation context with processor specific information
 * <i>Replace the operation request context per the transition context if the erros happened whithin a transition context
 * @param oper com.ibm.dse.base.DSEServerOperation The operation with errors
 * @param request javax.servlet.HttpServletRequest request object
 * @param kcReqData com.ibm.dse.base.KeyedCollection kColl received from client
 * @throws com.ibm.dse.base.DSEInvalidArgumentException
 * @throws com.ibm.dse.base.DSEObjectNotFoundException
 */
public void manageValidationErrors(DSEServerOperation oper, HttpServletRequest request, KeyedCollection kcReqData) throws DSEInvalidArgumentException, DSEObjectNotFoundException
{ 
	//------
	// Update the flag indicating that there have been val errors
	//------
	oper.setValueAt(HtmlConstants.ERRORFLAG, new Boolean(true));
	//------
	// Save this context as if the operation ran so it can feed the
	// original replyPage which had a type mismatch on it.
	//------
	//request.setAttribute (HtmlConstants.OPERATIONCONTEXT, oper.getContext());
	if (oper instanceof DSEHtmlProcessorOperation)
	{
		request.setAttribute(HtmlConstants.OPERATIONCONTEXT, ((DSEHtmlProcessorOperation) oper).getProcessor().getContext());
	}
	else
		request.setAttribute(HtmlConstants.OPERATIONCONTEXT, oper.getContext());
	if (oper instanceof DSEHtmlProcessorOperation)
	{
		//------
		// If the request is running within the automaton, the JSP needs information about the processor.
		//------
		oper.setValueAt(HtmlConstants.PROCESSORSTATE, ((DSEHtmlProcessorOperation) oper).getProcessor().getCurrentState().getName());
		oper.setValueAt(HtmlConstants.PROCESSORID, ((DSEHtmlProcessorOperation) oper).getProcessor().getProcessorId());

		//------
		// Get the event name to get the transition
		//------
		String eventName = (String) kcReqData.getValueAt(HtmlConstants.NEXTEVENTNAME);
		if (eventName.indexOf(".") == -1)
			eventName = ((DSEHtmlProcessorOperation) oper).getProcessor().getCurrentState().getName() + "." + eventName;
		//------
		// Get the transition because the val errors could come from a transition context validation.
		//------
		com.ibm.dse.automaton.TransitionDescriptor transition = (com.ibm.dse.automaton.TransitionDescriptor) ((DSEHtmlProcessorOperation) oper).getProcessor().getCurrentState().getTransitions().get(eventName);
		if (transition instanceof HtmlTransitionDescriptor)
		{
			Context trCtx = ((HtmlTransitionDescriptor) transition).getContext();
			if (trCtx != null)
			{
				//------
				// If errors found come from a transition context, the presHandler need access to this ctx to get the error info.
				//------
				request.setAttribute(HtmlConstants.OPERATIONCONTEXT, trCtx);
				oper.setContext(trCtx);
			}
		}
	}
}
/**
 * Adds current request to the pendingRequestTable
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 * @throws DSEException Is not possible to retrieve a valid RequestValidationServices for this session.
 */
public void preProcessRequest(ChannelContext channelContext) throws DSEException
{
	getRVS(channelContext).addPendingRequest(new HtmlRequestData (channelContext, null));
}
/**
 * Manages an attempt of abnormal navigation detected. These cases are all
 * related with abnormal navigation support: pending requests, aging requests and unconsistent
 * processor exceptions.
 * The handling of these situations involve in all the cases following steps:
 * <i>Update operation context with error information about what has happened
 * <i>Do not execute the request but answer the client as if current request was the first one 
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 * @param type java.lang.String The type of error
 * @throws DSEException
 */
protected void processAbnormalNavigation(ChannelContext channelContext, String type) throws DSEException
{
	PresentationHandler presHandler = getPresentationHandler(channelContext);
	String message;
	DSEServerOperation theOperation = getRVS(channelContext).getOperationInProcess(channelContext);

	if (type.equals(HtmlConstants.PENDINGEXCEPTIONCODE) && getRVS(channelContext).isUnconsistentProcessor(channelContext))
	{
		// If the request comes with a pending code, means that the original request was running when the current one arrived, so it can not be aging, it can only be either pending or unconsistent.
		type = HtmlConstants.INCONSISTENTPROCESSOREXCEPTIONCODE;
	}

	message = createErrorMessage(type);

	addErrorInfo(theOperation, message);

	((HttpServletRequest) channelContext.getChannelRequest()).setAttribute(HtmlConstants.OPERATIONCONTEXT, theOperation.getContext());

	// Check if there has been val errors because presHandler must know whether the operation had val errors or not.
	if (((Boolean) theOperation.getValueAt(HtmlConstants.ERRORFLAG)).booleanValue())
	{
		// Simulate the normal behaviour to make the presentation handler return the val error page
		theOperation = null;
	}
	presHandler.processReply(channelContext, theOperation);
	endRequest(channelContext);
}
/**
 * Process the request when there is no abnormal navigation.
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public void processNavigation(ChannelContext channelContext) throws Exception
{
	PresentationHandler presHandler = (PresentationHandler) getPresentationHandler(channelContext);
	DSEServerOperation theOperation;
	String sessionId, applicationId;
	boolean isProcessorOp = false;
	try
	{
		preProcessRequest(channelContext);
		theOperation = (DSEServerOperation) executeRequest(channelContext);
		if (theOperation == null)
		{
			// There have been validation errors
			presHandler.processReply(channelContext, null);
			getRVS(channelContext).beginAgingRequest(channelContext);
		}
		else
		{
			if (theOperation instanceof DSEHtmlProcessorOperation)
				isProcessorOp = true;
	
			sessionId = (String) theOperation.getValueAt(HtmlConstants.SESSIONID);
			applicationId = (String) theOperation.getValueAt(HtmlConstants.APPLICATIONID);
			setNextPageId(channelContext, theOperation, sessionId, applicationId, presHandler);
	
			if (isProcessorOp)
				saveProcessor((DSEHtmlProcessorOperation) theOperation, sessionId, applicationId);
	
			presHandler.processReply(channelContext, theOperation);
			endRequest(channelContext);
	
			if (isProcessorOp)
				putProcessorIntoCache((DSEHtmlProcessorOperation) theOperation);
		}
	}
	catch (DSEException e)
	{
		endRequest(channelContext);
		throw e;
	}
}
/**
 * Process the request originated from a request that already has a session
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public void processRequest(ChannelContext channelContext)
{
	try
	{
		// get the session
		String sessionId = getSessionId(channelContext);
		SessionEntry se = Context.getSession(sessionId);
		RequestValidationServices rvs = getRVS(channelContext);
		synchronized(rvs)
		{
			if (rvs.isMarkedAsNotAnswered(channelContext))
			{
				processAbnormalNavigation(channelContext, HtmlConstants.PENDINGEXCEPTIONCODE);
			}
			else if (rvs.isRequestAging(channelContext))
			{
				processAbnormalNavigation(channelContext, HtmlConstants.AGINGEXCEPTIONCODE);
			}
			else if (rvs.isUnconsistentProcessor(channelContext))
			{
				processAbnormalNavigation(channelContext, HtmlConstants.INCONSISTENTPROCESSOREXCEPTIONCODE);
			}
			else
			{
				processNavigation(channelContext);
			}
		}
	}
	catch(Exception e)
	{
		if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while processing the request. Channel Context = " + channelContext);
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
		}
		try
		{
			getPresentationHandler(channelContext).processException(channelContext, e);
			endRequest(channelContext);
		}
		catch (DSENoRegisteredPresentationHandlerException pres)
		{
			if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while trying to send an exception. No registered presentation handler. Exception: " + pres);
		}
	}
}
/**
 * This method is similar to processRequest, only back and double click validation
 * are not performed as a new session is being established.
 *
 * A dummy session has been created during the preProcessRequest method. This session
 * must be overriden by the startup operation (executed inside the executeSessionRequest)
 * so the application is not aware of the existence of this session until the operation
 * is executed. If an error occurs before the operation is executed the toolkit must 
 * handle the dummy session created. The dummy session has an identifier: dse_DummySession
 * to be checked by the infrastructure in order to handle the dummy session when the
 * application has not set its own session object.
 *
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public void processSessionRequest(ChannelContext channelContext)
{
	PresentationHandler presHandler=null;
	ServerOperation theOperation=null;
	try
	{
		presHandler = (PresentationHandler) getPresentationHandler(channelContext);
		try
		{
			theOperation = executeSessionRequest(channelContext);


			// A session entry has been created for this session. Save there the channel session object.
			Context.setSessionObjectForSession( getSessionId(channelContext), channelContext.getChannelSession() );


			if (theOperation != null){
				
				// set the next page id for the client
				String nextPageId = getRVS(channelContext).getNextPageId((String) theOperation.getValueAt(HtmlConstants.SESSIONID)) ;
				// get the session ctx. It must have been created by the operation
				Context sessionCtx = Context.getCurrentContextForSession( getSessionId(channelContext) );
				
				if(sessionCtx == null){
					//The session ctx has not been created. The pageId will be setted in the operation ctx.
					sessionCtx = theOperation.getContext();
				}
				// add the consequentiality information to the session ctx
				addFieldTo(sessionCtx, HtmlConstants.PAGEID, nextPageId) ;
			}
			
			presHandler.processReply(channelContext, theOperation);
			
		}
		catch (Exception e)
		{
			try {
				// Remove the dummy session
				String sessionType = Context.getTypeForSession(getSessionId(channelContext));
				if (sessionType != null && (sessionType.toString()).equals(HtmlConstants.DUMMYSESSIONTYPE))
					Context.removeSession(getSessionId(channelContext));
			}
			catch (DSEException sessionExc)	{
				// Session not existing. Go on with the exception handling
			}
			
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while processing the Session request. Channel Context = "+channelContext) ;
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}				
			presHandler.processException(channelContext, e);
		}
	}
	catch (Exception e)
	{
		try {
				// Remove the dummy session
				String sessionType = Context.getTypeForSession(getSessionId(channelContext));
				if (sessionType != null && (sessionType.toString()).equals(HtmlConstants.DUMMYSESSIONTYPE))
					Context.removeSession(getSessionId(channelContext));
			}
			catch (DSEException sessionExc)	{
				// Session not existing. Go on with the exception handling
		}
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while processing the Session request. Channel Context = "+channelContext) ;
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
		}						
		handleException(channelContext, e);
	}


	if (theOperation != null) endSessionRequest(theOperation);
	
	return;
}
/**
 * Gets the processor instance from the processor operation and puts it into cache in order 
 * to be reused by further requests.
 * @param oper DSEHtmlProcessorOperation
 */
protected void putProcessorIntoCache(DSEHtmlProcessorOperation oper)
{
	if ((oper instanceof DSEHtmlProcessorOperation) && (!(((DSEHtmlProcessorOperation) oper).getProcessor().getCurrentState().getType().equals(com.ibm.dse.automaton.DSEState.FINAL_TYPE))))
	{
		/** <HSBC> **/
		//Cleaning the processor
		if (oper.getProcessor() instanceof DSEHtmlProcessor) {
			DSEHtmlProcessor proc = (DSEHtmlProcessor)oper.getProcessor();
			if (proc.getContext() != proc.getProcessorContext()) {
				// If state context is available, unchain it and remove it from state.
				try{
					proc.getContext().unchain();
				}catch(Exception e)
				{
					if(Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
					{
						Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "The processor was not successfully cleaned: " + e.toString());
						Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), e);
					}
				}
				
				((DSEHtmlState) proc.getCurrentState()).setContext(null);
			}
		}
		/** </HSBC> **/
		
		// Put the processor into the cache to be reused by other requests
		try
		{
			com.ibm.dse.automaton.ProcessorExternalizer.putIntoCache((DSEHtmlProcessor) ((DSEHtmlProcessorOperation) oper).getProcessor());
		}
		catch (java.io.IOException ioe)
		{
			if (Trace.doTrace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), "It has not been possible to store the processor in cache: " + ioe.toString());
				Trace.trace(DSEHtmlProcessor.COMPID, Trace.High, Trace.Error, Settings.getTID(), ioe);
			}
		}
		// Clear up the operation's instance
		 ((DSEHtmlProcessorOperation) oper).setProcessor(null);
	}
}
/**
 * Returns an operation containing either the operation going to be executed
 * or a processor operation encapsulating the processor to execute. If this 
 * is the case, the processor is initialized before return the operation.
 * @param isProcessorOp boolean indicating if the request intends to execute a processor or an operation
 * @param kcReqData KeyedCollection created with the info sent from client
 * @param operationName String with the id of the operation or processor that will be passed to the right externalizer
 * @param sessionId String
 * @exception DSECSSessionNotEstablishedException The session is expired
 * @exception DSEInvalidRequestException Operation or processor can not be found, instantiated or initialized
 */
protected DSEServerOperation readOperation(String operationName, boolean isProcessorOp, KeyedCollection kcReqData, String sessionId) throws DSECSSessionNotEstablishedException, DSEInvalidRequestException
{
	DSEServerOperation oper = null;
	try
	{
		if(!Context.getSession(sessionId).getExpired())
		{
			if (!isProcessorOp)
			{
				oper = (DSEServerOperation) DSEOperation.readObject(operationName);
			}
			else 
			{
				oper = (DSEServerOperation) DSEOperation.readObject(HtmlConstants.PROCESSOROPERATIONNAME);
				( (DSEHtmlProcessorOperation) oper).initializeProcessor(kcReqData);
			}
			return oper;
		}
		else
		{
			if (Trace.doTrace(CSConstants.COMPID, Trace.High, Trace.Error))
			{
				String tid = Settings.getTID();
				Trace.trace(CSConstants.COMPID,Trace.High,Trace.Error, tid, "The session "+sessionId+" has expired");
			}
			// Trace and throw an exception
			throw new DSECSSessionNotEstablishedException();
		}
			
	}
	catch (DSEObjectNotFoundException dseobjnf)
	{
		//The session has been removed from the session table.	
			if (Trace.doTrace(CSConstants.COMPID, Trace.High, Trace.Error))
			{
				String tid = Settings.getTID();
				Trace.trace(CSConstants.COMPID,Trace.High,Trace.Error, tid, "The session "+sessionId+" has expired");
			}
			// Trace and throw an exception
			throw new DSECSSessionNotEstablishedException();
	}
		
	catch (Exception e)
	{
		throw new DSEInvalidRequestException(DSEException.critical, HtmlConstants.SESSIONID, "Cannot create operation " + operationName + ". It may not correspond to any defined operation or processor. "+e.getMessage());
	}

}
/**
 * Saves the processor information needed to rebuild it in further requests for this processor.
 * @param oper DSEHtmlProcessorOperation encapsulating the processor
 * @param sessionId String containing the application session id
 * @param applId String containing the application SUB session id (if any)
 * @exception DSEException The processor can not be saved
 */
protected void saveProcessor(DSEHtmlProcessorOperation oper, String sessionId, String applId) throws DSEException
{
	if ((oper instanceof DSEHtmlProcessorOperation) && (!(((DSEHtmlProcessorOperation) oper).getProcessor().getCurrentState().getType().equals(com.ibm.dse.automaton.DSEState.FINAL_TYPE))))
	{
		if (applId.equals(HtmlConstants.IGNORE))
		{
			HtmlProcessorManager.getUniqueHTMLInstance().saveProcessor(sessionId, (DSEHtmlProcessor) ((DSEHtmlProcessorOperation) oper).getProcessor());
		}
		else
		{
			HtmlProcessorManager.getUniqueHTMLInstance().saveProcessor(applId, (DSEHtmlProcessor) ((DSEHtmlProcessorOperation) oper).getProcessor());
		}
	}
}
/**
 * Creates a new locale object from the headers information and stores it in the context.
 * @param req Request object where to get the headers from
 * @param ctx context where to store the Locale object
 * @exception Exception
 */
public void setLocale(HttpServletRequest req,Context ctx) throws Exception{
	
	String lang = null;
	Locale locale = null;
	
	lang = req.getHeader("accept-language");
	if(lang != null && lang.indexOf("-")!=-1)
	{
		
		String language = lang.substring(0,lang.indexOf("-"));
		String country = lang.substring(lang.indexOf("-")+1,lang.indexOf("-")+3);
		
		locale = new Locale(language,country);
	} else {
		locale = Locale.getDefault();
	}
	
	addFieldTo(ctx,HtmlConstants.LOCALE,locale);
		 
}
/**
 * Updates the application session context with the next page id.
 * @param channelContext Used to get the related RequestValidationServices
 * @param theOperation
 * @param sessionId
 * @param applicationId
 * @param presHandler
 * @exception DSEException
 */
protected void setNextPageId(ChannelContext channelContext, DSEServerOperation theOperation, String sessionId, String applicationId, PresentationHandler presHandler) throws DSEException
{
	try
	{
		String nextPageId = getRVS(channelContext).getNextPageId(sessionId);
		if (!(applicationId.equals(HtmlConstants.IGNORE)))
		{
			(Context.getCurrentContextForSession(sessionId, applicationId)).setValueAt(HtmlConstants.PAGEID, nextPageId);
		}
		else
		{
			(Context.getCurrentContextForSession(sessionId)).setValueAt(HtmlConstants.PAGEID, nextPageId);
		}
	}
	catch (Exception e)
	{
		try
		{
			if (!((Boolean) theOperation.getValueAt("dse_sessExpFlag")).booleanValue())
			{
				//Do nothing
			}
		}
		catch (DSEException dsexc)
		{
			if (Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when setting the next page id");
			}
			throw dsexc;
		}
	}
}
/**
 * Updates the timestamp for this session and for the application session if it exists.
 * @param sessionId java.lang.String
 * @param operation com.ibm.dse.base.Operation
 */
protected void updateTimestamp(String sessionId, String appSessionId, Operation operation, ChannelContext channelContext) throws DSEInvalidArgumentException, DSEInvalidRequestException
{
	try
	{
		Context.setTimeStampForSession(sessionId, new Long(System.currentTimeMillis()));

		// Try to get the appSessionId from the operation because it might have changed since the request came in
		if (operation!=null)
			appSessionId = (String) operation.getValueAt(HtmlConstants.APPLICATIONID);
			
		if (!appSessionId.equals(HtmlConstants.IGNORE))
		{
			SessionEntry se = (SessionEntry) Context.getSession(sessionId).getElementAt(appSessionId);
			se.setTimeStamp(new Long(System.currentTimeMillis()));
		}
	}
	catch (DSEObjectNotFoundException dsee) 
	{
		//------
		//Ignore the exception. This is not the end of the world. The session may have been removed by the operation
		//------
		if (Trace.doTrace(CSConstants.COMPID, Trace.Medium, Trace.Debug)) {
			String aMessage, aTID;
			aMessage = "No session when updating timestamp...moving on ";
			aTID = Settings.getTID();
			Trace.trace(CSConstants.COMPID,Trace.Medium,Trace.Debug, aTID,getSessionId(channelContext) +": "+ aMessage);
		}
		//-----
		// Putting the sessionExpired flag in the operation context in case the operation is not null
		//-----
		if(operation != null){
			DataField sessionExpiredFlag = new DataField();
			sessionExpiredFlag.setName("dse_sessExpFlag");
			sessionExpiredFlag.setValue(new Boolean(true));
			operation.addElement(sessionExpiredFlag);
		}
	}
}
}
