package com.ibm.dse.cs.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import com.ibm.dse.base.*;

/**
 * Interface that defines the protocol to access the context from a JSP
 *
 * @copyright(c) Copyright IBM Corporation 2000.
 */
public interface JspContextServices
{
/** 
 * Add a data element in the the transition context for the specified transition
 * @param String - The transition name
 * @param String - The name of the data element to be added
 */ 
public void addDataElementToTransitionContext ( String transitionName, String dataName) throws DSEException;
/** 
 * Create a transiton context for the specified transition 
 * with the set of dataNames
 * @param String - The transition name
 * @param Vector - A vector with the data element names to be added
 */ 
public void createTransitionContext(String transitionName, com.ibm.dse.base.Vector dataNames) throws DSEException;
/**
 * Obtain the relative path (relative to the web app server web path) for
 * jsp resources.
 * Creation date: (6/21/00 5:38:55 PM)
 * @return java.lang.String
 */
public String getBaseWebPath ( );
/**
 * Get the operation context for the page which initialized this bean.
 * Creation date: (6/20/00 12:50:57 PM)
 * @return com.ibm.dse.base.Context
 */
public Context getContext ( );
/**
 * Return the current list of cross validation errors.
 * Creation date: (6/23/00 11:31:45 AM)
 * @return java.lang.String[]
 */
public String[] getCrossValidationErrorList ( );
/**
 * Returns the named data element from the context.  If the
 * element is not present in the context, null is returned.
 * Creation date: (6/22/00 2:46:27 PM)
 * @return java.lang.Object
 * @param name java.lang.String
 */
public DataElement getElement ( String name );
/**
 * Returns the error info object associated to the named field in the context.
 * @return ErrorInfo
 */
public ErrorInfo getErrorInfo (String name);
/**
 * Return the error page name
 * @return String - The <I>error page</I> html.
 */
public String getErrorPageHtml ( String name );
/**
 * If the toolkit placed an exception in the HttpServletRequest with key name
 * <b>dseexception</b>, then it can be saved here when the bean is initialized.
 * Creation date: (6/20/00 2:52:32 PM)
 * @return com.ibm.dse.base.DSEException
 */
public DSEException getException ( );
/**
 * Return message text cooresponding to the current exception.
 * Creation date: (6/20/00 12:56:19 PM)
 * @param param javax.servlet.jsp.JspWriter
 */
public String getExceptionMessage ( );
/**
 * Return the current error marker only if the given field is in error.
 * Creation date: (6/22/00 4:52:56 PM)
 * @return java.lang.String
 * @param name java.lang.String
 */
public String getFieldErrorMarker ( String name );
/**
 * Returns the label associated with the given field in
 * the context.
 * Creation date: (6/22/00 1:22:11 PM)
 * @return java.lang.String
 * @param name java.lang.String
 */
public String getFieldLabel ( String name );
/**
 * Return the field error message text at the specified index.
 * This method returns a blank string if the field has no errors.
 * Creation date: (6/23/00 12:13:19 PM)
 * @return java.lang.String
 * @param name java.lang.String
 * @param idx int
 */
public String getFieldValidationError ( String name, int idx );
/**
 * Return the field error message text at the specified index
 * decorated with prefix and suffix HTML strings.  This method
 * returns a blank string if the field has no errors.
 * Creation date: (6/23/00 12:13:19 PM)
 * @return java.lang.String
 * @param name java.lang.String
 * @param idx int
 */
public String getFieldValidationErrorDecorated ( String name, int idx );
/**
 * Retrieve the entire error list for the field.  An array of zero
 * size is returned if there are no errors associated with the field.
 * Creation date: (6/23/00 12:30:22 PM)
 * @return java.lang.String[]
 * @param name java.lang.String
 */
public String[] getFieldValidationErrorList ( String name );
/**
 * Returns the value of the wrong data typed by the user.
 * This data is considered to be wrong according to the validation process.
 * The typed data is stored in the ErrorInfo object associated to the named field
 * @return String
 */
public String getInputDataInError (String name);
//_AutoVersion end.
/**
 * Returns the list of Jsp Format Errors that were found
 * during Jsp Processing.
 * Creation date: (7/5/00 3:12:40 PM)
 * @return java.lang.String[]
 */
public String[] getJspFormatErrorList ( );
/**
 * Emit the Html to the JspWriter stream to create a link with given text to
 * a given JSP saved in a given web path (relative to the base JSP path for the
 * application server). The link will include the session id using URL rewriting.
 * Creation date: (6/20/00 3:39:50 PM)
 * @param param javax.servlet.jsp.JspWriter
 * @param webPath java.lang.String
 * @param pageName java.lang.String
 * @param linkText java.lang.String
 */
public String getLinkToPage ( String webPath, String pageName, String linkText );
/**
 * Obtain any extra path information passed on the call to establish
 * the session.  For example, if the URL was invoked as:
 * <br>http://.../CSEstablishSessionServlet/en
 * <br>then the <b>locale id</b> is the string <b>en</b>.
 * Creation date: (6/21/00 5:38:55 PM)
 * @return java.lang.String
 */
public String getLocaleId ( );
/**
 * Returns an string containing the HTML for all of the HTML Client
 * required fields.
 * @return java.lang.String
 */
public String getRequiredHtmlFields();
/**
 * Returns an string containing the HTML for all of the HTML Client
 * required fields using the passed parameters to fill in the user
 * specifiable information.
 * @return java.lang.String
 * @param parentctx java.lang.String
 * @param opname java.lang.String
 */
public String getRequiredHtmlFields ( String parentctx, String opname);
/**
 * Returns a string containing the HTML for all of the
 * HTML Client required fields using the passed parameters 
 * to fill in the user specifiable information.
 * If no opName is specified, assume that this page is being
 * displayed within a processor flow, so the processor name is
 * got from the table of stored processors.
 * The returned String is for use as part of an URL.
 * The format is the needed to be sent via the GET method:
 * <I>param1=value1&param2=value2&...&paramN=valueN</I>
 * @return String - The <I>required fields</I> html in the format needed to send using the GET method.
 */
public String getRequiredHtmlFieldsForGETMethod();
/**
 * Returns an string containing the HTML for all of the HTML Client
 * required fields using the passed parameters to fill in the user
 * specifiable information.
 * The returned String is for use as part of an URL.
 * The format is the needed to be sent via the GET method:
 * <I>param1=value1&param2=value2&...&paramN=valueN</I>
 * @return java.lang.String
 * @param parentctx java.lang.String
 * @param opname java.lang.String
 */
public String getRequiredHtmlFieldsForGETMethod ( String parentctx, String opname);
/**
 * Returns the current session identifier string "as-is"
 * Creation date: (6/20/00 3:27:10 PM)
 * @return java.lang.String
 */
public String getSessionId ( );
/**
 * Returns the current session identifier string as an HTML hidden field.
 * Creation date: (6/22/00 2:27:48 PM)
 * @return java.lang.String
 */
public String getSessionIdHtmlField ( );
/**
 * Returns the current session identifier string as a URL parameter.
 * Creation date: (6/22/00 1:41:24 PM)
 * @return java.lang.String
 */
public String getSessionIdParameter ( );
/**
 * Returns the specified value from the operation context as a string.
 * Handles exceptions by returning a null string value.
 * Creation date: (6/22/00 1:22:11 PM)
 * @return java.lang.String
 * @param name java.lang.String
 */
public String getStringValue ( String name );
/**
 * Returns all validation errors in a single list array.
 * Creation date: (6/23/00 11:31:14 AM)
 * @return java.lang.String[]
 */
public String[] getValidationErrorList ( );
/**
 * Returns the named value from the context in its natural format.
 * If the element is not present in the context, null is returned.
 * Creation date: (6/22/00 2:46:27 PM)
 * @return java.lang.Object
 * @param name java.lang.String
 */
public Object getValue ( String name );
/**
 * Returns the "hidden" status of the named field
 * in the context.
 * Creation date: (6/22/00 4:57:08 PM)
 * @return boolean
 * @param name java.lang.String
 */
public boolean isFieldHidden ( String name );
/**
 * Returns the error status of the named field in the context.
 * Creation date: (6/22/00 4:57:08 PM)
 * @return boolean
 * @param name java.lang.String
 */
public boolean isFieldInError ( String name );
/**
 * Returns the "input mandatory" status of the named field
 * in the context.
 * Creation date: (6/22/00 4:57:08 PM)
 * @return boolean
 * @param name java.lang.String
 */
public boolean isFieldMandatory ( String name );
/**
 * Returns the "read only" status of the named field
 * in the context.
 * Creation date: (6/22/00 4:57:08 PM)
 * @return boolean
 * @param name java.lang.String
 */
public boolean isFieldReadOnly ( String name );
/**
 * Returns whether there has been any error during the validation process
 * @return boolean
 */
public boolean isThereAnyError();
/** 
 * Removes a data element from the the transition context for the specified transition
 * @param String - The transition name
 * @param String - The name of the data element to be removed
 */ 
public void removeDataElementFromTransitionContext ( String transitionName, String dataName) throws DSEException;
/**
 * Sets up the error decorator prefix.  This prefix is prepended
 * to any error information returned in a "decorated" error string.
 * Creation date: (6/23/00 2:17:22 PM)
 * @param pfx java.lang.String
 */
public void setErrorDecorationPrefix ( String pfx );
/**
 * Sets up the error decorator suffix.  This suffix is appended
 * to any error information returned in a "decorated" error string.
 * Creation date: (6/23/00 2:17:22 PM)
 * @param pfx java.lang.String
 */
public void setErrorDecorationSuffix ( String sfx );
/**
 * Sets up the error marker value.  This is the string that is 
 * returned by the "getFieldErrorMarker" method if the field is
 * found to be in error.
 * Creation date: (6/22/00 5:03:20 PM)
 * @param newmarker java.lang.String
 */
public void setErrorMarker ( String newmarker );
/** 
 * Sets a parameter given by paramName with a value given by paramValue for a transitionName of this state
 * @param String - The transitionName
 * @param String - The parameter name 
 * @param String - The parameter value 
 */ 
public void setTransitionParameter(String transitionName, String paramName, String paramValue) throws DSEObjectNotFoundException;
} // end interface JspContextServices
