package com.ibm.dse.cs.html;

/*_
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 5648-D89
 * (C) Copyright IBM Corp. 2000, 2003
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp
 */
import java.util.*;

import javax.servlet.http.HttpServletRequest;

import com.ibm.dse.clientserver.*;
import com.ibm.dse.cs.html.HtmlConstants;
import com.ibm.dse.cs.html.HtmlRequestData;
import com.ibm.dse.cs.servlet.*;
import com.ibm.dse.base.*;
/**
 * Back and double post validation requires that we keep a table of operations
 * which are in progress. The table key is a composite key of sessionid.pageid.
 *
 * The data held for each record is a RequestData object, which encapsulates
 * the channelContext and the operation which is running.
 *
 * Double (multiple) post validation is performed by testing that no entry
 * is present for a sessionid.pageid before processing the request.
 *
 * Back button validation is related to processes. This support has not been
 * added yet.
 *
 * @copyright(c) Copyright IBM Corporation 2000, 2003.
 */
public class RequestValidationServices
{
	private static final java.lang.String COPYRIGHT =
		"Licensed Materials - Property of IBM "+//$NON-NLS-1$
		"Restricted Materials of IBM "+//$NON-NLS-1$
		"5648-D89 "+//$NON-NLS-1$
		"(C) Copyright IBM Corp. 2000, 2003 All Rights Reserved. "+//$NON-NLS-1$
		"US Government Users Restricted Rights - Use, duplication or disclosure "+//$NON-NLS-1$
		"restricted by GSA ADP Schedule Contract with IBM Corp ";//$NON-NLS-1$

	protected java.util.Hashtable pendingRequestTable = new java.util.Hashtable();
	protected long ageLimit = 0;
	//------
	// static initializer for the ageLimit field.
	//------
	static
	{
		
	}
	/**
	 * Constructor for RequestValidationServices.
	 */
	public RequestValidationServices()
	{
		super();

		try
		{
			String ageLimitString = (String) Settings.getSettings().getValueAt(HtmlConstants.MINREQUESTRESUBMITTIME);
			ageLimit = Long.parseLong(ageLimitString) * 1000;
		}
		catch (Exception ignored)
		{
			// Just use the default ageLimit of 0.
		}
	}

/**
 * This method adds toolkit related fields to a context.
 *
 * @param ctx com.ibm.dse.base.Context
 * @param name java.lang.String
 * @param value java.lang.Object 
 */
public static void addFieldTo(Context ctx, String name, Object value) throws Exception
{
	DataField df = new DataField();
	df.setName(name);
	df.setValue(value);
	ctx.addElement(df);
}
/**
 * Once a record is added in the pending requests table, this method adds to this
 * record the operation related with the request.
 *
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 * @param oper com.ibm.dse.cs.html.HtmlServerOperation
 */
public void addOpToPendingRequest(ChannelContext channelContext, DSEServerOperation addOperation) throws DSEException
{
	// Construct an identifier from the channel context.
	String compositeKey = getCompositeKey(channelContext);

	// Upgrade the table record.
	( (HtmlRequestData) pendingRequestTable.get(compositeKey) ).operation = addOperation;
}
/**
 * Add request into the pending requests table
 * @param requestData com.ibm.dse.cs.html.HtmlRequestData
 */
public void addPendingRequest(HtmlRequestData requestData) throws DSEException{
	
	
	String compositeKey = getCompositeKey(requestData.channelContext);
	pendingRequestTable.put(compositeKey, requestData);
	
}
/**
 * If the system has a non-zero age limit, then set a timestamp (actually
 * a date) for the pending request matching the session and page that is passed in.
 * Then if another request from the same session.page comes in, and it is before
 * the aging process ends, we declare a double post and we will not process
 * the request any further.
 *
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public void beginAgingRequest(ChannelContext channelContext) throws DSEException
{//channelContext.getRequestData().getValueAt("dse_allErrors");
	String sessionId = (String) channelContext.getRequestData().getValueAt(CSConstants.DATASESSIONIDKEY);
	String pageId = (String) channelContext.getRequestData().getValueAt(HtmlConstants.DATAPAGEID);
	
	String compositeKey = getCompositeKey(channelContext);
	HtmlRequestData pendingRequestData = (HtmlRequestData)pendingRequestTable.get(compositeKey);
	
	if (ageLimit == 0)
	{
		// This request should just be removed from the table of pending
		// requests and the operation closed, because it is not supposed to age.
		try
		{
			if(pendingRequestData.operation instanceof com.ibm.dse.automaton.html.DSEHtmlProcessorOperation)
			{
				// Get the delegateValAssessment flag in the proces context
				Object delValField=((com.ibm.dse.automaton.html.DSEHtmlProcessorOperation)pendingRequestData.operation).getProcessor().getContext().getKeyedCollection().getElements().get(HtmlConstants.DELVALASSESSMENT);
				if ((delValField !=null) && (((String)((DataField)delValField).getValue()).equals("true")))
				{
					// if true  destroy the TC
					Context tcCtxt = ((com.ibm.dse.automaton.html.DSEHtmlProcessorOperation)pendingRequestData.operation).getProcessor().getContext();
					((com.ibm.dse.automaton.html.DSEHtmlProcessorOperation)pendingRequestData.operation).getProcessor().setContext(tcCtxt.getParent());
					tcCtxt.unchain();
				}
				((com.ibm.dse.automaton.html.DSEHtmlProcessorOperation)pendingRequestData.operation).closeWithoutCleanProcessor();
			}
			else{
				pendingRequestData.operation.close();
			}
		}
		catch (Exception e){
			if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
			{
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when closing the operation belonging to the pending request") ;
				Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
			}					
		}
		/*** New for syncPresentationHandler ***/
		if (!isMarkedAsNotAnswered(channelContext))
			deletePendingRequest(compositeKey);
		/*** New for syncPresentationHandler ***/

		if(Trace.doTrace(HtmlConstants.COMPID, Trace.Low, Trace.Debug))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.Low, Trace.Debug, Settings.getTID(), "Deleted pending request for sessionId/pageId = "+sessionId+" / "+pageId);
		}					
	}
	else
	{
		if (pendingRequestData != null)
		{
			pendingRequestData.operCompleteTime = new Date(System.currentTimeMillis()) ;
			pendingRequestTable.put(compositeKey, pendingRequestData);
		}
	}
}
/**
 * This method removes an operation which has completed for a given page in a
 * given session.
 *
 * @return boolean
 * @param compositeKey java.lang.String
 */
public boolean deletePendingRequest(String compositeKey)
{
	boolean retCode = false;
	
	HtmlRequestData pendingRequestData = (HtmlRequestData)pendingRequestTable.remove(compositeKey);

	if (pendingRequestData != null)
	{
		retCode = true;
	}

	return retCode;
}
/**
 * Builds the composite key used to identify a request in the pending requests table.
 * It is constructed concatenating three keys:
 * <br>
 * <ul>
 *   <li> sessionId </li>
 *   <li> pageId </li>
 *   <li> operation/processor identifier </li>
 * </ul>
 * <br>
 * If the request is intended to execute an operation instead of a processor, the
 * operation identifier is just the operation name.
 * <br>
 * If the request is intended to execute a processor we need more than the processor
 * name because the same page could contain more than one reference to the same processor,
 * each of them asking for different transitions, so the processor name is constructed
 * concatenating processor name and next event name.
 *
 * @return java.lang.String
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public String getCompositeKey(ChannelContext channelContext) throws DSEException
{
	StringBuffer compositeKeyBuffer = new StringBuffer();
	
	// Get the keyed colection obtained from the request
	KeyedCollection requestData = (KeyedCollection) channelContext.getRequestData().getElementAt(CSConstants.DATAKEY);

	// Get the two common fields used to build the key
	String sessionId = (String) requestData.getValueAt(CSConstants.SESSIONIDKEY);
	String callingPageId = (String) requestData.getValueAt(HtmlConstants.PAGEID);

	// Get the third part of the key. Is different depending whether the request is for a processor or for an operation
	String opName = null;

	opName = (String) requestData.getValueAt(HtmlConstants.OPERATIONNAME);

	compositeKeyBuffer.append(sessionId);
	compositeKeyBuffer.append(".");
	compositeKeyBuffer.append(callingPageId);
	compositeKeyBuffer.append(".");
	compositeKeyBuffer.append(opName);
	if ( isProcessorRequest(channelContext) )
	{
		// Means that the request wants to execute a processor instead of an operation
		compositeKeyBuffer.append(".");
		compositeKeyBuffer.append((String) requestData.getValueAt(HtmlConstants.NEXTEVENTNAME));
	}
	

	return compositeKeyBuffer.toString();
}
/**
 * Generate and return the next page ID for a given session.
 *
 * @return java.lang.String
 * @param sessionId java.lang.String
 */
public static String getNextPageId(String sessionId) 
{
	Integer pageId = new Integer(0);
	try {
		
	// Get the session context
	Context ctx = Context.getCurrentContextForSessionOrApplication(sessionId);
	
	//------
	// Try to get the next pageId for this session.
	//------
	try {
	pageId = (Integer) ctx.getValueAt(HtmlConstants.LASTPAGEIDPROCESSED);
	}
	catch (Exception e) {
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Information))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Information, Settings.getTID(), "Last page id not found in context: "+ctx.getName()+". Adding and setting a new field...") ;
		}		
		addFieldTo(ctx, HtmlConstants.LASTPAGEIDPROCESSED, pageId);
	}
	
	//------
	// Update the value for this session (increment it).
	//------
	int nextPageId = pageId.intValue();
	
	pageId = new Integer(++nextPageId);
	//------
	// Enter the new or updated entry for this session into the table.
	//------
	ctx.setValueAt(HtmlConstants.LASTPAGEIDPROCESSED,pageId);
	

	}
	catch ( Exception e2) {
		if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
		{
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught while getting the next page id for session: "+sessionId) ;
			Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e2) ;			
		}				
	}
	return pageId.toString();
}
/**
 * Return the operation in process that corresponds to the session id and page id specified in the channelContext
 * 
 * @param com.ibm.dse.clientserver.ChannelContext
 */
public DSEServerOperation getOperationInProcess(ChannelContext channelContext) throws DSEException
{
	DSEServerOperation retOper = null;

	String compositeKey = getCompositeKey(channelContext);

	HtmlRequestData pendingRequestData = (HtmlRequestData)pendingRequestTable.get(compositeKey);

	if (pendingRequestData != null)
	{
		retOper = pendingRequestData.operation;
	}

	return retOper;
}
/**
 * checks whether this request is product of a double click.
 * @param channelContext
 * @exception DSEException
 */
public boolean isMarkedAsNotAnswered(ChannelContext channelContext) throws DSEException
{
	/*** Whole method new for syncPresentationHandler ***/
	String compositeKey = getCompositeKey(channelContext);
	HtmlRequestData request = (HtmlRequestData) pendingRequestTable.get(compositeKey);
	if (request==null) return false;
	else return request.answered;
}
/**
 * See if the request asks or a processor instead of an operation.
 * It looks for a valid event name in the request data received
 * from client. If this field does not exist or it is empty, this
 * request won't be considered as an automaton request.
 * 
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public boolean isProcessorRequest(ChannelContext channelContext) throws DSEException
{
	// Get the keyed colection obtained from the request
	KeyedCollection requestData = (KeyedCollection) channelContext.getRequestData().getElementAt(CSConstants.DATAKEY);

	String eventName;
	try
	{
		// Try to get the event name field
		eventName= (String) requestData.getValueAt(HtmlConstants.NEXTEVENTNAME);
	}
	catch (DSEObjectNotFoundException e)
	{
		// If there is no event name defined, it is not a processor request
		return false;
	}

	// If event name exist, but its value is empty, return false
	if ( eventName.equals("") ) return false;

	return true;
}
/**
 * See if a request recently completed for a given page
 * in a given session.
 * 
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public boolean isRequestAging(ChannelContext channelContext) throws DSEException
{
	boolean retCode = false;

	//------
	// Every time a request is processed we call a method to
	// remove requests from the table which have been there past their
	// aging limit.
	//------
	removeExpiredRequests();
	
	String compositeKey = getCompositeKey(channelContext);
	HtmlRequestData pendingRequestData = (HtmlRequestData) pendingRequestTable.get(compositeKey);
	if ((pendingRequestData != null) && (pendingRequestData.operCompleteTime != null))
	{
		if (pendingRequestData.operCompleteTime.after(new Date(System.currentTimeMillis() - ageLimit)))
		{
			retCode = true;
		}
	}
	return retCode;
}
/**
 * Utility method inditating if there is any operation in process for a given session id and page id.
 * @param sessionId
 * @param pageId 
 */
public boolean isSomeOpProcessing(String sessionId, String pageId) throws DSEException
{
	// The composite key for a processor is sessionId.pageId.processorName.eventName but we
	// don't want the eventName
	String compositeKey = sessionId+"."+pageId;
	String key;

	for (Enumeration enum = pendingRequestTable.keys(); enum.hasMoreElements(); )
	{
		key = ((String) enum.nextElement());
		int firstPoint = key.indexOf(".");
		int secondPoint = key.indexOf(".",firstPoint+1);
		
		if ( compositeKey.equals(key.substring(0, secondPoint)) )
		{
			return true;
		}
	}

	return false;
}
/**
 * See if a request for this processor is still in process or its
 * aging timeout has not been reached yet.
 * 
 * @param channelContext com.ibm.dse.clientserver.ChannelContext
 */
public boolean isUnconsistentProcessor(ChannelContext channelContext) throws DSEException
{
	// The composite key for a processor is sessionId.pageId.processorName.eventName but we
	// don't want the eventName
	String compositeKey = getCompositeKey(channelContext);
	int lastDot = compositeKey.lastIndexOf(".");
	String processorKey = compositeKey.substring(0, lastDot);
	String key, subKey;

	for (Enumeration enum = pendingRequestTable.keys(); enum.hasMoreElements(); )
	{
		key = ((String) enum.nextElement());

		if ( key.startsWith(processorKey))
		{
			subKey = key.substring(0, key.lastIndexOf("."));
			if (subKey.equals(key))
			{
				return true;
			}
			/*** New for syncPresentationHandler ***/
			//HtmlRequestData data = (HtmlRequestData) pendingRequestTable.get(key);
			//if (data.operCompleteTime == null) return true; // Means that this request is still in process
			//if (data.operCompleteTime.after(new Date(System.currentTimeMillis() - ageLimit))) return true;
			/*** New for syncPresentationHandler ***/
		}
	}

	return false;
}
/**
 * Marks the request as not answered, which means that there is another request from the same client
 * which will handle the answering.
 * @param channelContext
 * @exception DSEException
 */
public void markAsNotAnswered(ChannelContext channelContext) throws DSEException
{
	String compositeKey = getCompositeKey(channelContext);
	HtmlRequestData request = (HtmlRequestData) pendingRequestTable.get(compositeKey);
	if (request != null){
	request.answered = true;
}
}
/**
 * Marks the request as answered, which means that there is no other request from the same client.
 * @param channelContext
 * @exception DSEException
 */
public void markAsAnswered(ChannelContext channelContext) throws DSEException
{
	String compositeKey = getCompositeKey(channelContext);
	HtmlRequestData request = (HtmlRequestData) pendingRequestTable.get(compositeKey);
	if (request != null){
	request.answered = false;
}
}
/**
 * Remove requests that are in the pendingRequestTable which have
 * been there longer than the max aging time set for the system.
 */
protected void removeExpiredRequests()
{
	try
	{
	// Clear table of requests that are past the aging limit.	
	Enumeration enum = pendingRequestTable.keys();

	while (enum.hasMoreElements())
	{
		String thisKey = (String)enum.nextElement();
		
		HtmlRequestData next = (HtmlRequestData)pendingRequestTable.get(thisKey);
		
		if (next.operCompleteTime != null && next.operCompleteTime.before(new Date(System.currentTimeMillis()-ageLimit)))
		{
			try
			{
				next.operation.close();
			}
			catch(Exception e){
				if(Trace.doTrace(HtmlConstants.COMPID, Trace.High, Trace.Error))
				{
					Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), "Exception caught when closing the expired operation from the pending requests table: "+next+"operation: "+next.operation) ;
					Trace.trace(HtmlConstants.COMPID, Trace.High, Trace.Error, Settings.getTID(), e) ;			
				}						
			}
			
			pendingRequestTable.remove(thisKey);
		}
		
	}
	}catch(Exception e)
	{
	}
	
}
/**
 * Utility method that returns the pending requests table.
 * @exception Exception
 */
public java.util.Hashtable getPendingRequestTable() throws Exception
{
	return pendingRequestTable;
}
}
